using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.datamodel.spd;
using jvms.xmlgeneration;


namespace jvms.datamodel.dcd
{
	[Serializable]
	/// <summary>
	/// "The componentplacement element (see Figure D-34) is used to define a particular deployment of a component. The componentfileref element identifies the component to be deployed. The componentinstantiation element identifies the actual component created and its id attribute is a DCE UUID value with the format as specified in section D.2.1. Multiple components of the same kind can be created within the same componentplacement element. The optional deployondevice element indicates the device on which the componentinstantiation element is deployed. The optional compositepartofdevice element indicates the device that the componentinstantiation element is aggregated with to form an aggregate relationship. When the component is a logical Device, the devicepkgfile element indicates the hardware device information for the logical Device."
	/// </summary>
	public class ComponentPlacement : XMLObject
	{
		//we will only support one instantiation
		private ComponentInstantiation instantiation = new ComponentInstantiation();
		//private TypedArrayList instantiations = new TypedArrayList(typeof(ComponentInstantiation));
		/// <summary>
		/// BJL - I don't believe that we are actually supporting this.
		/// </summary>
		private Object devicePackageFile = null;
		/// <summary>
		/// "The compositepartofdevice element is used when an aggregate relationship exists to reference the componentinstantiation element that describes the whole Device for which this Devices componentinstantiation element describes a part of the aggregate Device."
		/// </summary>
		private ComponentPlacement compositePartOfDevice;
		/// <summary>
		/// "The deployondevice element is used to reference a componentinstantiation element on which this componentinstantiation is deployed."
		/// </summary>
		private ComponentPlacement deployedOnDevice;
		/// <summary>
		/// Component that is being placed.
		/// 
		/// "The componentfileref element is used to reference a componentfile element within the componentfiles element. The componentfileref elements refid attribute corresponds to a componentfile elements id attribute."
		/// </summary>
		private NonSCAComponent componentFile;

		/// <summary>
		/// The list of instantiations.
		/// </summary>
		public ComponentInstantiation Instantiation
		{
			get
			{
				return instantiation;
			}

			set
			{
				instantiation = value;
			}
		}

		/// <summary>
		/// The device package file, not sure if we are supporting this.
		/// </summary>
		public Object DevicePackageFile
		{
			get
			{
				return devicePackageFile;
			}

			set
			{
				devicePackageFile = value;
			}
		}
		
		/// <summary>
		/// This will get the composite part of the device.
		/// </summary>
		public ComponentPlacement CompositePartOfDevice
		{
			get
			{
				return compositePartOfDevice;
			}

			set
			{
				compositePartOfDevice = value;
			}
		}

		/// <summary>
		/// This will get the deployed on device relationship.
		/// </summary>
		public ComponentPlacement DeployedOnDevice
		{
			get
			{
				return deployedOnDevice;
			}

			set
			{
				deployedOnDevice = value;
			}
		}

		/// <summary>
		/// The component for this placement.
		/// </summary>
		public NonSCAComponent ComponentFile
		{
			get
			{
				return componentFile;
			}

			set
			{
				componentFile = value;
			}
		}

		/// <summary>
		/// This will turn this object to an xml element.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "componentplacement" );
			XmlElement componentFileElement = document.CreateElement( "componentfileref" );
			element.AppendChild( componentFileElement );
			componentFileElement.SetAttribute( "refid", componentFile.Name );
			if( deployedOnDevice != null )
			{
				XMLHelper.writeElement( document, element, "deployondevice", deployedOnDevice.Instantiation.ID.DCEUUID );
			}
			if( compositePartOfDevice != null )
			{
				XMLHelper.writeElement( document, element, "compositepartofdevice", compositePartOfDevice.Instantiation.ID.DCEUUID );
			}
			if( devicePackageFile != null )
			{
				//maybe some day we will support the device
			}
			element.AppendChild( instantiation.toXML( document ) );


			return element;
		}

		public override String ToString()
		{
			return instantiation.UsageName;
		}
	}
}
