using System;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;

using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.prf
{
	[Serializable]
	/// <summary>
	/// "The simple element (see Figure D-12) provides for the definition of a property 
	/// which includes a unique id, type, name and mode attributes of the property that 
	/// will be used in the CF Resource configure() and query() operations, for 
	/// indication of component capabilities, or in the CF TestableObject runTest 
	/// operation. The simple element is specifically designed to support idvalue pair 
	/// definitions. A simple property id attribute corresponds to the id of the 
	/// id-value pair.  The value and range of a simple property correspond to the 
	/// value of the id-value pair. If no value is given, then the property cannot be 
	/// used for input test values for testing and/or as an initial configuration or 
	/// execute parameter of a component. The optional enumerations element allows for 
	/// the definition of a label-to-value for a particular property. The mode 
	/// attribute defines whether the properties element is readonly, writeonly or 
	/// readwrite. The id attribute is an identifier for the simple property element. 
	/// The id attribute for a simple property that is an allocation type is a DCE UUID 
	/// value, as specified in section D.2.1. The id attribute for all other simple 
	/// property elements can be any valid XML ID type. The mode attribute is only 
	/// meaningful when the type of the kind element is configure."
	/// 
	/// </summary>
	public class Simple : XMLObject
	{
		private Range range = new Range();
		/// <summary>
		/// An element defining an enumerated type.  I think the CF is responsible for translating this into a CORBA long type prior to passing to the application.
		/// </summary>
		private TypedArrayList enumerations = new TypedArrayList(typeof(Enumeration));
		/// <summary>
		/// The action element is used to define the type of comparison used to 
		/// compare an SPD  property value to a CF Device property value, during the 
		/// process of checking SPD dependencies.  The type attribute, of the action 
		/// element, will determine the type of comparison to be made (e.g., equal, not 
		/// equal, greater than, etc.).
		/// 
		/// In principle, the action element defines the operation executed during the 
		/// comparison of the allocation property value, provided by an SPD dependency 
		/// element, to the associated allocation property value of a CF Device.  The 
		/// allocation property is on the left side of the action and the dependency value 
		/// is on the right side of the action.  This process allows for the allocation of 
		/// appropriate objects within the system based on their attributes, as defined by 
		/// their dependent relationships.
		/// 
		/// For example, if a CF Device's properties file defines a DeviceKind allocation 
		/// property whose action element is set to "equal", then at the time of dependency 
		/// checking a valid DeviceKind property is checked for equality.  If a software 
		/// component implementation is dependent on a DeviceKind property with its value 
		/// set to "NarrowBand", then the component's SPD dependency propertyref element 
		/// will reference the id of the DeviceKind allocation property with a value of 
		/// "NarrowBand".  At the time of dependency checking, the CF ApplicationFactory 
		/// will check CF Devices whose properties kind element is set to "allocation" and 
		/// property id is DeviceKind for equality against a "NarrowBand" value."
		/// 
		/// External means that the ApplicationFactory must go to the device and first 
		/// try to perform allocation.  If the allocation succeeds, then the dependency 
		/// passes.  Otherwise it fails, and deallocation ensues.
		/// 
		/// </summary>
		private String action = "";
		private KindArrayList kinds = new KindArrayList();
		private String description = "";
		private String units = "";
		private String simpleValue = "";
		private String mode = "";
		private String name = "";
		private String type = "";
		/// <summary>
		/// BJL - This is a UUID if this is an allocation property otherwise it just has to be unique.
		/// 
		/// 
		/// </summary>
		private ID id = new ID();

		/// <summary>
		/// Constructor
		/// </summary>
		public Simple()
		{
		}

		/// <summary>
		/// Copy constructor
		/// </summary>
		/// <param name="other"></param>
		public Simple( Simple other )
		{
			range.Max = other.Range.Max;
			range.Min = other.Range.Min;
			
			Enumeration enumeration = null;
			foreach( Enumeration e in other.Enumerations )
			{
				enumeration = new Enumeration();
				enumeration.Label = e.Label;
				enumeration.Value = e.Value;
				enumerations.Add( enumeration );
			}

			action = other.Action;
			kinds = new KindArrayList( other.Kinds.KindsList );
			description = other.Description;
			units = other.Units;
			simpleValue = other.Value;
			mode = other.Mode;
			name = other.Name;
			type = other.Type;
			id = other.ID;
		}

		/// <summary>
		/// The range of values.
		/// </summary>
		public Range Range
		{
			get
			{
				return range;
			}
			set
			{
				range = value;
			}
		}
		/// <summary>
		/// A list of Enumeration objects
		/// </summary>
		public TypedArrayList Enumerations
		{
			get
			{
				return enumerations;
			}
			set
			{
				enumerations = value;
			}
		}

		/// <summary>
		/// The action used for checking.
		/// </summary>
		public String Action
		{
			get
			{
				return action;
			}
			set
			{
				action = value;
			}
		}
		/// <summary>
		/// The kinds of property that this is.
		/// </summary>
		public KindArrayList Kinds
		{
			get
			{
				return kinds;
			}
			set
			{
				kinds = value;
			}
		}
		/// <summary>
		/// The units of this property.
		/// </summary>
		public String Units
		{
			get
			{
				return units;
			}
			set
			{
				units = value;
			}
		}
		/// <summary>
		/// The value of this property.
		/// </summary>
		public String Value
		{
			get
			{
				return simpleValue;
			}
			set
			{
				simpleValue = value;
			}
		}
		/// <summary>
		/// The mode of this property.
		/// </summary>
		public String Mode
		{
			get
			{
				return mode;
			}
			set
			{
				mode = value;
			}
		}
		/// <summary>
		/// The name of this property.
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}
		/// <summary>
		/// The type of this property.
		/// </summary>
		public String Type
		{
			get
			{
				return type;
			}
			set
			{
				type = value;
			}
		}
		/// <summary>
		/// The id of this property.
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}

		/// <summary>
		/// The description element.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}

		/// <summary>
		/// This will return a string representation of the Author class.
		/// </summary>
		/// <returns>This author represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Simple>";
			if( !Name.Equals( "" ) )
			{
				retval = Name;
			}
			return retval;
		}

		/// <summary>
		/// This will write this as an XML element.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "simple" );
			XMLHelper.writeAttribute( element, "id", id.DCEUUID );
			XMLHelper.writeAttribute( element, "type", type );
			XMLHelper.writeAttribute( element, "name", name );
			XMLHelper.writeAttribute( element, "mode", mode );

			XMLHelper.writeElement( document, element, "description", description );
			XMLHelper.writeElement( document, element, "value", simpleValue );
			XMLHelper.writeElement( document, element, "units", units );
			if( !range.Min.Equals( "" ) && !range.Max.Equals( "" ) )
			{
				element.AppendChild( range.toXML( document ) );
			}
			if( enumerations.Count > 0 )
			{
				XmlElement enumsElement = document.CreateElement( "enumerations" );
				element.AppendChild( enumsElement );
				XMLHelper.writeListOfXMLElements( document, enumsElement, enumerations );
			}
			foreach( String kind in kinds.KindsList )
			{
				XmlElement kindElement = document.CreateElement( "kind" );
				element.AppendChild( kindElement );
				kindElement.SetAttribute( "kindtype", kind );
			}
			if( !action.Equals( "" ) )
			{
				XmlElement actionElement = document.CreateElement( "action" );
				actionElement.SetAttribute( "type", action );
				element.AppendChild( actionElement );
			}
			
			return element;
		}
	}

	/// <summary>
	/// Represents a list that holds property kinds.  Needed in order to display
	/// kind type in PropertyGrid.
	/// </summary>
	[Serializable]
	public class KindArrayList
	{
		private ArrayList kindsList = new ArrayList();

		/// <summary>Sets or gets the object at the given index.</summary>
		/// <value>An object in the list.</value>
		public Object this [int index]
		{
			get
			{
				return kindsList[index];
			}

			set
			{
				kindsList[index] = value;
			}
		}

		/// <summary>Gets the number of elements in the list.</summary>
		/// <value>An integer.</value>
		public int Count
		{
			get
			{
				return kindsList.Count;
			}
		}

		/// <summary>Returns the underlying list object.</summary>
		/// <value>An IList.</value>
		public IList KindsList
		{
			get
			{
				return kindsList;
			}

			set
			{
				kindsList = new ArrayList( value );
			}
		}

		/// <summary>
		/// Creates a new KindArrayList.
		/// </summary>
		public KindArrayList()
		{
		}

		/// <summary>
		/// Creates a new KindArrayList using the given list.
		/// </summary>
		/// <param name="kindsList">Another list.</param>
		public KindArrayList( IList kindsList )
		{
			this.kindsList = new ArrayList( kindsList );
		}

		/// <summary>
		/// Adds the specified item to the end of the list.
		/// </summary>
		/// <param name="o">The item to add.</param>
		public void Add( Object o )
		{
			kindsList.Add( o );
		}

		/// <summary>
		/// Clears the list.
		/// </summary>
		public void Clear()
		{
			kindsList.Clear();
		}

		/// <summary>
		/// Returns true if the specified item is in the list, false otherwise.
		/// </summary>
		/// <param name="o">The item to test.</param>
		/// <returns>True if the item is in the list, false otherwise.</returns>
		public bool Contains( Object o )
		{
			return kindsList.Contains( o );
		}

		/// <summary>
		/// Returns a string representation of the object.
		/// </summary>
		/// <returns>A String representation of the object.</returns>
		public override String ToString()
		{
			String retVal = "";

			if ( kindsList.Count == 2 && ( this[0].Equals( "configuration" ) && this[1].Equals( "allocation" ) ) )
			{
				retVal = "configuration/allocation";
			}
			else
			{
				retVal = this[0].ToString();
			}

			return retVal;
		}
	}
}
