using System;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;

using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.prf
{
	[Serializable]
	/// <summary>
	/// simplesequence is exactly the same as simple (with the same elements and 
	/// cardinality as simple, except that a sequence of (similar type) values is included.  
	/// I am also skeptical if the kind type can be of type allocation, test, execparam - 
	/// I doubt it.
	/// 
	/// SCA text:
	/// "The simplesequence element (see Figure D-13) is used to specify a list of 
	/// properties with the same characteristics (e.g., type, range, units, etc.).  
	/// The simplesequence element definition is similar to the simple element definition 
	/// except that it has a list of values instead of one value.  The simplesequence 
	/// element maps to the sequence types for CF and PortTypes CORBA modules, defined in 
	/// SCA Appendix C section C.2, based upon the type attribute."
	///  
	/// 
	/// </summary>
	public class SimpleSequence : XMLObject
	{
		private String action = "";
		private KindArrayList kinds = new KindArrayList();
		private Range range = new Range();
		private String units = "";
		private IList values  = new ArrayList();
		private String description = "";
		private String mode = "";
		private String name = "";
		private String type = "";
		private ID id = new ID();

		public SimpleSequence()
		{
		}

		public SimpleSequence( SimpleSequence other )
		{
			range.Max = other.Range.Max;
			range.Min = other.Range.Min;
			foreach( object obj in other.Values )
			{
				values.Add( obj );
			}

			action = other.Action;
			kinds = new KindArrayList( other.kinds.KindsList );
			description = other.Description;
			units = other.Units;
			mode = other.Mode;
			name = other.Name;
			type = other.Type;
			id = other.ID;
		}

		/// <summary>
		/// The action to be performed when checking this property.
		/// </summary>
		public String Action
		{
			get
			{
				return action;
			}
			set
			{
				action = value;
			}
		}
		/// <summary>
		/// The list of kinds of properties that this represents.
		/// </summary>
		public KindArrayList Kinds
		{
			get
			{
				return kinds;
			}
			set
			{
				kinds = value;
			}
		}
		/// <summary>
		/// The allowed range for this property.
		/// </summary>
		public Range Range
		{
			get
			{
				return range;
			}
			set
			{
				range = value;
			}
		}
		/// <summary>
		/// The units for this property.
		/// </summary>
		public String Units
		{
			get
			{
				return units;
			}
			set
			{
				units = value;
			}
		}
		/// <summary>
		/// The list of values for this property.
		/// </summary>
		public IList Values
		{
			get
			{
				return values;
			}
			set
			{
				values = value;
			}
		}
		/// <summary>
		/// The description of this property.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}
		/// <summary>
		/// The mode of this property.
		/// </summary>
		public String Mode
		{
			get
			{
				return mode;
			}
			set
			{
				mode = value;
			}
		}
		/// <summary>
		/// The name of this property.
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}
		/// <summary>
		/// The type of this property.
		/// </summary>
		public String Type
		{
			get
			{
				return type;
			}
			set
			{
				type = value;
			}
		}
		/// <summary>
		/// The id of this property.
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}
		/// <summary>
		/// This will return a string representation of the Author class.
		/// </summary>
		/// <returns>This author represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Simple Sequence>";
			if( !Name.Equals( "" ) )
			{
				retval = Name;
			}
			return retval;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "simplesequence" );

			XMLHelper.writeAttribute( element, "id", id.DCEUUID );
			XMLHelper.writeAttribute( element, "type", type );
			XMLHelper.writeAttribute( element, "name", name );
			XMLHelper.writeAttribute( element, "mode", mode );

			XMLHelper.writeElement( document, element, "description", description );
			if( values.Count > 0 )
			{
				XmlElement valuesElement = document.CreateElement( "values" );
				element.AppendChild( valuesElement );
				XMLHelper.writeListOfStringElements( document, valuesElement, "value", values );
			}
			XMLHelper.writeElement( document, element, "units", units );
			if( !range.Min.Equals( "" ) && !range.Max.Equals( "" ) )
			{
				element.AppendChild( range.toXML( document ) );
			}
			foreach( String kind in kinds.KindsList )
			{
				XmlElement kindElement = document.CreateElement( "kind" );
				element.AppendChild( kindElement );
				kindElement.SetAttribute( "kindtype", kind );
			}
			if( !action.Equals( "" ) )
			{
				XmlElement actionElement = document.CreateElement( "action" );
				actionElement.SetAttribute( "type", action );
				element.AppendChild( actionElement );
			}

			return element;
		}
	}
}
