using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.prf
{
	[Serializable]
	/// <summary>
	/// "The struct element (see Figure D-15) is used to group properties with different characteristics (i.e., similar to a structure or record entry). Each item in the struct element can be a different simple type (e.g., short, long, etc.). The struct element corresponds to the CF Properties type where each struct item (ID, value) corresponds to a properties element list item. The properties element list size is based on the number of struct items."
	/// </summary>
	public class Struct : XMLObject
	{
		private TypedArrayList simples = new TypedArrayList(typeof(Simple));
		/// <summary>
		/// configurekind is limited to configure and factoryParam types.
		/// 
		/// I am assuming that all of the elements of a struct by extension 
		/// are also limited to this same type.
		/// </summary>
		private String configurationKind = "";		
		private String mode = "";
		private String name = "";
		private ID id = new ID();
		private String description = "";

		public Struct()
		{
		}

		public Struct( Struct other )
		{
			foreach( Simple simple in other.Simples )
			{
				simples.Add( new Simple(simple) );
			}
			configurationKind = other.ConfigurationKind;
			mode = other.Mode;
			name = other.Name;
			id = other.ID;
			description = other.Description;
		}

		/// <summary>
		/// A description of this element.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}

		/// <summary>
		/// The list of simple objects.
		/// </summary>
		public TypedArrayList Simples
		{
			get
			{
				return simples;
			}
			set
			{
				simples = value;
			}
		}
		/// <summary>
		/// The comfiguration kind of the property.
		/// </summary>
		public String ConfigurationKind
		{
			get
			{
				return configurationKind;
			}
			set
			{
				configurationKind = value;
			}
		}
		/// <summary>
		/// The mode of the property.
		/// </summary>
		public String Mode
		{
			get
			{
				return mode;
			}
			set
			{
				mode = value;
			}
		}
		/// <summary>
		/// The name of the property.
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}
		/// <summary>
		/// The id of the property.
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}

		/// <summary>
		/// This will return a string representation of the Author class.
		/// </summary>
		/// <returns>This author represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Struct>";
			if( !Name.Equals( "" ) )
			{
				retval = Name;
			}
			return retval;
		}

		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "struct" );
			XMLHelper.writeAttribute( element, "id", id.DCEUUID );
			XMLHelper.writeAttribute( element, "name", name );
			XMLHelper.writeAttribute( element, "mode", mode );

			XMLHelper.writeElement( document, element, "description", description );
			XMLHelper.writeListOfXMLElements( document, element, simples );
			if( !configurationKind.Equals( "" ) )
			{
				XmlElement ckElement = document.CreateElement( "configurationkind" );
				element.AppendChild( ckElement );
				ckElement.SetAttribute( "kindtype", configurationKind );
			}

			return element;
		}
	}
}
