using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.datamodel.scd;
using jvms.datamodel.spd;

using jvms.xmlgeneration;

namespace jvms.datamodel.sad
{
	[Serializable]
	/// <summary>
	/// The intent of the software assembly is to provide the means of describing the 
	/// assembled functional application and the interconnection characteristics of the 
	/// SCA components within that application.  The component assembly provides four 
	/// basic types of application information for Domain Management.  The first is 
	/// partitioning information that indicates special requirements for collocation of 
	/// components, the second is the assembly controller for the software assembly, 
	/// the third is connection information for the various components that make up the 
	/// application assembly, and the fourth is the visible ports for the application 
	/// assembly.  
	/// The installation of an application into the system involves the installation of 
	/// a SAD file. The SAD file references component's SPD files to obtain deployment 
	/// information for these components.The softwareassembly element's id attribute is 
	/// a DCE UUID, as specified in section D.2.1, which uniquely identifies the 
	/// assembly. The softwareassembly element's name attribute is the user-friendly 
	/// name for the CF ApplicationFactory name attribute.
	/// 
	/// </summary>
	public class Application : FileSystemObject
	{
		private TypedArrayList externalPorts = new TypedArrayList(typeof(Port));
		private TypedArrayList connections = new TypedArrayList(typeof(ConnectInterface));
		private ApplicationComponentInstantiation assemblyController;
		/// <summary>
		/// BJL - A list of ComponentInstantiation objects.  I don't think this will
		/// be populated via the property sheet.
		/// </summary>
		private TypedArrayList componentPlacements = new TypedArrayList(typeof(ApplicationComponentInstantiation));
		/// <summary>
		/// BJL - A list of host colocation objects.  I don't think this will
		/// be populated via the property sheet.  We are not supporting this
		/// at this time.
		/// </summary>
		private IList hostCollocation = new ArrayList();
		private String description = "";
		/// <summary>
		/// The softwareassembly element's name attribute is the user-friendly name for the CF ApplicationFactory name attribute.
		/// 
		/// Put another way, the created ApplicationFactory and its associated applications should be named after this name.
		/// </summary>
		private String name = "";
		/// <summary>
		/// The ID (a UUID) for the assembly.  This has to be unique, but I am not sure if it is referenced by anything.
		/// </summary>
		private ID id = new ID();

		/// <summary>
		/// All external ports for this application.
		/// </summary>
		public TypedArrayList ExternalPorts
		{
			get
			{
				return externalPorts;
			}
			set
			{
				externalPorts = value;
			}
		}
		/// <summary>
		/// The list of all the connections.
		/// </summary>
		public TypedArrayList Connections
		{
			get
			{
				return connections;
			}
			set
			{
				connections = value;
			}
		}

		/// <summary>
		/// The assembly controller for this application.
		/// </summary>
		public ApplicationComponentInstantiation AssemblyController
		{
			get
			{
				return assemblyController;
			}
			set
			{
				assemblyController = value;
			}
		}
		/// <summary>
		/// The partitioning of the application
		/// </summary>
		public TypedArrayList ComponentPlacements
		{
			get
			{
				return componentPlacements;
			}
			set
			{
				componentPlacements = value;
			}
		}
		/// <summary>
		/// The partitioning of the application
		/// </summary>
		public IList HostCollocation
		{
			get
			{
				return hostCollocation;
			}
			set
			{
				hostCollocation = value;
			}
		}

		/// <summary>
		/// A description of the application.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}

		/// <summary>
		/// The name of the application.
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}
		/// <summary>
		/// The id of the application.
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}

		/// <summary>
		/// This will generate the list of component files that are necessary.
		/// </summary>
		/// <returns>A list of NonSCAComponent objects.</returns>
		public ICollection getComponentFiles()
		{
			Hashtable componentFiles = new Hashtable();
			
			foreach( ApplicationComponentInstantiation instance in componentPlacements )
			{
				componentFiles[ instance.Component ] = instance.Component;
			}
			return componentFiles.Keys;
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public override XmlElement toXML( XmlDocument document )
		{
			document.XmlResolver = new JVMSXMLResolver();

			XmlDocumentType type = document.CreateDocumentType( "softwareassembly", "", "softwareassembly.2.2.dtd", null );
			XmlDeclaration decl = document.CreateXmlDeclaration( "1.0", "ISO-8859-1", "no" );
			document.AppendChild( decl );
			document.AppendChild( type );

			XmlElement element = document.CreateElement( "softwareassembly" );
			element.SetAttribute( "id", id.DCEUUID );
			element.SetAttribute( "name", name );
			XMLHelper.writeElement( document, element, "description", description );

			//ok now for the hard part we need to write out the list of component files that
			//we are using, this list will be determined via the instantiations in the 
			//partitioning.  We are using a hashtable because there is no Set, not sure why?
			ICollection componentFiles = getComponentFiles();

			XmlElement filesElement = document.CreateElement( "componentfiles" );
			element.AppendChild( filesElement );
			foreach( NonSCAComponent component in componentFiles)
			{
				XmlElement fileElement = 
					XMLHelper.writeFileReference( document, filesElement, "componentfile", this, component );					
				fileElement.SetAttribute( "id", component.Name );
			}
		
		
			XmlElement partitioningElement = document.CreateElement( "partitioning" );
			element.AppendChild( partitioningElement );
			XMLHelper.writeListOfXMLElements( document, partitioningElement, componentPlacements );
			
			XmlElement assemblyElement = document.CreateElement( "assemblycontroller" );
			element.AppendChild( assemblyElement );
			XmlElement instanceRef = document.CreateElement( "componentinstantiationref" );
			assemblyElement.AppendChild( instanceRef );
			if( assemblyController != null )
			{
				instanceRef.SetAttribute( "refid", assemblyController.ID.DCEUUID );
				XmlComment aControllerComment = document.CreateComment( assemblyController.UsageName );
				assemblyElement.AppendChild( aControllerComment );
			}

			if( connections.Count > 0 )
			{
				XmlElement connectionElement = document.CreateElement( "connections" );
				element.AppendChild( connectionElement );
				XMLHelper.writeListOfXMLElements( document, connectionElement, connections );
			}

			XmlElement externalPortsElement = document.CreateElement( "externalports" );
			foreach( ApplicationComponentInstantiation instance in componentPlacements )
			{
				foreach( ComponentPortEndPoint port in instance.ExternalPorts )
				{
					XmlElement portElement = document.CreateElement( "port" );
					externalPortsElement.AppendChild( portElement );
					if( instance.Component is SCAComponent )
					{
						SCAComponent component = (SCAComponent)instance.Component;
						if( component.Descriptor.Features.UsesPorts.Contains( port ) )
						{
							XMLHelper.writeElement( document, portElement, "usesidentifier", port.RepositoryID.RepositoryID );
						}
						else if( component.Descriptor.Features.ProvidesPorts.Contains( port ) )
						{
							XMLHelper.writeElement( document, portElement, "providesidentifier", port.RepositoryID.RepositoryID );
						}
						else if( component.Descriptor.Features.SupportInterfaces.Contains( port.RepositoryID ) )
						{
							XMLHelper.writeElement( document, portElement, "supportsidentifier", port.RepositoryID.RepositoryID );
						}
						else
						{
							throw new Exception( "External port without identifier." );
						}
					}
					XmlElement instanceRefElement = document.CreateElement( "componentinstantiationref" );
					portElement.AppendChild( instanceRefElement );
					instanceRefElement.SetAttribute( "refid", instance.ID.DCEUUID );
					XmlComment instanceComment = document.CreateComment( instance.UsageName );
					portElement.AppendChild( instanceComment );
				}
			}
			if( externalPortsElement.ChildNodes.Count > 0 )
			{
				element.AppendChild( externalPortsElement );
			}
			

			return element;
		}

		/// <summary>
		/// This will get the relative name of the file to write to.  This
		/// string shall begin with a \
		/// </summary>
		/// <returns>The path to write this file to.</returns>
		public override String getFilename()
		{
			return "\\" + Name + ".sad.xml";
		}

		public override String ToString()
		{
			return Name;
		}

	}
}
