using System;
using System.Collections;
using System.Xml;

using jvms.xmlgeneration;

namespace jvms.datamodel.scd
{
	[Serializable]
	/// <summary>
	/// This element wraps all information about how this component communicates with the outer world.  Such communication is done via ports or the components base interface.
	/// 
	/// This is an XML "outer structure" which does nothing but house additional elements.  As such it has no IDs.
	/// 
	/// "The componentfeatures element (see Figure D-18) is used to describe a component with respect to the components that it inherits from, the interfaces the component supports, and its provides and uses ports. At a minimum, the component interface has to be a CF Resource, CF ResourceFactory, or CF Device interface. If a component extends the CF Resource or CF Device interface then all the inherited interfaces (e.g., CF Resource) are depicted as supportsinterface elements."
	/// </summary>
	public class ComponentFeatures : XMLObject
	{
		private TypedArrayList supportInterfaces = new TypedArrayList(typeof(Interface));
		/// <summary>
		/// "The ports element (see Figure D-19) describes what interfaces 
		/// a component provides and uses.  The provides elements are 
		/// interfaces that are not part of a components interface 
		/// but are independent interfaces known as facets (in CORBA 
		/// Components terminology) (i.e. a provides port at the end 
		/// of a path, like I/O Device or Modem Device, does not need 
		/// to be a CF Port type).  The uses element is a CF Port 
		/// interface type that is connected to a provides or 
		/// supportinterfaces interface. Any number of uses and 
		/// provides elements can be given in any order. Each ports 
		/// element has a name and references an interface by repid 
		/// (see interfaces section D.5.1.5). The port names are used 
		/// in the Software Assembly Descriptor to connect ports together. 
		/// A ports element also has an optional porttype element that 
		/// allows for identification of port classification.  Values 
		/// for porttype include data, control, responses, and 
		/// test. If a porttype is not given then control is assumed."
		/// 
		/// The ports element contains a list of all formal provides and uses ports.
		/// </summary>
		private TypedArrayList providesPorts = new TypedArrayList(typeof(ComponentPortEndPoint));
		private TypedArrayList usesPorts = new TypedArrayList(typeof(ComponentPortEndPoint));

		public TypedArrayList SupportInterfaces
		{
			get
			{
				return supportInterfaces;
			}
			set
			{
				supportInterfaces = value;
			}
		}
		/// <summary>
		/// This is a list of ComponentPortEndPoint objects.
		/// </summary>
		public TypedArrayList ProvidesPorts
		{
			get
			{
				return providesPorts;
			}
			set
			{
				providesPorts = value;
			}
		}
		/// <summary>
		/// This is a list of ComponentPortEndPoint objects.
		/// </summary>
		public TypedArrayList UsesPorts
		{
			get
			{
				return usesPorts;
			}
			set
			{
				usesPorts = value;
			}
		}

		/// <summary>
		/// This will convert this object to XML.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "componentfeatures" );

			foreach( Interface next in supportInterfaces )
			{
				XmlElement supportsElement = document.CreateElement( "supportsinterface" );
				supportsElement.SetAttribute( "repid", next.RepositoryID );
				supportsElement.SetAttribute( "supportsname", next.Name );
				element.AppendChild( supportsElement );
			}
			XmlElement portsElement = document.CreateElement( "ports" );
			element.AppendChild( portsElement );

			foreach( ComponentPortEndPoint endPoint in providesPorts )
			{
				XmlElement portElement = document.CreateElement( "provides" );
				portsElement.AppendChild( portElement );
				if( endPoint.RepositoryID != null )
				{
					portElement.SetAttribute( "repid", endPoint.RepositoryID.RepositoryID );
				}
				portElement.SetAttribute( "providesname", endPoint.UsesName );
				foreach( String type in endPoint.PortTypes )
				{
					XmlElement typeElement = document.CreateElement( "porttype" );
					typeElement.SetAttribute( "type", type );
					portElement.AppendChild( typeElement );
				}
			}

			foreach( ComponentPortEndPoint endPoint in usesPorts )
			{
				XmlElement portElement = document.CreateElement( "uses" );
				portsElement.AppendChild( portElement );
				if( endPoint.RepositoryID != null )
				{
					portElement.SetAttribute( "repid", endPoint.RepositoryID.RepositoryID );
				}
				portElement.SetAttribute( "usesname", endPoint.UsesName );
				foreach( String type in endPoint.PortTypes )
				{
					XmlElement typeElement = document.CreateElement( "porttype" );
					typeElement.SetAttribute( "type", type );
					portElement.AppendChild( typeElement );
				}
			}

			return element;
		}
	}
}
