using System;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;

using jvms.xmlgeneration;

namespace jvms.datamodel.scd
{
	[Serializable]
	/// <summary>
	/// "The interface element describes an interface that 
	/// the component, either directly or through inheritance, 
	/// provides, uses, or supports.  The name attribute is the 
	/// character-based non-qualified name of the interface.  The 
	/// repid attribute is the unique repository id of the interface, 
	/// which has formats specified in the CORBA specification.  The 
	/// repid is also used to reference an interface element elsewhere 
	/// in the SCD, for example from the inheritsinterface element."
	/// </summary>
	public class Interface : XMLObject
	{
		/// <summary>
		/// BJL - hmm, what should go in here?  Do we need a separate class to hold
		/// a collection of repIds?
		/// </summary>
		private TypedArrayList inherits = new TypedArrayList( typeof(Interface) );
		/// <summary>
		/// The informal name of the interface - don't know where it gets used.  
		/// Note that the name of an interface is different from the name of a port.  
		/// </summary>
		private String name = "";
		/// <summary>
		/// "The repid attribute is the unique repository id of the 
		/// interface, which has formats specified in the CORBA specification.  
		/// The repid is also used to reference an interface element elsewhere 
		/// in the SCD, for example from the inheritsinterface element."
		/// 
		/// Fundamentally, I think repid is meant to correspond to a unique 
		/// chunk of IDL.  Presumably IDL also has a repository ID and this 
		/// is meant to match it.
		/// </summary>
		private String repID = "";

		/// <summary>
		/// A list of inheritance.
		/// </summary>
		public TypedArrayList Inherits
		{
			get
			{
				return inherits;
			}
			set
			{
				inherits = value;
			}
		}
		/// <summary>
		/// The name of the interface.
		/// </summary>
		public String Name
		{
			get
			{
				return name;
			}
			set
			{
				name = value;
			}
		}
		/// <summary>
		/// The repository id.
		/// </summary>
		public String RepositoryID
		{
			get
			{
				return repID;
			}
			set
			{
				repID = value;
			}
		}

		public override bool Equals( Object obj )
		{
			bool retval = false;
			if( obj != null && obj.GetType().Equals( this.GetType() ) )
			{
				Interface other = (Interface)obj;
				retval = this.Name.Equals( other.Name ) &&
					this.RepositoryID.Equals( other.RepositoryID ) &&
					this.Inherits.Equals( other.Inherits );
			}
			return retval;
		}

		public override int GetHashCode()
		{
			return Name.GetHashCode() + RepositoryID.GetHashCode();
		}

		/// <summary>
		/// Returns a String representation of this object.
		/// </summary>
		/// <returns>A String representation.</returns>
		public override String ToString()
		{
			String retVal = "<New Interface>";
			if ( !name.Equals( "" ) )
			{
				retVal = name;
			}
			return retVal;
		}

		/// <summary>
		/// This will convert this object to XML.
		/// </summary>
		/// <param name="document"></param>
		/// <returns></returns>
		public XmlElement toXML( XmlDocument document )
		{
			XmlElement element = document.CreateElement( "interface" );
			element.SetAttribute( "repid", repID );
			element.SetAttribute( "name", name );
			foreach( Interface inheritsValue in inherits )
			{
				XmlElement inheritsElement = document.CreateElement( "inheritsinterface" );
				inheritsElement.SetAttribute( "repid", inheritsValue.RepositoryID );
				element.AppendChild( inheritsElement );
			}
			return element;
		}
	}
}
