using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.prf;
using jvms.datamodel.spd;

using jvms.xmlgeneration;

namespace jvms.datamodel.scd
{
	[Serializable]
	/// <summary>
	/// Each SCA component has an .scd file, and hence the information stored in this class.  This class functions as a specification for a CORBA capable SCA component, including a definition of the type, interfaces supported, and ports either provided or used.
	/// </summary>
	public class SoftwareComponent : FileSystemObject
	{
		private SCAComponent parentComponent = null;

		/// <summary>
		/// We are not supporting the properties at this level just
		/// at the SPD level.  So this attribute will never be used.
		/// </summary>
		private Properties properties = null;

		private ComponentFeatures features = new ComponentFeatures();
		/// <summary>
		/// This is the standardized type of component.  It is a string instead 
		/// of an enumeration, but values are given in D5.1.3 (resource, device, 
		/// resourcefactory, etc).  It is interesting to note that log, filesystem, 
		/// filemanager, etc are also listed, although these do not have required 
		/// SPDs as I recall.
		/// </summary>
		private String componentType = "";
		/// <summary>
		/// "The componentrepid uniquely identifies the interface that 
		/// the component is implementing.  The componentrepid may be referred 
		/// to by the componentfeatures element.  The componentrepid is derived 
		/// from the CF Resource, CF Device, or CF ResourceFactory".
		/// 
		/// This is described elsewhere as the "fully qualified repository 
		/// id of the component".  The format for this string is specified
		///  somewhere in CORBA.  In the example XML this always corresponded 
		///  to the "base" SCA interface this component inherits from, 
		///  i.e. Resource, LoadableDevice, etc.  I think that if the 
		///  component extended these interfaces (i.e. to talk with the HMI)
		///  this extended interface name would be included instead.  There is 
		///  some overlap here with supportsinterfaces that I don't quite understand.
		/// 
		/// I will assume that this is referenced, rather than referencing something else.
		/// 
		/// </summary>
		private Interface componentRep= null;
		/// <summary>
		/// The corbaversion element is intended to indicate the version 
		/// of CORBA that the delivered component supports.
		/// </summary>
		private String corbaVersion = "";

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="component">The parent SCA component.</param>
		public SoftwareComponent( SCAComponent component )
		{
			parentComponent = component;
		}

		/// <summary>
		/// The properties that are available for the component.
		/// </summary>
		public Properties Properties
		{
			get
			{
				return properties;
			}
			set
			{
				properties = value;
			}
		}
		/// <summary>
		/// The component features.
		/// </summary>
		public ComponentFeatures Features
		{
			get
			{
				return features;
			}
			set
			{
				features = value;
			}
		}
		/// <summary>
		/// The component type.
		/// </summary>
		public String ComponentType
		{
			get
			{
				return componentType;
			}
			set
			{
				componentType = value;
			}
		}
		/// <summary>
		/// The repositor id for the component.
		/// </summary>
		public Interface ComponentRepositoryID
		{
			get
			{
				return componentRep;
			}
			set
			{
				componentRep = value;
			}
		}
		/// <summary>
		/// The corba version.
		/// </summary>
		public String CorbaVersion
		{
			get
			{
				return corbaVersion;
			}
			set
			{
				corbaVersion = value;
			}
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public override XmlElement toXML( XmlDocument document )
		{
			document.XmlResolver = new JVMSXMLResolver();

			XmlDocumentType type = document.CreateDocumentType( "softwarecomponent", "", "softwarecomponent.2.2.dtd", null );
			XmlDeclaration decl = document.CreateXmlDeclaration( "1.0", "ISO-8859-1", "no" );
			document.AppendChild( decl );
			document.AppendChild( type );

			XmlElement element = document.CreateElement( "softwarecomponent" );
			XMLHelper.writeElement( document, element, "corbaversion", corbaVersion );
			
			if( componentRep != null )
			{
				XmlElement repElement = document.CreateElement( "componentrepid" );
				element.AppendChild( repElement );
				repElement.SetAttribute( "repid", componentRep.RepositoryID );
			}

			XMLHelper.writeElement( document, element, "componenttype", componentType );

			element.AppendChild( features.toXML( document ) );



			XmlElement interfacesElement = document.CreateElement( "interfaces" );
			element.AppendChild( interfacesElement );
			XMLHelper.writeListOfXMLElements( document, interfacesElement, getInterfaces() );

			return element;
		}

		/// <summary>
		/// This will get the list of interfaces that this component uses.
		/// </summary>
		/// <returns>The list of interfaces that this component uses.</returns>
		public IList getInterfaces()
		{
			//This should really be a set but C# does not have Sets.
			//We will collect all the interfaces that we need to care about
			//and then output them
			Hashtable interfacesSet = new Hashtable();
			foreach( ComponentPortEndPoint port in features.ProvidesPorts )
			{
				if( port.RepositoryID != null )
				{
					interfacesSet[port.RepositoryID] = port.RepositoryID;
				}
			}
			foreach( ComponentPortEndPoint port in features.UsesPorts )
			{
				if( port.RepositoryID != null )
				{
					interfacesSet[port.RepositoryID] = port.RepositoryID;
				}
			}
			foreach( Interface nextInterface in features.SupportInterfaces )
			{
				interfacesSet[nextInterface] = nextInterface;
			}
			if( componentRep != null )
			{
				interfacesSet[componentRep]= componentRep;
			}
			IList retval = new ArrayList( interfacesSet.Values );
			return retval;
		}

		/// <summary>
		/// This will get the relative name of the file to write to.  This
		/// string shall begin with a \
		/// </summary>
		/// <returns>The path to write this file to.</returns>
		public override String getFilename()
		{
			return "\\" + parentComponent.Name + ".scd.xml";
		}
	}
}
