using System;
using System.Xml;

using jvms.datamodel.common;
using jvms.xmlgeneration;

namespace jvms.datamodel.spd
{
	[Serializable]
	/// <summary>
	/// In general an implementation can have a list of generalized dependencies beyond the "builtins" specified by compiler, programminglanguage, humanlanguate, runtime, os and processor.  This element (of which there can be multiple) represents a single dependency.  It can refer to either required property, or require that other components (softpkgref) have to be loaded prior to this implementation being loaded.
	/// 
	/// SCA text: "The dependency element (see Figure D-6) is used to indicate the dependent relationships between the components being delivered and other components and devices, in an SCA compliant system.  The softpkgref element is used to specify a Software Package file that must be resident within the system for the component, described by this softpkg element, to load without errors.  The propertyref will reference a specific allocation property, using a unique identifier, and provide the value that will be used by a CF Device capacity model.  The CF DomainManager will use these dependency definitions to assure that components and devices that are necessary for proper operation of the implementation are present and available.  The type attribute is descriptive information indicating the type of dependency."
	/// </summary>
	public class Dependency
	{
		private SimpleRef propertyRef;
		private SoftwarePackageReference softwareDependency;
		private String type = "";

		/// <summary>
		/// A reference to a property.
		/// </summary>
		public SimpleRef PropertyRef
		{
			get
			{
				return propertyRef;
			}
			set
			{
				propertyRef = value;
			}
		}
		/// <summary>
		/// A reference to a software package dependency.
		/// </summary>
		public SoftwarePackageReference SoftwareDependency
		{
			get
			{
				return softwareDependency;
			}
			set
			{
				softwareDependency = value;
			}
		}
		/// <summary>
		/// The is simply descriptive information.
		/// </summary>
		public String Type
		{
			get
			{
				return type;
			}
			set
			{
				type = value;
			}
		}

		/// <summary>
		/// This will return a string representation of this class.
		/// </summary>
		/// <returns>This class represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Dependency>";
			if( !Type.Equals( "" ) )
			{
				retval = Type;
			}
			return retval;
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public XmlElement toXML( XmlDocument document, FileSystemObject parent )
		{
			XmlElement element = document.CreateElement( "dependency" );
			element.SetAttribute( "type", type );
			if( propertyRef != null &&
				softwareDependency == null )
			{
				XmlElement propertyElement = document.CreateElement( "propertyref" );
				propertyElement.SetAttribute( "refid", propertyRef.Simple.ID.DCEUUID );
				propertyElement.SetAttribute( "value", propertyRef.Value );
				element.AppendChild( propertyElement );
			}
			else if( propertyRef == null &&
				softwareDependency != null )
			{
				XmlElement softPkgElement = softwareDependency.toXML( document, parent );
				if( softPkgElement != null )
				{
					element.AppendChild( softwareDependency.toXML( document, parent ) );
				}
			}
			else
			{
				throw new Exception( "Invalid state in dependency" );
			}
			return element;
		}
	}
}
