using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.datamodel.prf;
using jvms.datamodel;

using jvms.xmlgeneration;

namespace jvms.datamodel.spd
{
	[Serializable]
	/// <summary>
	/// The implementation element (see Figure D-4) contains descriptive information about the particular implementation template for a software component contained in the softpkg element. The implementation element is intended to allow multiple component templates to be delivered to the system in one Software Package.  Each implementation element is intended to allow the same component to support different types of processors, operating systems, etc.  The implementation element will also allow definition of implementation-dependent properties for use in CF Device, CF Application, or CF Resource creation.  The implementation element's id attribute uniquely identifies a specific implementation of the component and is a DCE UUID value, as stated in section D.2.1.  The compiler, programminglanguage, humanlanguage, os, processor, and runtime elements are optional dependency elements.
	/// </summary>
	public class Implementation
	{
		private Code code = new Code();
		private TypedArrayList usesDevices = new TypedArrayList(typeof(UsesDevice));
		private TypedArrayList dependencies = new TypedArrayList(typeof(Dependency));
		/// <summary>
		/// The processor element will be used to indicate the processor and/or processor family on which this software component will operate. The processor name attribute will be defined in a property file as an allocation property of string type and with a name of processor_name and with an action element value other than external. The processor element is automatically interpreted as a dependency and compared against an allocation property with a name of processor_name. Legal processor_name attribute values are listed in Attachment 2 to this appendix. 
		/// </summary>
		private IList processors = new ArrayList();
		/// <summary>
		/// An optional dependency element specifying the os this implementation was developed for.  This is compared against the available os's for loadable/executable devices so that the appropriate implementation could be chosen.
		/// 
		/// BJL - This should be a list of NameVersion objects.
		/// </summary>
		private TypedArrayList osList = new TypedArrayList(typeof(OSNameVersion));
		/// <summary>
		/// An optional dependency element specifying the runtime required by this particular implemetation.  If specified you could require a particular platform runtime to match before this element is selected (applies mostly to Ada and Java)
		/// </summary>
		private NameVersion runtime = new NameVersion();
		/// <summary>
		/// An optional dependency element specifying the humanlanguage supported by this particular implementation.  If specified you could require a particular platform language to match before this element is selected (e.g. this is an english box, so don't install the french language versions of these components?)
		/// </summary>
		private String humanLanguage = "";
		/// <summary>
		/// An optional dependency element specifying the development language the code was written in.  If specified, you could require support for a particular language to match before this element is selected (efficiency considerations?)
		/// </summary>
		private NameVersion programmingLanguage = new NameVersion();
		/// <summary>
		/// An optional dependency element specifying the compiler used to generate the implemetation.  If specified you could require a particular platform compiler to match before this element is selected (possibly guarantee that the link file was in the proper format?)
		/// </summary>
		private NameVersion compiler = new NameVersion();		
		/// <summary>
		/// BJL - We are not supporting these right now, all properties
		/// will be defined at the SPD level.
		/// </summary>
		private Properties properties = null;
		/// <summary>
		/// Human readable description of the component
		/// </summary>
		private String description = "";
		/// <summary>
		/// This enum notes if the implementation is AEP compliant or not.  Note that if non-AEP calls are used you no longer have to make the component a Device  - just set this flag.
		/// </summary>
		private String aepCompliance = "";
		/// <summary>
		/// "The implementation element's id attribute uniquely identifies a specific implementation of the component and is a DCE UUID value, as stated in section D.2.1."
		/// 
		/// This implementation is referenced in the SAD when it needs to be know which particular implementation was choosen by the ApplicationFactory.  It is also returned by DeviceManager::getComponentImplementationId
		/// </summary>
		private ID id = new ID();

		/// <summary>
		/// The code for this implementation.
		/// </summary>
		public Code Code
		{
			get
			{
				return code;
			}
			set
			{
				code = value;
			}
		}
		/// <summary>
		/// The list of uses devices for this implementation.
		/// </summary>
		public TypedArrayList UsesDevices
		{
			get
			{
				return usesDevices;
			}
			set
			{
				usesDevices = value;
			}
		}
		/// <summary>
		/// The list of dependencies for this implementation.
		/// </summary>
		public TypedArrayList Dependencies
		{
			get
			{
				return dependencies;
			}
			set
			{
				dependencies = value;
			}
		}
		/// <summary>
		/// The list of processors for this implementation.
		/// </summary>
		public IList Processors
		{
			get
			{
				return processors;
			}
			set
			{
				processors = value;
			}
		}
		/// <summary>
		/// The os list for this implementation.
		/// </summary>
		public TypedArrayList OSList
		{
			get
			{
				return osList;
			}
			set
			{
				osList = value;
			}
		}
		/// <summary>
		/// The runtime for this implementation.
		/// </summary>
		public NameVersion Runtime
		{
			get
			{
				return runtime;
			}
			set
			{
				runtime = value;
			}
		}
		/// <summary>
		/// The human language for this implementation.
		/// </summary>
		public String HumanLanguage
		{
			get
			{
				return humanLanguage;
			}
			set
			{
				humanLanguage = value;
			}
		}
		/// <summary>
		/// The programming language for this implementation.
		/// </summary>
		public NameVersion ProgrammingLanguage
		{
			get
			{
				return programmingLanguage;
			}
			set
			{
				programmingLanguage = value;
			}
		}
		/// <summary>
		/// The compiler info for this implementation.
		/// </summary>
		public NameVersion Compiler
		{
			get
			{
				return compiler;
			}
			set
			{
				compiler = value;
			}
		}
		/// <summary>
		/// The properties that are associated with this implementation.
		/// </summary>
		public Properties Properties
		{
			get
			{
				return properties;
			}
			set
			{
				properties = value;
			}
		}
		/// <summary>
		/// The description of this implementation.
		/// </summary>
		public String Description
		{
			get
			{
				return description;
			}
			set
			{
				description = value;
			}
		}
		/// <summary>
		/// Boolean telling if this implementation is aep compliant.
		/// </summary>
		public String AEPCompliance
		{
			get
			{
				return aepCompliance;
			}
			set
			{
				aepCompliance = value;
			}
		}
		/// <summary>
		/// The id for the implementation.
		/// </summary>
		public ID ID
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
			}
		}

		/// <summary>
		/// This will return a string representation of this class.
		/// </summary>
		/// <returns>This class represented in a human readable form.</returns>
		public override String ToString()
		{
			String retval = "<New Implementation>";
			if( !Description.Equals( "" ) )
			{
				retval = Description;
			}
			return retval;
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public XmlElement toXML( XmlDocument document, FileSystemObject parent )
		{
			XmlElement element = document.CreateElement( "implementation" );
			element.SetAttribute( "id", id.DCEUUID );
			if( !aepCompliance.Equals( "" ) )
			{
				element.SetAttribute( "aepcompliance", aepCompliance );
			}
			XMLHelper.writeElement( document, element, "description", description );

			//We are not supporting properties at the implementation level
			//just the SPD level.
			//XMLHelper.writeFileReference( document, element, "propertyfile", this, properties );
			element.AppendChild( code.toXML( document ) );
			XMLHelper.writeNameVersion( document, element, "compiler", compiler );
			XMLHelper.writeNameVersion( document, element, "programminglanguage", programmingLanguage );
			XMLHelper.writeElement( document, element, "humanlanguage", humanLanguage );
			foreach( NameVersion os in osList )
			{
				XMLHelper.writeNameVersion( document, element, "os", os );
			}
			foreach( String processor in processors )
			{
				XMLHelper.writeElementWithAttribute( document, element, "processor", "name", processor );
			}
			foreach( Dependency dep in dependencies )
			{
				element.AppendChild( dep.toXML( document, parent ) );
			}
			XMLHelper.writeListOfXMLElements( document, element, usesDevices );
			

			return element;
		}

	}
}
