using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.common;
using jvms.datamodel.prf;

using jvms.xmlgeneration;

namespace jvms.datamodel.spd
{
	[Serializable]
	/// <summary>
	/// "The SCA Software Package Descriptor is used at deployment time to load an SCA compliant component and its various implementations.  The information contained in the Software Package Descriptor will provide the basis for the Domain Management function to manage the component within the SCA architecture.
	/// The software package descriptor may contain various implementations of any given component.  Within the specification of a software package descriptor several other files are referenced including a component level propertyfile and a software component descriptor file.  Within any given implementation there may be additional propertyfiles.
	/// 
	///       All files referenced by a Software Package are located in the same directory as the SPD file or a directory that is relative to the directory where the SPD file is located.
	/// 
	/// </summary>
	public class NonSCAComponent : FileSystemObject
	{
		private TypedArrayList usesDevices = new TypedArrayList(typeof(UsesDevice));
		private TypedArrayList authors = new TypedArrayList(typeof(Author));
		private TypedArrayList implementations = new TypedArrayList(typeof(Implementation));
		private Properties properties = null;
		/// <summary>
		/// Human readable description of the component
		/// </summary>
		private String description = "";
		/// <summary>
		/// The human-readable title for the software component.  Useful in HMI, installers, uninstallers, etc.  Optional.
		/// </summary>
		private String title = "";
		/// <summary>
		///  The version attribute specifies the version of the component.
		/// </summary>
		private String version = "";
		/// <summary>
		/// The type attribute indicates whether or not the component implementation is SCA compliant.
		/// </summary>
		private String type = "";
		/// <summary>
		/// The name attribute is a user-friendly label for the softpkg element.
		/// </summary>
		private String name = "";
		/// <summary>
		/// The softpkg element (see Figure D-2) indicates a Software Package Descriptor (SPD) definition.  The softpkg id uniquely identifies the package and is a DCE UUID.  The DCE UUID is as defined by the DCE UUID standard (adopted by CORBA).  The DCE UUID format starts with the characters "DCE:" and is followed by the printable form of the UUID, a colon, and a decimal minor version number, for example: "DCE:700dc518-0110-11ce-ac8f-0800090b5d3e:1".  The decimal minor version number is optional. 
		/// </summary>
		private ID id = new ID();

		public TypedArrayList UsesDevices
		{
			get
			{
				return usesDevices;
			}
			set
			{
				usesDevices = value;
			}
		}
		public TypedArrayList Authors
		{
			get
			{
				return authors;
			}
			set
			{
				authors = value;
			}
		}
		public TypedArrayList Implementations
		{
			get
			{
				return implementations;
			}
			set
			{
				implementations = value;
			}
		}
		public Properties Properties
		{
			get
			{
				return properties;
			}
			set
			{
				properties = value;
			}
		}
		public String Description
		{
			get { return description; }
			set { description = value; }
		}
		public String Title
		{
			get { return title; }
			set { title = value; }
		}
		public String Version
		{
			get { return version; }
			set { version = value; }
		}
		public String Type
		{
			get { return type; }
			set { type = value; }
		}
		public String Name
		{
			get { return name; }
			set { name = value; }
		}
		public ID ID
		{
			get { return id; }
			set { id = value; }
		}

		/// <summary>
		/// This will convert this object to an XML element.
		/// </summary>
		/// <param name="document">The document that this element will go into.</param>
		/// <returns>The element representation of the object.</returns>
		public override XmlElement toXML( XmlDocument document )
		{
			document.XmlResolver = new JVMSXMLResolver();
			XmlDocumentType xmlType = document.CreateDocumentType( "softpkg", "", "softpkg.2.2.dtd", null );
			XmlDeclaration decl = document.CreateXmlDeclaration( "1.0", "ISO-8859-1", "no" );
			document.AppendChild( decl );
			document.AppendChild( xmlType );

			XmlElement element = document.CreateElement( "softpkg" );
			element.SetAttribute( "id", id.DCEUUID );
			element.SetAttribute( "name", name );
			element.SetAttribute( "type", type );
			element.SetAttribute( "version", version );


			XMLHelper.writeElement( document, element, "title", title );
			XMLHelper.writeListOfXMLElements( document, element, authors );
			XMLHelper.writeElement( document, element, "description", description );
			XMLHelper.writeFileReference( document, element, "propertyfile", this, properties );
			if( this is SCAComponent )
			{
				XMLHelper.writeFileReference( document, element, "descriptor", this, ((SCAComponent)this).Descriptor );
			}
			foreach( Implementation impl in implementations )
			{
				element.AppendChild( impl.toXML( document, this ) );
			}
			XMLHelper.writeListOfXMLElements( document, element, usesDevices );
			return element;
		}

		/// <summary>
		/// This will get the relative name of the file to write to.  This
		/// string shall begin with a \
		/// </summary>
		/// <returns>The path to write this file to.</returns>
		public override String getFilename()
		{
			return "\\" + Name + ".spd.xml";
		}

		public override String ToString()
		{
			String retval = "<no name>";
			if( !name.Equals( "" ) )
			{
				retval = name;
			}
			return retval;
		}
	}
}
