using System;
using System.Collections;
using System.Xml;

using jvms.datamodel.spd;

namespace jvms.xmlgeneration
{
	/// <summary>
	/// Summary description for XMLHelper.
	/// </summary>
	public class XMLHelper
	{
		private XMLHelper()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		/// <summary>
		/// This will iterate over the elements in the list and append them to the
		/// parent element.
		/// </summary>
		/// <param name="parentElement"></param>
		/// <param name="elements"></param>
		public static void writeListOfXMLElements( XmlDocument doc, XmlElement parentElement, ICollection elements )
		{
			foreach( XMLObject xmlObject in elements )
			{
				parentElement.AppendChild( xmlObject.toXML( doc ) );
			}
		}

		/// <summary>
		/// This will iterate over the elements in the list and append them to the
		/// parent element.
		/// </summary>
		/// <param name="parentElement"></param>
		/// <param name="elements"></param>
		public static void writeListOfStringElements( XmlDocument doc, XmlElement parentElement, String name, IList elements )
		{
			foreach( String xmlObject in elements )
			{
				XmlElement element = doc.CreateElement( name );
				element.InnerText = xmlObject;
				parentElement.AppendChild( element );
			}
		}

		public static void writeElement( XmlDocument doc, XmlElement parentElement, String name, String value )
		{
			if( value != null && !value.Equals( "" ) )
			{
				XmlElement element = doc.CreateElement( name );
				element.InnerText = value;
				parentElement.AppendChild( element );
			}
		}

		/// <summary>
		/// This will write a reference to a local file.
		/// </summary>
		/// <param name="doc">The document to create elements from.</param>
		/// <param name="parentElement">The parent element to write to.</param>
		/// <param name="name">The name of the element to write out.</param>
		/// <param name="parentObject">The parent FileSystemObject.</param>
		/// <param name="childObject">The child FileSystemObject.</param>
		/// <returns>The element that was created.</returns>
		public static XmlElement writeFileReference( XmlDocument doc, XmlElement parentElement, String name, FileSystemObject parentObject, FileSystemObject childObject )
		{
			//break out early if child is null
			if( childObject == null )
			{
				return null;
			}

			//see appendix-2.2 D-5
			//ok the reason for the algorithm is because the child object
			//file reference must be relative to the object that we are currently
			//writing.  for example
			// parent=/common/parent.spd.xml 
			// child=/common/props.prf.xml
			// result=./props.pdf.xml (because it is in the same directory)
			//
			// parent=/parent.spd.xml 
			// child=/common/props.prf.xml
			// result=./common/props.pdf.xml (because it is in the below directory)
			//
			// parent=/common/parent.spd.xml 
			// child=/props.prf.xml
			// result=../props.pdf.xml (because it is in the above directory)

			
			XmlElement element = doc.CreateElement( name );
			parentElement.AppendChild( element );
			XmlElement localFile = doc.CreateElement( "localfile" );
			element.AppendChild( localFile );

			//and the algorithm goes like this!
			//tokenize the two paths
			char[] characters = new char[] { '\\' };
			String[] parentPath = parentObject.Directory.ToString().Split( characters );
			String[] childPath = childObject.Directory.ToString().Split( characters );

			String finalPath = "";
			//The maximum number of ../ that are possible is the number
			//of directories the parent has.
			int doubleDotCount = parentPath.Length;

			//have we veered off the path,
			//this will handle the case of
			//parent=/sca1/common/blah.spd.xml
			//child=/sca2/common/props.prf.xml
			//where we don't want to match the 'common' directory
			bool veeredOffPath = false;
			for( int i=0; i<childPath.Length; i++ )
			{				
				//if we are still on path and the directory is the same then
				//we can remove one set of ../
				if( !veeredOffPath &&
					i< parentPath.Length && 
					parentPath[i].Equals( childPath[i] ) )
				{
					doubleDotCount--;
				}
				else
				{
					//else we are off the path for good and we need to start
					//writing the different paths.
					veeredOffPath = true;
					if( !childPath[i].Equals( "" ) )
					{
						finalPath += childPath[i] + "/";
					}
				}
			}

			//if the doubleDotCount is zero that must mean we are in the same directory
			//or a directory below so we will start with ./
			//otherwise write out all the ../ that we need
			if( doubleDotCount == 0 || 
				(parentPath.Length>=2 && parentPath[1].Equals( "" ) ) )
			{
				finalPath = "./" + finalPath;
			}
			else
			{
				for( int i=0; i<doubleDotCount; i++ )
				{
					finalPath = "../" + finalPath;
				}
			}
			//append child name and remove leading slash
			finalPath += childObject.getFilename().Substring( 1 );
			localFile.SetAttribute( "name", finalPath );
			return element;
		}

		public static void writeAttribute( XmlElement parent, String name, String value )
		{
			if( !value.Equals( "" ) )
			{
				parent.SetAttribute( name, value );
			}
		}

		public static void writeElementWithAttribute( XmlDocument doc, XmlElement parentElement, String element, String attribute, String value )
		{
			if( !value.Equals( "" ) )
			{
				XmlElement xmlElement = doc.CreateElement( element );
				xmlElement.SetAttribute( attribute, value );
				parentElement.AppendChild( xmlElement );
			}
		}

		public static void writeNameVersion( XmlDocument doc, XmlElement parentElement, String name, NameVersion nv )
		{
			if( !nv.Name.Equals( "" ) || !nv.Version.Equals( "" ) )
			{
				XmlElement element = doc.CreateElement( name );
				element.SetAttribute( "name", nv.Name );
				element.SetAttribute( "version", nv.Version );
				parentElement.AppendChild( element );
			}
		}
	}	
}
