using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Reflection;
using System.Windows.Forms;
using System.Windows.Forms.Design;

using jvms.datamodel.scd;
using jvms.validation;

namespace jvms.presentation
{
	/// <summary>
	/// Creates an editor for a number of items that can be selected individually.
	/// </summary>
	public class DynamicCheckBoxesValueEditor : UITypeEditor
	{
		private IWindowsFormsEditorService editorService;

		/// <summary>
		/// Creates a new DynamicCheckBoxesValueEditor.
		/// </summary>
		public DynamicCheckBoxesValueEditor()
		{
		}

		/// <summary>
		/// Edit's the specified object's value.
		/// </summary>
		/// <param name="context">Provides contextual information about the object.</param>
		/// <param name="provider">Can be used by the editor to obtain services.</param>
		/// <param name="value">The object to be edited.</param>
		/// <returns>The new value of the object.</returns>
		public override object EditValue( ITypeDescriptorContext context, IServiceProvider provider, Object value )
		{
			editorService = (IWindowsFormsEditorService)provider.GetService( typeof( IWindowsFormsEditorService ) );
			Panel panel = new Panel();
			panel.AutoScroll = true;

			ICollection itemsForList = getValuesForList( context );

			ArrayList checkBoxes = new ArrayList();
			CheckBox lastCheckBox = null;
			foreach ( Object obj in itemsForList )
			{
				CheckBox cb = new CheckBox();
				cb.TextAlign = ContentAlignment.MiddleLeft;
				cb.Text = obj.ToString();
				cb.Tag = obj.ToString();
				cb.Height = 15;

				if ( lastCheckBox == null )
				{
					cb.Location = new Point( 3, 3 );
				}
				else
				{
					cb.Location = new Point( lastCheckBox.Left, lastCheckBox.Bottom + 2 );
				}

				foreach ( String selection in (IList)value )
				{
					if ( selection.Equals( obj.ToString() ) )
					{
						cb.Checked = true;
						break;
					}
				}

				lastCheckBox = cb;
				checkBoxes.Add( cb );
				panel.Controls.Add( cb );
			}

			editorService.DropDownControl( panel );

			ArrayList newValues = new ArrayList();
			foreach ( CheckBox cb in checkBoxes )
			{
				if ( cb.Checked )
				{
					newValues.Add( cb.Tag );
				}
			}
			return newValues;
		}

		/// <summary>
		/// Gets the editor style that will be used to edit the type.
		/// </summary>
		/// <param name="context">Provides contextual information.</param>
		/// <returns>The UITypeEditorEditStyle that will be used.</returns>
		public override UITypeEditorEditStyle GetEditStyle( ITypeDescriptorContext context )
		{
			return UITypeEditorEditStyle.DropDown;
		}

		/// <summary>
		/// Returns a list of values for the list.
		/// </summary>
		/// <param name="context">Contextual information.</param>
		/// <returns>A list of port types</returns>
		private ICollection getValuesForList( ITypeDescriptorContext context )
		{
			Flobbster.Windows.Forms.PropertyBag.PropertySpecDescriptor desc = (Flobbster.Windows.Forms.PropertyBag.PropertySpecDescriptor)context.PropertyDescriptor;
			BoundPropertySpec spec = (BoundPropertySpec)desc.Item;
			String propertyName = spec.PropertyName;
			
			AttributeSheet attSheet = (AttributeSheet)context.Instance;
			AttributeType currentType = null;
			
			String[] propertyPath = propertyName.Split( new char[] {'.'} );
			propertyName = propertyPath[ propertyPath.Length-1];
			foreach ( AttributeType attributeType in attSheet.AttributeTypes )
			{
				if ( propertyName.Equals( attributeType.BoundSpec.BoundProperty.Name ) )
				{					
					currentType = attributeType;
				}
			}		

			IList itemsForList = null;
			if ( currentType.DataProviderMethod != null )
			{
				DataProvider dataProvider = DataProvider.Instance;
				MethodInfo method = dataProvider.GetType().GetMethod( currentType.DataProviderMethod );
				itemsForList = (IList)method.Invoke( dataProvider, new Object[] {} );
			}
			else if ( currentType.ValidValues != null )
			{
				itemsForList = currentType.ValidValues;
			}

			return itemsForList;
		}
	}
}
