using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using jvms.project;
using jvms.system;
using jvms.presentation.drawablecomponent;

namespace jvms.presentation
{
	/// <summary>
	/// Main user interface form for JVMS.
	/// </summary>
	public class JVMSForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		private System.Windows.Forms.StatusBar statusBar1;
		private System.Windows.Forms.StatusBarPanel statusTextPanel;
		private System.Windows.Forms.StatusBarPanel progressBarPanel;
		private System.Windows.Forms.StatusBarPanel helpTextPanel;
		private System.Windows.Forms.MenuItem fileMenu;
		private System.Windows.Forms.MenuItem newProjectItem;
		private System.Windows.Forms.MenuItem saveProjectItem;
		private System.Windows.Forms.MenuItem saveProjectAsItem;
		private System.Windows.Forms.MenuItem exitItem;
		private System.Windows.Forms.MenuItem openProjectItem;
		private System.Windows.Forms.MenuItem fileSeparator1;
		private System.Windows.Forms.MenuItem editMenu;
		private System.Windows.Forms.MenuItem copyItem;
		private System.Windows.Forms.MenuItem pasteItem;
		private System.Windows.Forms.MenuItem deleteItem;
		private System.Windows.Forms.MenuItem toolsMenu;
		private System.Windows.Forms.MenuItem projectSettingsItem;
		private System.Windows.Forms.MenuItem optionsItem;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.ImageList toolBarImageList;
		private System.Windows.Forms.ToolBar jvmsToolBar;
		private System.Windows.Forms.ToolBarButton viewButton;
		private System.Windows.Forms.ToolBarButton openButton;
		private System.Windows.Forms.ToolBarButton saveButton;
		private System.Windows.Forms.ToolBarButton saveAsButton;
		private System.Windows.Forms.ToolBarButton separatorButton1;
		private System.Windows.Forms.ToolBarButton copyButton;
		private System.Windows.Forms.ToolBarButton pasteButton;
		private System.Windows.Forms.ToolBarButton taskListButton;
		private System.Windows.Forms.ToolBarButton separatorButton3;
		private System.Windows.Forms.ContextMenu viewContextMenu;
		private System.Windows.Forms.MenuItem platformMenu;
		private System.Windows.Forms.MenuItem applicationMenu;
		private System.Windows.Forms.MenuItem platCompItem;
		private System.Windows.Forms.MenuItem platAssmItem;
		private System.Windows.Forms.MenuItem appCompItem;
		private System.Windows.Forms.MenuItem appAssmItem;	
		private System.Windows.Forms.MenuItem fileSeparator2;
		private System.Windows.Forms.MenuItem toolsSeparator1;
		private System.Windows.Forms.MenuItem helpMenu;
		private System.Windows.Forms.MenuItem creditsItem;
		private System.Windows.Forms.MenuItem aboutItem;
		private System.Windows.Forms.MenuItem helpTopicsItem;
		private System.Windows.Forms.MenuItem helpSeparator1;
		private System.Windows.Forms.MenuItem exportMenuItem;
		private System.Windows.Forms.MenuItem exportImageItem;
		private System.Windows.Forms.ToolBarButton separatorButton2;
		private System.ComponentModel.IContainer components;
		#endregion
		
		private AttributeSheetFactory attributeSheetFactory = AttributeSheetFactory.Instance;
		private ControlPane controlPane;
		private TaskList taskList;
		private Panel viewPanel = new Panel();
		private View view;
		private DrawableComponent selectedComponent;
		private String loadedFromShellFile;
		private Splitter controlPaneSplitter;
		private Splitter taskListSplitter;

		/// <summary>Gets the status bar component.</summary>
		/// <value>The progress bar component associated with this form.</value>
		public StatusBar FormStatusBar
		{
			get
			{
				return statusBar1;
			}
		}

		/// <summary>Gets the attribute sheet.</summary>
		/// <value>The attribute sheet component.</value>
		public AttributeSheetControl CurrentAttributeSheetControl
		{
			get
			{
				return controlPane.CurrentAttributeSheetControl;
			}
		}

		/// <summary>Gets the toolbox control.</summary>
		/// <value>The toolbox control.</value>
		public JVMSToolBox ToolBoxControl
		{
			get
			{
				return controlPane.ToolBoxControl;
			}
		}

		/// <summary>Gets the task list control.</summary>
		/// <value>The tasklist control.</value>
		public TaskList TaskListControl
		{
			get
			{
				return taskList;
			}
		}

		/// <summary>Gets or sets the current view.</summary>
		/// <value>The current view.</value>
		public View CurrentView
		{
			get
			{
				return view;
			}

			set
			{
				this.Cursor = Cursors.WaitCursor;
				this.SuspendLayout();
                view = value;
				view.Dock = DockStyle.Fill;
				viewPanel.Controls.Clear();
				viewPanel.Controls.Add( view );
				controlPane.ToolBoxControl.viewChanged( value );
				this.ResumeLayout();
				this.Cursor = Cursors.Default;
				SelectedComponent = null;
			}
		}

		/// <summary>
		/// This represents the currently selected component.  This will be null
		/// if no component is selected right now.
		/// </summary>
		/// <value>The currently selected component.</value>
		public DrawableComponent SelectedComponent
		{
			get
			{
				return selectedComponent;
			}

			set
			{
				//don't do anything if the component is already selected
				//this speeds up drawing when moving a component around
				if ( value == null )
				{
					if ( selectedComponent != null )
					{
						selectedComponent.Selected = false;
						if ( controlPane.CurrentAttributeSheetControl.AttributeSheet != null &&
							 controlPane.CurrentAttributeSheetControl.AttributeSheet.WorkingObject == selectedComponent.getWrappedObject() )
						{							
							controlPane.CurrentAttributeSheetControl.clearAttributeSheet();
						}
					}
					selectedComponent = null;
				}
				else if ( value != selectedComponent )
				{
					if ( selectedComponent != null )
					{
						selectedComponent.Selected = false;
					}
					selectedComponent = value;
					selectedComponent.Selected = true;
					//for prototype			
					AttributeSheet sheet = 
						attributeSheetFactory.createAttributeSheet(selectedComponent.getWrappedObject());
					controlPane.CurrentAttributeSheetControl.AttributeSheet = sheet;
				}
			}
		}

		/// <summary>Gets or sets the path to a file loaded from the shell.</summary>
		/// <value>A file path.</value>
		public String LoadedFile
		{
			get
			{
				return loadedFromShellFile;
			}

			set
			{
				loadedFromShellFile = value;
			}
		}

		/// <summary>
		/// Creates a new JVMSForm.
		/// </summary>
		public JVMSForm()
		{
			initializeLayout();
			InitializeComponent();
			loadPreferences();
			initMenu();
			initToolBar();
			
			statusBar1.DrawItem += new StatusBarDrawItemEventHandler( drawProgressBar );
			this.Closing += new CancelEventHandler( windowClosing );
			this.Layout += new LayoutEventHandler( doLayout );
			this.Load += new EventHandler( doStartup );
		}

		/// <summary>
		/// Called when the form has been started and is being displayed.
		/// Perform various startup procedures.
		/// </summary>
		public void doStartup( Object source, EventArgs e )
		{
			JVMSProperties props = JVMSProperties.Instance;
			CurrentView = ViewManager.Instance.PlatCompView;

			if ( loadedFromShellFile != null )
			{
				try
				{
					Cursor = Cursors.WaitCursor;
					JVMSSerializer.Instance.readXml( loadedFromShellFile );
					ProjectManager.Instance.ProjectFile = loadedFromShellFile;
					updateTitle();
					CurrentAttributeSheetControl.clearAttributeSheet();
					view.Refresh();
				}
				catch ( Exception )
				{
					DialogResult r = MessageBox.Show( "Cannot load project.  Start a new project?", "Error", MessageBoxButtons.YesNo, MessageBoxIcon.Error );
					if ( r == DialogResult.Yes )
					{
						newProjectItem.PerformClick();
					}
				}
				finally
				{
					Cursor = Cursors.Default;
				}

				return;
			}

			if ( (bool)props.getProperty( "loadlast", false ) )
			{
				String projectFile = props.getProperty( "lastprojectfile", "" );
				if ( !projectFile.Equals( "" ) )
				{
					try
					{
						Cursor = Cursors.WaitCursor;
						JVMSSerializer.Instance.readXml( projectFile );
						ProjectManager.Instance.ProjectFile = projectFile;
						updateTitle();
						CurrentAttributeSheetControl.clearAttributeSheet();
						view.Refresh();
					}
					catch ( Exception )
					{
						DialogResult r = MessageBox.Show( "Cannot load last saved project.  Start a new project?", "Error", MessageBoxButtons.YesNo, MessageBoxIcon.Error );
						if ( r == DialogResult.Yes )
						{
							newProjectItem.PerformClick();
						}
					}
					finally
					{
						Cursor = Cursors.Default;
					}
				}
				else
				{
					DialogResult r = MessageBox.Show( "There is no previously saved project.  Start a new project?", "Information", MessageBoxButtons.YesNo, MessageBoxIcon.Question );
					if ( r == DialogResult.Yes )
					{
						newProjectItem.PerformClick();
					}
				}
			}
			else
			{
				newProjectItem.PerformClick();
			}
		}

		/// <summary>
		/// Called when a project has been loaded or created.
		/// </summary>
		public void initProject()
		{
			ProjectManager pm = ProjectManager.Instance;

			updateTitle();
			CurrentView = ViewManager.Instance.PlatCompView;
		}

		/// <summary>
		/// Updates the project name, which is displayed in the window header.
		/// </summary>
		public void updateTitle()
		{
			ProjectManager pm = ProjectManager.Instance;

			String headerText = "JTRS Visual Modeling Studio";

			if ( pm.Project.Name.Equals( "" ) )
			{
				headerText += " - Unnamed Project";
			}
			else
			{
				headerText += " - " + pm.Project.Name;
			}

			if ( !pm.ProjectFile.Equals( "" ) )
			{
				headerText += " (" + pm.ProjectFile + ")";
			}

			if ( pm.ChangedFlag )
			{
				headerText += "*";
			}

			Text = headerText;
		}

		/// <summary>
		/// Shows or hides the task list.
		/// </summary>
		/// <param name="visible">True if the task list is to be shown, false if it is to be hidden.</param>
		public void setTaskListVisible( bool visible )
		{
			taskListButton.Pushed = visible;
			taskListSplitter.Visible = visible;
			taskList.Visible = visible;
		}

		/// <summary>
		/// Initializes events for various main menu items.
		/// </summary>
		private void initMenu()
		{
			JVMSFormMenuListener menuListener = new JVMSFormMenuListener();

			this.saveProjectItem.Click += new EventHandler( menuListener.saveProjectItemClicked );
			this.saveProjectAsItem.Click += new EventHandler( menuListener.saveProjectAsItemClicked );
			this.openProjectItem.Click += new EventHandler( menuListener.openProjectItemClicked );
			this.newProjectItem.Click += new EventHandler( menuListener.newProjectItemClicked );
			this.deleteItem.Click += new EventHandler( menuListener.deleteItemClicked );
			this.exitItem.Click += new EventHandler( menuListener.exitItemClicked );
			this.projectSettingsItem.Click += new EventHandler( menuListener.projectSettingsItemClicked );
			this.optionsItem.Click += new EventHandler( menuListener.optionsItemClicked );
			this.exportImageItem.Click += new EventHandler( menuListener.exportImageItemClicked );
			this.helpTopicsItem.Click += new EventHandler( menuListener.helpTopicsItemClicked );
			this.creditsItem.Click += new EventHandler( menuListener.creditsItemClicked );
			this.aboutItem.Click += new EventHandler( menuListener.aboutItemClicked );
		}

		/// <summary>
		/// Initializes events for various toolbar items.
		/// </summary>
		private void initToolBar()
		{
			JVMSToolBarListener toolBarListener = new JVMSToolBarListener();

			this.jvmsToolBar.ButtonClick += new ToolBarButtonClickEventHandler( toolBarListener.buttonClicked );
            			
			this.platCompItem.Click += new EventHandler( toolBarListener.platformComponentViewSelected );
			this.platAssmItem.Click += new EventHandler( toolBarListener.platformAssemblyViewSelected );
			this.appCompItem.Click += new EventHandler( toolBarListener.applicationComponentViewSelected );
			this.appAssmItem.Click += new EventHandler( toolBarListener.applicationAssemblyViewSelected );
		}

		/// <summary>
		/// Initializes the layout of the form.
		/// </summary>
		private void initializeLayout()
		{
			this.controlPane = new ControlPane();
			this.controlPane.Dock = DockStyle.Right;
			this.taskList = new TaskList();
			this.taskList.Dock = DockStyle.Bottom;
			this.taskList.Visible = false;
			this.controlPaneSplitter = new Splitter();
			this.controlPaneSplitter.Dock = DockStyle.Right;
			this.taskListSplitter = new Splitter();
			this.taskListSplitter.Dock = DockStyle.Bottom;
			this.taskListSplitter.Visible = false;

			viewPanel.Dock = DockStyle.Fill;
			Controls.AddRange( 
				new Control[] { viewPanel, taskListSplitter, taskList, controlPaneSplitter, controlPane } );
		}

		/// <summary>
		/// Applied user preferences to the form.
		/// </summary>
		private void loadPreferences() 
		{
			JVMSProperties props = JVMSProperties.Instance;
			this.Width = props.getProperty( "window.width", 800 );
			this.Height = props.getProperty( "window.height", 600 );
			this.Left = props.getProperty( "window.x", 0 );
			this.Top = props.getProperty( "window.y", 0 );
			String windowState = props.getProperty( "window.state", "Maximized" );
			if ( windowState.Equals( "Maximized" ) )
			{
				this.WindowState = FormWindowState.Maximized;
			}
			else
			{
				this.WindowState = FormWindowState.Normal;
			}
			this.controlPaneSplitter.SplitPosition = props.getProperty( "jvmsform.controlpanesplit", 200 );
			this.taskListSplitter.SplitPosition = props.getProperty( "jvmsform.tasklistsplit", 200 );
			this.taskList.Height = props.getProperty( "jvmsform.tasklistsplit", 200 );
		}

		/// <summary>
		/// Called when the form should perform layout operations on its components.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void doLayout( Object source, LayoutEventArgs e )
		{
			if ( view != null )
			{
				view.PerformLayout();
			}
		}

		/// <summary>
		/// Called when the form is being closed.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void windowClosing( Object source, CancelEventArgs e )
		{
			ProjectManager pm = ProjectManager.Instance;
			if ( pm.ChangedFlag )
			{
				String projectName = pm.Project.Name;
				if ( projectName.Equals( "" ) )
				{
					projectName = "Unnamed Project";
				}

				DialogResult result = MessageBox.Show( this, "Save changes to " + projectName + "?",
					"Save Changes", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question );
				if ( result == DialogResult.Yes )
				{
					pm.saveProject();
				}
				else if ( result == DialogResult.Cancel )
				{
					e.Cancel = true;
				}
			}

			JVMSProperties props = JVMSProperties.Instance;
			if ( this.WindowState == FormWindowState.Normal )
			{
				props.setProperty( "window.width", this.Width );
				props.setProperty( "window.height", this.Height );
				props.setProperty( "window.x", this.Left );
				props.setProperty( "window.y", this.Top );
			}

			props.setProperty( "window.state", this.WindowState );
			props.setProperty( "jvmsform.controlpanesplit", this.controlPaneSplitter.SplitPosition );
			if ( this.taskListSplitter.SplitPosition > 0 )
			{
				props.setProperty( "jvmsform.tasklistsplit", this.taskListSplitter.SplitPosition );
			}
			props.setProperty( "lastprojectfile", ProjectManager.Instance.ProjectFile );

			controlPane.setPreferences();

			props.writeProps();
		}

		/// <summary>
		/// Called when the owner-drawn portion of the status bar (the progress bar) is
		/// to be updated.  Draws the status bar according to its current state (progressMin,
		/// progressMax, and progressVal).
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void drawProgressBar( Object source, StatusBarDrawItemEventArgs e )
		{
			Graphics g = e.Graphics;
			Rectangle rect = e.Bounds;
			int progressMax = StatusBarManager.Instance.ProgressMax;
			int progressMin = StatusBarManager.Instance.ProgressMin;
			int progressVal = StatusBarManager.Instance.ProgressValue;
			
			if ( progressVal != 0 && ( progressMax - progressMin != 0 ) )
			{
				float factor = (float)progressVal / (float)( progressMax - progressMin );
				float barWidth = ( rect.Width - 6 ) * factor;
				SolidBrush brush = new SolidBrush( Color.Navy );
				g.FillRectangle( brush, rect.X + 3, rect.Y + 3, barWidth, rect.Height - 6 );
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(JVMSForm));
			this.statusBar1 = new System.Windows.Forms.StatusBar();
			this.statusTextPanel = new System.Windows.Forms.StatusBarPanel();
			this.progressBarPanel = new System.Windows.Forms.StatusBarPanel();
			this.helpTextPanel = new System.Windows.Forms.StatusBarPanel();
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.fileMenu = new System.Windows.Forms.MenuItem();
			this.newProjectItem = new System.Windows.Forms.MenuItem();
			this.openProjectItem = new System.Windows.Forms.MenuItem();
			this.fileSeparator1 = new System.Windows.Forms.MenuItem();
			this.saveProjectItem = new System.Windows.Forms.MenuItem();
			this.saveProjectAsItem = new System.Windows.Forms.MenuItem();
			this.fileSeparator2 = new System.Windows.Forms.MenuItem();
			this.exitItem = new System.Windows.Forms.MenuItem();
			this.editMenu = new System.Windows.Forms.MenuItem();
			this.copyItem = new System.Windows.Forms.MenuItem();
			this.pasteItem = new System.Windows.Forms.MenuItem();
			this.deleteItem = new System.Windows.Forms.MenuItem();
			this.toolsMenu = new System.Windows.Forms.MenuItem();
			this.projectSettingsItem = new System.Windows.Forms.MenuItem();
			this.optionsItem = new System.Windows.Forms.MenuItem();
			this.toolsSeparator1 = new System.Windows.Forms.MenuItem();
			this.exportMenuItem = new System.Windows.Forms.MenuItem();
			this.exportImageItem = new System.Windows.Forms.MenuItem();
			this.helpMenu = new System.Windows.Forms.MenuItem();
			this.helpTopicsItem = new System.Windows.Forms.MenuItem();
			this.helpSeparator1 = new System.Windows.Forms.MenuItem();
			this.creditsItem = new System.Windows.Forms.MenuItem();
			this.aboutItem = new System.Windows.Forms.MenuItem();
			this.toolBarImageList = new System.Windows.Forms.ImageList(this.components);
			this.jvmsToolBar = new System.Windows.Forms.ToolBar();
			this.viewButton = new System.Windows.Forms.ToolBarButton();
			this.viewContextMenu = new System.Windows.Forms.ContextMenu();
			this.platformMenu = new System.Windows.Forms.MenuItem();
			this.platCompItem = new System.Windows.Forms.MenuItem();
			this.platAssmItem = new System.Windows.Forms.MenuItem();
			this.applicationMenu = new System.Windows.Forms.MenuItem();
			this.appCompItem = new System.Windows.Forms.MenuItem();
			this.appAssmItem = new System.Windows.Forms.MenuItem();
			this.separatorButton1 = new System.Windows.Forms.ToolBarButton();
			this.openButton = new System.Windows.Forms.ToolBarButton();
			this.saveButton = new System.Windows.Forms.ToolBarButton();
			this.saveAsButton = new System.Windows.Forms.ToolBarButton();
			this.separatorButton2 = new System.Windows.Forms.ToolBarButton();
			this.copyButton = new System.Windows.Forms.ToolBarButton();
			this.pasteButton = new System.Windows.Forms.ToolBarButton();
			this.separatorButton3 = new System.Windows.Forms.ToolBarButton();
			this.taskListButton = new System.Windows.Forms.ToolBarButton();
			((System.ComponentModel.ISupportInitialize)(this.statusTextPanel)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.progressBarPanel)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.helpTextPanel)).BeginInit();
			this.SuspendLayout();
			// 
			// statusBar1
			// 
			this.statusBar1.Location = new System.Drawing.Point(0, 549);
			this.statusBar1.Name = "statusBar1";
			this.statusBar1.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																						  this.statusTextPanel,
																						  this.progressBarPanel,
																						  this.helpTextPanel});
			this.statusBar1.ShowPanels = true;
			this.statusBar1.Size = new System.Drawing.Size(792, 24);
			this.statusBar1.SizingGrip = false;
			this.statusBar1.TabIndex = 0;
			// 
			// statusTextPanel
			// 
			this.statusTextPanel.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.statusTextPanel.MinWidth = 200;
			this.statusTextPanel.Text = "JTRS Visual Modeling Studio";
			this.statusTextPanel.Width = 472;
			// 
			// progressBarPanel
			// 
			this.progressBarPanel.Alignment = System.Windows.Forms.HorizontalAlignment.Center;
			this.progressBarPanel.MinWidth = 50;
			this.progressBarPanel.Style = System.Windows.Forms.StatusBarPanelStyle.OwnerDraw;
			this.progressBarPanel.Width = 120;
			// 
			// helpTextPanel
			// 
			this.helpTextPanel.MinWidth = 50;
			this.helpTextPanel.Width = 200;
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.fileMenu,
																					 this.editMenu,
																					 this.toolsMenu,
																					 this.helpMenu});
			// 
			// fileMenu
			// 
			this.fileMenu.Index = 0;
			this.fileMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.newProjectItem,
																					 this.openProjectItem,
																					 this.fileSeparator1,
																					 this.saveProjectItem,
																					 this.saveProjectAsItem,
																					 this.fileSeparator2,
																					 this.exitItem});
			this.fileMenu.Text = "File";
			// 
			// newProjectItem
			// 
			this.newProjectItem.Index = 0;
			this.newProjectItem.Shortcut = System.Windows.Forms.Shortcut.CtrlShiftN;
			this.newProjectItem.Text = "New Project";
			// 
			// openProjectItem
			// 
			this.openProjectItem.Index = 1;
			this.openProjectItem.Shortcut = System.Windows.Forms.Shortcut.CtrlShiftO;
			this.openProjectItem.Text = "Open Project";
			// 
			// fileSeparator1
			// 
			this.fileSeparator1.Index = 2;
			this.fileSeparator1.Text = "-";
			// 
			// saveProjectItem
			// 
			this.saveProjectItem.Index = 3;
			this.saveProjectItem.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
			this.saveProjectItem.Text = "Save Project";
			// 
			// saveProjectAsItem
			// 
			this.saveProjectAsItem.Index = 4;
			this.saveProjectAsItem.Text = "Save Project As...";
			// 
			// fileSeparator2
			// 
			this.fileSeparator2.Index = 5;
			this.fileSeparator2.Text = "-";
			// 
			// exitItem
			// 
			this.exitItem.Index = 6;
			this.exitItem.Text = "Exit";
			// 
			// editMenu
			// 
			this.editMenu.Index = 1;
			this.editMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.copyItem,
																					 this.pasteItem,
																					 this.deleteItem});
			this.editMenu.Text = "Edit";
			// 
			// copyItem
			// 
			this.copyItem.Enabled = false;
			this.copyItem.Index = 0;
			this.copyItem.Shortcut = System.Windows.Forms.Shortcut.CtrlC;
			this.copyItem.Text = "Copy";
			// 
			// pasteItem
			// 
			this.pasteItem.Enabled = false;
			this.pasteItem.Index = 1;
			this.pasteItem.Shortcut = System.Windows.Forms.Shortcut.CtrlV;
			this.pasteItem.Text = "Paste";
			// 
			// deleteItem
			// 
			this.deleteItem.Index = 2;
			this.deleteItem.Shortcut = System.Windows.Forms.Shortcut.Del;
			this.deleteItem.Text = "Delete";
			// 
			// toolsMenu
			// 
			this.toolsMenu.Index = 2;
			this.toolsMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.projectSettingsItem,
																					  this.optionsItem,
																					  this.toolsSeparator1,
																					  this.exportMenuItem});
			this.toolsMenu.Text = "Tools";
			// 
			// projectSettingsItem
			// 
			this.projectSettingsItem.Index = 0;
			this.projectSettingsItem.Text = "Project Settings";
			// 
			// optionsItem
			// 
			this.optionsItem.Index = 1;
			this.optionsItem.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.optionsItem.Text = "Options";
			// 
			// toolsSeparator1
			// 
			this.toolsSeparator1.Index = 2;
			this.toolsSeparator1.Text = "-";
			// 
			// exportMenuItem
			// 
			this.exportMenuItem.Index = 3;
			this.exportMenuItem.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						   this.exportImageItem});
			this.exportMenuItem.Text = "Export View";
			// 
			// exportImageItem
			// 
			this.exportImageItem.Index = 0;
			this.exportImageItem.Text = "To Image";
			// 
			// helpMenu
			// 
			this.helpMenu.Index = 3;
			this.helpMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.helpTopicsItem,
																					 this.helpSeparator1,
																					 this.creditsItem,
																					 this.aboutItem});
			this.helpMenu.Text = "Help";
			// 
			// helpTopicsItem
			// 
			this.helpTopicsItem.Index = 0;
			this.helpTopicsItem.Shortcut = System.Windows.Forms.Shortcut.F1;
			this.helpTopicsItem.Text = "Help Topics";
			// 
			// helpSeparator1
			// 
			this.helpSeparator1.Index = 1;
			this.helpSeparator1.Text = "-";
			// 
			// creditsItem
			// 
			this.creditsItem.Index = 2;
			this.creditsItem.Text = "Credits";
			// 
			// aboutItem
			// 
			this.aboutItem.Index = 3;
			this.aboutItem.Text = "About JVMS";
			// 
			// toolBarImageList
			// 
			this.toolBarImageList.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.toolBarImageList.ImageSize = new System.Drawing.Size(18, 18);
			this.toolBarImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("toolBarImageList.ImageStream")));
			this.toolBarImageList.TransparentColor = System.Drawing.Color.Cyan;
			// 
			// jvmsToolBar
			// 
			this.jvmsToolBar.Appearance = System.Windows.Forms.ToolBarAppearance.Flat;
			this.jvmsToolBar.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
																						   this.viewButton,
																						   this.separatorButton1,
																						   this.openButton,
																						   this.saveButton,
																						   this.saveAsButton,
																						   this.separatorButton2,
																						   this.copyButton,
																						   this.pasteButton,
																						   this.separatorButton3,
																						   this.taskListButton});
			this.jvmsToolBar.DropDownArrows = true;
			this.jvmsToolBar.ImageList = this.toolBarImageList;
			this.jvmsToolBar.Name = "jvmsToolBar";
			this.jvmsToolBar.ShowToolTips = true;
			this.jvmsToolBar.Size = new System.Drawing.Size(792, 27);
			this.jvmsToolBar.TabIndex = 1;
			this.jvmsToolBar.TextAlign = System.Windows.Forms.ToolBarTextAlign.Right;
			// 
			// viewButton
			// 
			this.viewButton.DropDownMenu = this.viewContextMenu;
			this.viewButton.ImageIndex = 0;
			this.viewButton.Style = System.Windows.Forms.ToolBarButtonStyle.DropDownButton;
			this.viewButton.Tag = "view";
			this.viewButton.Text = "Views";
			this.viewButton.ToolTipText = "Views";
			// 
			// viewContextMenu
			// 
			this.viewContextMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																							this.platformMenu,
																							this.applicationMenu});
			// 
			// platformMenu
			// 
			this.platformMenu.Index = 0;
			this.platformMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.platCompItem,
																						 this.platAssmItem});
			this.platformMenu.Text = "Platform";
			// 
			// platCompItem
			// 
			this.platCompItem.Index = 0;
			this.platCompItem.Text = "Component";
			// 
			// platAssmItem
			// 
			this.platAssmItem.Index = 1;
			this.platAssmItem.Text = "Assembly";
			// 
			// applicationMenu
			// 
			this.applicationMenu.Index = 1;
			this.applicationMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																							this.appCompItem,
																							this.appAssmItem});
			this.applicationMenu.Text = "Application";
			// 
			// appCompItem
			// 
			this.appCompItem.Index = 0;
			this.appCompItem.Text = "Component";
			// 
			// appAssmItem
			// 
			this.appAssmItem.Index = 1;
			this.appAssmItem.Text = "Assembly";
			// 
			// separatorButton1
			// 
			this.separatorButton1.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// openButton
			// 
			this.openButton.ImageIndex = 4;
			this.openButton.Tag = "open";
			this.openButton.ToolTipText = "Open Project";
			// 
			// saveButton
			// 
			this.saveButton.ImageIndex = 1;
			this.saveButton.Tag = "save";
			this.saveButton.ToolTipText = "Save Project";
			// 
			// saveAsButton
			// 
			this.saveAsButton.ImageIndex = 2;
			this.saveAsButton.Tag = "saveas";
			this.saveAsButton.ToolTipText = "Save Project As";
			// 
			// separatorButton2
			// 
			this.separatorButton2.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// copyButton
			// 
			this.copyButton.Enabled = false;
			this.copyButton.ImageIndex = 5;
			this.copyButton.Tag = "copy";
			this.copyButton.ToolTipText = "Copy";
			// 
			// pasteButton
			// 
			this.pasteButton.Enabled = false;
			this.pasteButton.ImageIndex = 3;
			this.pasteButton.Tag = "paste";
			this.pasteButton.ToolTipText = "Paste";
			// 
			// separatorButton3
			// 
			this.separatorButton3.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
			// 
			// taskListButton
			// 
			this.taskListButton.ImageIndex = 6;
			this.taskListButton.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
			this.taskListButton.Tag = "tasklist";
			this.taskListButton.ToolTipText = "Task List";
			// 
			// JVMSForm
			// 
			this.AccessibleDescription = "The main JVMS window.";
			this.AccessibleName = "JVMS Window";
			this.AccessibleRole = System.Windows.Forms.AccessibleRole.Application;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(792, 573);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.jvmsToolBar,
																		  this.statusBar1});
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mainMenu;
			this.MinimumSize = new System.Drawing.Size(300, 300);
			this.Name = "JVMSForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "JTRS Visual Modeling Studio";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			((System.ComponentModel.ISupportInitialize)(this.statusTextPanel)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.progressBarPanel)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.helpTextPanel)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

	}
}
