using System;
using System.Drawing;
using System.Windows.Forms;

using jvms.project;

namespace jvms.presentation
{
	/// <summary>
	/// Listens for events from the main menu and takes appropriate action.
	/// </summary>
	public class JVMSFormMenuListener
	{
		/// <summary>
		/// Creates a new JVMSFormMenuListener.
		/// </summary>
		public JVMSFormMenuListener()
		{
		}

		/// <summary>
		/// Called when the designer selects the save project menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void saveProjectItemClicked( Object source, EventArgs e )
		{
			ProjectManager.Instance.saveProject();
		}

		/// <summary>
		/// Called when the designer selects the save project as menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void saveProjectAsItemClicked( Object source, EventArgs e )
		{
			ProjectManager.Instance.saveProjectAs();
		}

		/// <summary>
		/// Called when the designer selects the open project menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void openProjectItemClicked( Object source, EventArgs e )
		{
			ProjectManager.Instance.openProject();
		}

		/// <summary>
		/// Called when the designer selects the new project menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void newProjectItemClicked( Object source, EventArgs e )
		{
			NewProjectForm newProjectForm = new NewProjectForm();
			newProjectForm.StartPosition = FormStartPosition.Manual;
			newProjectForm.Location = new Point( 200, 200 );
			newProjectForm.ShowDialog( JVMSMain.JvmsForm );
		}

		/// <summary>
		/// Called when the designer selects the delete item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void deleteItemClicked( Object source, EventArgs e )
		{
			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			RelationshipManager rm = RelationshipManager.Instance;

			if ( jvmsForm.SelectedComponent != null )
			{
				DialogResult r = 
					MessageBox.Show( "Deleting this component will remove all references to it.  " +
					"Are you sure you want to delete this item?", 
					"Delete?", 
					MessageBoxButtons.YesNo, 
					MessageBoxIcon.Question );
				if ( r == DialogResult.Yes )
				{
					jvmsForm.SelectedComponent.deleteComponent( source, e );
					jvmsForm.CurrentView.Refresh();
					ProjectManager.Instance.ChangedFlag = true;
				}
			}
			else if ( rm.SelectedRelationship != null )
			{
				rm.removeRelationship( rm.SelectedRelationship );
				rm.SelectedRelationship = null;
				ProjectManager.Instance.ChangedFlag = true;
				jvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
				jvmsForm.CurrentView.Invalidate( false );
			}
		}

		/// <summary>
		/// Called when the designer selects the project settings menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void projectSettingsItemClicked( Object source, EventArgs e )
		{
			NewProjectForm projectForm = new NewProjectForm();
			projectForm.Text = "Project Settings";
			projectForm.setEditMode( true );
			projectForm.StartPosition = FormStartPosition.Manual;
			projectForm.Location = new Point( 200, 200 );
			projectForm.ShowDialog( JVMSMain.JvmsForm );
		}

		/// <summary>
		/// Called when the designer selects the options menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void optionsItemClicked( Object source, EventArgs e )
		{
			OptionsForm optionsForm = new OptionsForm();
			optionsForm.ShowDialog( JVMSMain.JvmsForm );
		}

		/// <summary>
		/// Called when the designer selects the export image menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void exportImageItemClicked( Object source, EventArgs e )
		{
			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			SaveFileDialog saveDialog = new SaveFileDialog();
			saveDialog.Title = "Save View to Image";
            saveDialog.RestoreDirectory = true;
			saveDialog.AddExtension = true;
			saveDialog.Filter = "Windows Bitmap (*.bmp)|*.bmp|" + 
								"JPEG (*.jpg;*.jpeg)|*.jpg;*.jpeg|" +
								"GIF (*.gif)|*.gif|" +
								"Portable Network Graphic (*.png)|*.png|" +
								"TIFF (*.tif;*.tiff)|*.tif;*.tiff";
			saveDialog.InitialDirectory = ProjectManager.Instance.Project.ProjectDirectory;

			if ( saveDialog.ShowDialog( jvmsForm ) == DialogResult.OK )
			{
				jvmsForm.Refresh();
				jvmsForm.Cursor = Cursors.WaitCursor;
				jvmsForm.CurrentView.saveAsImage( saveDialog.FileName );
				jvmsForm.Cursor = Cursors.Default;
			}
		}

		/// <summary>
		/// Called when the designer selects the help topics menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void helpTopicsItemClicked( Object source, EventArgs e )
		{
			Help.ShowHelp( JVMSMain.JvmsForm, JVMSMain.AssemblyDir + "jvms.chm" );
		}

		/// <summary>
		/// Called when the designed selects the credit menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void creditsItemClicked( Object source, EventArgs e )
		{
			CreditsForm creditsForm = new CreditsForm();
			creditsForm.ShowDialog( JVMSMain.JvmsForm );
		}

		/// <summary>
		/// Called when the designer selects the about menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void aboutItemClicked( Object source, EventArgs e )
		{
			AboutForm aboutForm = new AboutForm();
			aboutForm.ShowDialog( JVMSMain.JvmsForm );
		}

		/// <summary>
		/// Called when the designer selects the exit menu item.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void exitItemClicked( Object source, EventArgs e )
		{
			JVMSMain.JvmsForm.Close();
		}
	}
}
