using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.IO;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.project;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Presents controls for creating a new project.
	/// </summary>
	public class NewProjectForm : System.Windows.Forms.Form
	{
		#region Windows Form Designer generated state
		private System.Windows.Forms.Label projectNameLabel;		
		private System.Windows.Forms.Label authorLabel;
		private System.Windows.Forms.Label companyLabel;
		private System.Windows.Forms.TextBox projectNameTextBox;
		private System.Windows.Forms.TextBox authorTextBox;
		private System.Windows.Forms.TextBox companyTextBox;
		private System.Windows.Forms.Button cancelButton;
		private System.Windows.Forms.Button okButton;
		private System.Windows.Forms.Label projectLocationLabel;
		private System.Windows.Forms.TextBox projectLocationTextBox;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.Button browseButton;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		#endregion

		private FolderBrowser browseDialog;
		private bool editMode = false;

		/// <summary>
		/// Creates a new NewProjectForm.
		/// </summary>
		public NewProjectForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.browseButton.Click += new EventHandler( browseButtonClicked );
			this.cancelButton.Click += new EventHandler( cancelButtonClicked );
			this.okButton.Click += new EventHandler( okButtonClicked );
		}

		/// <summary>
		/// Sets the project form to edit mode, which edits the current project instead of creating a new one.
		/// </summary>
		/// <param name="edit">True if the form is to be in edit mode.</param>
		public void setEditMode( bool edit )
		{
			this.editMode = edit;

			Project proj = ProjectManager.Instance.Project;
			authorTextBox.Text = proj.Author;
			companyTextBox.Text = proj.Company;
			projectNameTextBox.Text = proj.Name;
			projectLocationTextBox.Text = proj.ProjectDirectory;
		}

		/// <summary>
		/// Called when the browse button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void browseButtonClicked( Object source, EventArgs e )
		{
			if ( browseDialog == null )
			{
				String folderPath = showBrowseDialog();
				this.projectLocationTextBox.Text = folderPath;
				browseDialog = null;
			}
		}

		/// <summary>
		/// Called when the cancel button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void cancelButtonClicked( Object source, EventArgs e )
		{
			this.Close();
		}

		/// <summary>
		/// Called when the ok button is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void okButtonClicked( Object source, EventArgs e )
		{
			if ( !Directory.Exists( projectLocationTextBox.Text ) )
			{
				MessageBox.Show( this, "The specified project directory does not exist or is invalid.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error );
				return;
			}

			if ( editMode )
			{
				Project proj = ProjectManager.Instance.Project;
				proj.Author = authorTextBox.Text;
				proj.Company = companyTextBox.Text;
				proj.Name = projectNameTextBox.Text;
				proj.ProjectDirectory = projectLocationTextBox.Text;

				JVMSMain.JvmsForm.initProject();
			}
			else
			{
				Project proj = new Project();
				proj.Author = authorTextBox.Text;
				proj.Company = companyTextBox.Text;
				proj.Name = projectNameTextBox.Text;
				proj.ProjectDirectory = projectLocationTextBox.Text;

				ProjectManager pm = ProjectManager.Instance;
				pm.ProjectFile = "";
				pm.Project = proj;
			
				ViewManager.Instance.resetViews();
				JVMSMain.JvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
				JVMSMain.JvmsForm.TaskListControl.clearTasks();
				RelationshipManager.Instance.clearRelationships();
			}
			this.Close();
		}

		/// <summary>
		/// Shows a dialog which allows a folder to be selected and returns the path to that folder.
		/// </summary>
		/// <returns>A string with the path to the selected folder.</returns>
		private String showBrowseDialog()
		{
			browseDialog = new FolderBrowser();
			browseDialog.Title = "Select a Project Folder";
			String folderPath = "";
			if ( browseDialog.ShowDialog() == DialogResult.OK )
			{
				folderPath = browseDialog.DirectoryPath;
			}
			return folderPath;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.projectNameLabel = new System.Windows.Forms.Label();
			this.projectNameTextBox = new System.Windows.Forms.TextBox();
			this.authorLabel = new System.Windows.Forms.Label();
			this.companyLabel = new System.Windows.Forms.Label();
			this.authorTextBox = new System.Windows.Forms.TextBox();
			this.companyTextBox = new System.Windows.Forms.TextBox();
			this.cancelButton = new System.Windows.Forms.Button();
			this.okButton = new System.Windows.Forms.Button();
			this.projectLocationLabel = new System.Windows.Forms.Label();
			this.projectLocationTextBox = new System.Windows.Forms.TextBox();
			this.browseButton = new System.Windows.Forms.Button();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.groupBox1.SuspendLayout();
			this.SuspendLayout();
			// 
			// projectNameLabel
			// 
			this.projectNameLabel.Location = new System.Drawing.Point(16, 16);
			this.projectNameLabel.Name = "projectNameLabel";
			this.projectNameLabel.Size = new System.Drawing.Size(80, 20);
			this.projectNameLabel.TabIndex = 0;
			this.projectNameLabel.Text = "Project Name";
			this.projectNameLabel.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// projectNameTextBox
			// 
			this.projectNameTextBox.Location = new System.Drawing.Point(104, 16);
			this.projectNameTextBox.Name = "projectNameTextBox";
			this.projectNameTextBox.Size = new System.Drawing.Size(216, 20);
			this.projectNameTextBox.TabIndex = 1;
			this.projectNameTextBox.Text = "";
			// 
			// authorLabel
			// 
			this.authorLabel.Location = new System.Drawing.Point(16, 44);
			this.authorLabel.Name = "authorLabel";
			this.authorLabel.Size = new System.Drawing.Size(76, 20);
			this.authorLabel.TabIndex = 2;
			this.authorLabel.Text = "Author";
			this.authorLabel.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// companyLabel
			// 
			this.companyLabel.Location = new System.Drawing.Point(16, 72);
			this.companyLabel.Name = "companyLabel";
			this.companyLabel.Size = new System.Drawing.Size(80, 20);
			this.companyLabel.TabIndex = 3;
			this.companyLabel.Text = "Company";
			this.companyLabel.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// authorTextBox
			// 
			this.authorTextBox.Location = new System.Drawing.Point(104, 44);
			this.authorTextBox.Name = "authorTextBox";
			this.authorTextBox.Size = new System.Drawing.Size(216, 20);
			this.authorTextBox.TabIndex = 4;
			this.authorTextBox.Text = "";
			// 
			// companyTextBox
			// 
			this.companyTextBox.Location = new System.Drawing.Point(104, 72);
			this.companyTextBox.Name = "companyTextBox";
			this.companyTextBox.Size = new System.Drawing.Size(216, 20);
			this.companyTextBox.TabIndex = 5;
			this.companyTextBox.Text = "";
			// 
			// cancelButton
			// 
			this.cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.cancelButton.Location = new System.Drawing.Point(360, 152);
			this.cancelButton.Name = "cancelButton";
			this.cancelButton.Size = new System.Drawing.Size(80, 24);
			this.cancelButton.TabIndex = 6;
			this.cancelButton.Text = "Cancel";
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(278, 152);
			this.okButton.Name = "okButton";
			this.okButton.TabIndex = 7;
			this.okButton.Text = "OK";
			// 
			// projectLocationLabel
			// 
			this.projectLocationLabel.Location = new System.Drawing.Point(8, 98);
			this.projectLocationLabel.Name = "projectLocationLabel";
			this.projectLocationLabel.Size = new System.Drawing.Size(88, 20);
			this.projectLocationLabel.TabIndex = 8;
			this.projectLocationLabel.Text = "Project Directory";
			this.projectLocationLabel.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// projectLocationTextBox
			// 
			this.projectLocationTextBox.Location = new System.Drawing.Point(104, 98);
			this.projectLocationTextBox.Name = "projectLocationTextBox";
			this.projectLocationTextBox.Size = new System.Drawing.Size(216, 20);
			this.projectLocationTextBox.TabIndex = 9;
			this.projectLocationTextBox.Text = "";
			// 
			// browseButton
			// 
			this.browseButton.Location = new System.Drawing.Point(328, 98);
			this.browseButton.Name = "browseButton";
			this.browseButton.Size = new System.Drawing.Size(80, 24);
			this.browseButton.TabIndex = 10;
			this.browseButton.Text = "Browse";
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.authorLabel,
																					this.projectNameTextBox,
																					this.companyLabel,
																					this.projectNameLabel,
																					this.projectLocationLabel,
																					this.companyTextBox,
																					this.browseButton,
																					this.authorTextBox,
																					this.projectLocationTextBox});
			this.groupBox1.Location = new System.Drawing.Point(8, 8);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(432, 134);
			this.groupBox1.TabIndex = 11;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "Project Information";
			// 
			// NewProjectForm
			// 
			this.AcceptButton = this.okButton;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.cancelButton;
			this.ClientSize = new System.Drawing.Size(448, 181);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.groupBox1,
																		  this.okButton,
																		  this.cancelButton});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "NewProjectForm";
			this.ShowInTaskbar = false;
			this.Text = "New Project";
			this.groupBox1.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

	}
}
