using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime;
using System.Runtime.Serialization;

using jvms.system;
using jvms.presentation.drawablecomponent;
using jvms.presentation.relationships;

namespace jvms.presentation
{
	[Serializable]
	/// <summary>
	/// Represents the platform assembly view.
	/// </summary>
	public class PlatformAssemblyView : View
	{
		private bool draggingRelationship = false;

		/// <summary>
		/// Creates a new PlatformAssemblyView.
		/// </summary>
		public PlatformAssemblyView()
		{
		}

		/// <summary>
		/// Creates a new ApplicationAssemblyView.
		/// </summary>
		protected PlatformAssemblyView( SerializationInfo info, StreamingContext context)  :
			base( info, context )
		{
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public override void GetObjectData( SerializationInfo info, StreamingContext context )
		{
			base.GetObjectData( info, context );
		}

		/// <summary>
		/// From View.
		/// </summary>
		protected override AddComponent provideAddComponentDelegate( String componentType )
		{
			AddComponent retVal = null;
			if ( componentType.Equals( "deviceconfig" ) )
			{
				retVal = new AddComponent( addDeviceConfig );
			}
			else if ( componentType.Equals( "instance" ) )
			{
				retVal = new AddComponent( addInstance );
			}
			else if ( componentType.Equals( "properties" ) )
			{
				retVal = new AddComponent( addProperties );
			}
			else
			{
				throw new ArgumentException(
					"Value must be one of 'deviceconfig', 'instance', or 'properties'", "componentType" );
			}
			return retVal;
		}

		/// <summary>
		/// Called when the view should paint its components.
		/// </summary>
		/// <param name="clipRect">The area in which the view is painted.</param>
		/// <param name="g">The graphics context to use.</param>
		protected override void paintComponentView( Rectangle clipRect, Graphics g )
		{
		}

		/// <summary>
		/// Called when the view receives a mouse down event.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected override void viewMouseDown( Object source, MouseEventArgs e )
		{
			RelationshipManager rm = RelationshipManager.Instance;
			Point eventPoint = new Point( e.X, e.Y );
			IRelationship selected = null;
			draggingRelationship = false;

			foreach ( DrawableComponent dc in DrawableComponents )
			{
				if ( dc.isOnComponent( e.X, e.Y ) )
				{
					return;
				}
			}

			ArrayList selectedRelationships = new ArrayList();
			foreach ( IRelationship r in rm.Relationships )
			{
				if ( r.Tag.Equals( ToString() ) && r.isIntersection( eventPoint ) )
				{
					selectedRelationships.Add( r );
				}
			}

			if ( selectedRelationships.Count > 1 )
			{
				if ( e.Button == MouseButtons.Right )
				{
					selected = (IRelationship)selectedRelationships[0];
				}
				else
				{
					SelectRelationshipDialog selectDialog = new SelectRelationshipDialog();
					selectDialog.Relationships = selectedRelationships;
					selected = selectDialog.showSelectionDialog();
					rm.SelectedRelationship = selected;
					return;
				}
			}
			else if ( selectedRelationships.Count == 1 )
			{
				selected = (IRelationship)selectedRelationships[0];
			}
			else
			{
				selected = null;
			}

			rm.SelectedRelationship = selected;

			if ( selected != null )
			{
				if ( e.Button == MouseButtons.Right )
				{
					rm.showRelationshipContextMenu( eventPoint );
				}
				else
				{
					draggingRelationship = selected.isDragging( eventPoint );
				}
			}
		}

		/// <summary>
		/// Called when the view receives a mouse move event.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected override void viewMouseMove( Object source, MouseEventArgs e )
		{
			if ( draggingRelationship )
			{
				RelationshipManager rm = RelationshipManager.Instance;
				double x = Math.Round( (double)e.X / gridSize ) * gridSize;
				double y = Math.Round( (double)e.Y / gridSize ) * gridSize;
				rm.SelectedRelationship.dragNotify( new Point( (int)x, (int)y ) );
				rm.SelectedRelationship.update();
			}
		}

		/// <summary>
		/// Called when the view receives a mouse up event.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected override void viewMouseUp( Object source, MouseEventArgs e )
		{
			RelationshipManager.Instance.viewClicked( source, e );
			draggingRelationship = false;
		}

		/// <summary>
		/// Adds a Device Configuration component to the view.
		/// </summary>		
		private DrawableComponent addDeviceConfig()
		{
			return new DeviceConfigurationDrawableComponent();
		}

		/// <summary>
		/// Adds an Instance component the view.
		/// </summary>
		private DrawableComponent addInstance()
		{
			DrawableComponent retval = null;
			NewDeviceConfigurationInstanceForm newInstanceForm = 
				new NewDeviceConfigurationInstanceForm();
			newInstanceForm.Location = Cursor.Position;
			newInstanceForm.ShowDialog();
			if( newInstanceForm.SelectedComponent != null )
			{
				retval = new
					DeviceConfigurationComponentInstantiationDrawableComponent( 
						newInstanceForm.SelectedComponent );
			}
			return retval;
		}

		/// <summary>
		/// Adds a properties component to the view.
		/// </summary>
		private DrawableComponent addProperties()
		{
			return new PropertiesDrawableComponent();		
		}

		/// <summary>
		/// Returns a string representation of this object.
		/// </summary>
		/// <returns>A String.</returns>
		public override String ToString()
		{
			return "Platform Assembly";
		}
	}
}
