using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime;
using System.Runtime.Serialization;

using jvms.system;
using jvms.presentation.drawablecomponent;
using jvms.presentation.relationships;

namespace jvms.presentation
{
	[Serializable]
	/// <summary>
	/// Represents the platform component view.
	/// </summary>
	public class PlatformComponentView : View
	{
		/// <summary>
		/// Creates a new PlatformComponentView.
		/// </summary>
		public PlatformComponentView()
		{
		}

		/// <summary>
		/// Creates a new ApplicationAssemblyView.
		/// </summary>
		protected PlatformComponentView(SerializationInfo info, StreamingContext context) :
			base( info, context )
		{
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData( info, context );
		}

		/// <summary>
		/// From View.
		/// </summary>
		protected override AddComponent provideAddComponentDelegate( String componentType )
		{
			AddComponent retVal = null;
			if ( componentType.Equals( "cfdevice" ) )
			{
				retVal = new AddComponent( addCFDevice );
			}
			else if ( componentType.Equals( "nonscacomponent" ) )
			{
				retVal = new AddComponent( addNonScaComponent );
			}
			else if ( componentType.Equals( "cffilesystem" ) )
			{
				retVal = new AddComponent( addCFFilesystem );
			}
			else if ( componentType.Equals( "devicemanager" ) )
			{
				retVal = new AddComponent( addDeviceManager );
			}
			else if ( componentType.Equals( "cflogger" ) )
			{
				retVal = new AddComponent( addCFLogger );
			}
			else if ( componentType.Equals( "properties" ) )
			{
				retVal = new AddComponent( addProperties );
			}
			else
			{
				throw new ArgumentException(
					"Value must be one of 'cfdevice', 'nonscacomponent', 'cffilesystem', 'devicemanager', 'cflogger', or 'properties'", "componentType" );
			}
			return retVal;
		}

		/// <summary>
		/// Adds a CF Device component to the view.
		/// </summary>
		private DrawableComponent addCFDevice()
		{
			return new SCAComponentDrawableComponent( "device" );			
		}

		/// <summary>
		/// Adds a Non-SCA Component to the view.
		/// </summary>
		private DrawableComponent addNonScaComponent()
		{
			return new NonSCAComponentDrawableComponent();
		}

		/// <summary>
		/// Adds a CF Filesystem component to the view.
		/// </summary>
		private DrawableComponent addCFFilesystem()
		{
			return new SCAComponentDrawableComponent( "filesystem" );			
		}

		/// <summary>
		/// Adds a Device Manager component to the view.
		/// </summary>
		private DrawableComponent addDeviceManager()
		{
			return new SCAComponentDrawableComponent( "devicemanager" );
		}

		/// <summary>
		/// Adds a CF Logger component to the view.
		/// </summary>
		private DrawableComponent addCFLogger()
		{
			return new SCAComponentDrawableComponent( "log" );
		}

		/// <summary>
		/// Adds a properties component to the view.
		/// </summary>
		private DrawableComponent addProperties()
		{
			return new PropertiesDrawableComponent();
		}

		/// <summary>
		/// Called when the view should paint its components.
		/// </summary>
		/// <param name="clipRect">The area in which the view is painted.</param>
		/// <param name="g">The graphics context to use.</param>
		protected override void paintComponentView( Rectangle clipRect, Graphics g )
		{
		}

		/// <summary>
		/// Called when the view receives a mouse down event.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected override void viewMouseDown( Object source, MouseEventArgs e )
		{
			RelationshipManager rm = RelationshipManager.Instance;
			Point eventPoint = new Point( e.X, e.Y );
			IRelationship selected = null;

			foreach ( DrawableComponent dc in DrawableComponents )
			{
				if ( dc.isOnComponent( e.X, e.Y ) )
				{
					return;
				}
			}

			foreach ( IRelationship r in rm.Relationships )
			{
				if ( r.Tag.Equals( ToString() ) && 
					r.isIntersection( eventPoint ))
				{
					selected = r;												
					break;
				}
			}
			rm.SelectedRelationship = selected;
			if( selected != null )
			{
				if ( e.Button == MouseButtons.Right )
				{
					rm.showRelationshipContextMenu( eventPoint );
				}
			}
		}

		/// <summary>
		/// Called when the view receives a mouse up event.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected override void viewMouseUp( Object source, MouseEventArgs e )
		{
			RelationshipManager.Instance.viewClicked( source, e );
		}

		/// <summary>
		/// Returns a string representation of this object.
		/// </summary>
		/// <returns>A String.</returns>
		public override String ToString()
		{
			return "Platform Component";
		}
	}
}
