using System;
using System.Collections;
using System.Drawing;
using System.Resources;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using jvms.datamodel;
using jvms.datamodel.dcd;
using jvms.datamodel.sad;
using jvms.datamodel.scd;
using jvms.datamodel.common;
using jvms.presentation.drawablecomponent;
using jvms.presentation.relationships;
using jvms.project;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Responsible for creating relationships.
	/// </summary>
	[Serializable]
	public class RelationshipManager
	{
		private static RelationshipManager instance = new RelationshipManager();

		public const int IDLE = 0;
		public const int SELECT_PROVIDES_PORT = 1;
		public const int SELECT_USES_PORT = 2;
		public const int SELECT_EITHER_PORT = 3;
		public const int SELECT_CONTAINER = 4;
		public const int SELECT_COMPONENT = 5;
		public const int SELECT_INSTANCE = 6;
		public const int SELECT_PROPERTIES = 7;

		private int mode = IDLE;
		private ArrayList relationships = new ArrayList();
		private IRelationship workingRelationship;
		private IRelationship selectedRelationship;
		private Cursor selectProvidesCursor;
		private Cursor selectUsesCursor;
		private Cursor selectPortCursor;
		private Cursor selectComponentCursor;
		private Cursor selectInstanceCursor;
		private Cursor selectPropsCursor;
		private Cursor selectContainerCursor;

		/// <summary>Gets the single instance of RelationshipManager.</summary>
		/// <value>The single RelationshipManager instance.</value>
		public static RelationshipManager Instance
		{
			get
			{
				return instance;
			}
		}

		/// <summary>Gets or sets the current mode.</summary>
		/// <value>The current mode.</value>
		public int Mode
		{
			get
			{
				return mode;
			}

			set
			{
				mode = value;
			}
		}

		/// <summary>Gets or sets a list of relationships.</summary>
		/// <value>An ArrayList of relationships.</value>
		public ArrayList Relationships
		{
			get
			{
				return relationships;
			}

			set
			{
				relationships = value;
			}
		}

		/// <summary>Gets or sets the selected relationship.</summary>
		/// <value>The selected relationship.</value>
		public IRelationship SelectedRelationship
		{
			get
			{
				return selectedRelationship;
			}

			set
			{
				selectedRelationship = value;

				foreach ( IRelationship r in relationships )
				{
					r.Selected = false;
				}

				if ( selectedRelationship != null )
				{
					selectedRelationship.Selected = true;
					JVMSForm jvmsForm = JVMSMain.JvmsForm;
					AttributeSheet sheet =
						AttributeSheetFactory.Instance.createAttributeSheet( selectedRelationship );
					jvmsForm.CurrentAttributeSheetControl.AttributeSheet = sheet;
					jvmsForm.CurrentView.Invalidate( false );
				}
			}
		}

		/// <summary>
		/// Creates a new RelationshipManager.
		/// </summary>
		private RelationshipManager()
		{
			selectProvidesCursor = new Cursor( new IntPtr( LoadCursorFromFile( JVMSMain.ResourcesDir + "SelectProvidesCursor.cur" ) ) );
			selectUsesCursor = new Cursor( new IntPtr( LoadCursorFromFile( JVMSMain.ResourcesDir + "SelectUsesCursor.cur" ) ) );
			selectPortCursor = new Cursor( new IntPtr( LoadCursorFromFile( JVMSMain.ResourcesDir + "SelectPortCursor.cur" ) ) );
			selectComponentCursor = new Cursor( new IntPtr( LoadCursorFromFile( JVMSMain.ResourcesDir + "SelectComponentCursor.cur" ) ) );
			selectInstanceCursor = new Cursor( new IntPtr( LoadCursorFromFile( JVMSMain.ResourcesDir + "SelectInstanceCursor.cur" ) ) );
			selectPropsCursor = new Cursor( new IntPtr( LoadCursorFromFile( JVMSMain.ResourcesDir + "SelectPropsCursor.cur" ) ) );
			selectContainerCursor = new Cursor( new IntPtr( LoadCursorFromFile( JVMSMain.ResourcesDir + "SelectContainerCursor.cur" ) ) );
		}

		/// <summary>
		/// Clears all relationships.
		/// </summary>
		public void clearRelationships()
		{
			relationships.Clear();
		}

		/// <summary>
		/// Notifies all relationships that they have been scrolled.
		/// </summary>
		/// <param name="direction">The direction.</param>
		/// <param name="amount">The amount.</param>
		public void notifyViewScrolled( Orientation direction, int amount )
		{
			foreach ( IRelationship r in relationships )
			{
				if( r.Tag.Equals( JVMSMain.JvmsForm.CurrentView.ToString() ) )
				{
					r.scrollNotify( direction, amount );
					r.update();
				}
			}
		}

		/// <summary>
		/// Initiates creating a relationship
		/// </summary>
		/// <param name="typename">The name of the type of relationship to create.</param>
		public void beginCreateRelationship( String typeName )
		{
			JVMSForm jvmsForm = JVMSMain.JvmsForm;
			workingRelationship = (IRelationship)Assembly.GetExecutingAssembly().CreateInstance( typeName );

			if ( workingRelationship is PortConnectionRelationship )
			{
				mode = SELECT_USES_PORT;
				StatusBarManager.Instance.StatusText = "Select Uses Port - Right-click to cancel";
				jvmsForm.CurrentView.Cursor = selectUsesCursor;
			}
			else if ( workingRelationship is SinglePortConnectionRelationship )
			{
				mode = SELECT_EITHER_PORT;
				StatusBarManager.Instance.StatusText = "Select a port - Right-click to cancel";
				jvmsForm.CurrentView.Cursor = selectPortCursor;
			}
			else if ( workingRelationship is DeviceConfigRelationship || workingRelationship is ApplicationRelationship )
			{
				mode = SELECT_CONTAINER;
				StatusBarManager.Instance.StatusText = "Select a container for the component - Right-click to cancel";
				jvmsForm.CurrentView.Cursor = selectContainerCursor;
			}
			else if ( workingRelationship is PropertiesRelationship )
			{
				mode = SELECT_PROPERTIES;
				StatusBarManager.Instance.StatusText = "Select a properties component - Right-click to cancel";
				jvmsForm.CurrentView.Cursor = selectPropsCursor;
			}
		}

		/// <summary>
		/// Shows a context menu for relationships at the specified point.
		/// </summary>
		/// <param name="p">The point at which to show the menu.</param>
		public void showRelationshipContextMenu( Point p )
		{
			ContextMenu menu = new ContextMenu();
			
			MenuItem deleteItem = new MenuItem();
			deleteItem.Text = "Delete";
			deleteItem.Click += new EventHandler( deleteRelationshipMenuItemSelected );

			menu.MenuItems.Add( deleteItem );

			menu.Show( JVMSMain.JvmsForm.CurrentView, p );
		}

		/// <summary>
		/// Draws all relationships.
		/// </summary>
		/// <param name="clientRect">The area in which to draw.</param>
		/// <param name="g">The graphics context to use.</param>
		public void drawRelationships( Rectangle clientRect, Graphics g )
		{
			foreach ( IRelationship r in relationships )
			{
				if ( r.Tag.Equals( JVMSMain.JvmsForm.CurrentView.ToString() ) )
				{
					r.drawRelationship( clientRect, g );
				}
			}
		}

		/// <summary>
		/// Called when a provides port has been selected in the view.
		/// </summary>
		/// <param name="component">The drawable component which contains the port.</param>
		/// <param name="port">The data model object representing the port.</param>
		public void providesPortSelected( BaseSCAComponentDrawableComponent component, ComponentPortEndPoint port )
		{
			if ( mode == SELECT_PROVIDES_PORT && workingRelationship != null )
			{
				JVMSForm jvmsForm = JVMSMain.JvmsForm;
				PortConnectionRelationship portRelationship = (PortConnectionRelationship)workingRelationship;
				portRelationship.ProvidesComponent = component;
				portRelationship.ProvidesPort = port;
				portRelationship.initRelationship();

				( (IRelationship)portRelationship ).Tag = jvmsForm.CurrentView.ToString();
				SelectedRelationship = (IRelationship)portRelationship;
				relationships.Add( portRelationship );
				workingRelationship = null;

				mode = IDLE;
				jvmsForm.CurrentView.Cursor = Cursors.Default;
				jvmsForm.CurrentView.Invalidate( false );
				StatusBarManager.Instance.StatusText = "";
				ProjectManager.Instance.ChangedFlag = true;
			}
			else if ( mode == SELECT_EITHER_PORT && workingRelationship != null )
			{	
				JVMSForm jvmsForm = JVMSMain.JvmsForm;
				SinglePortConnectionRelationship singlePortRelationship  = (SinglePortConnectionRelationship)workingRelationship;
				singlePortRelationship.Component = component;
				singlePortRelationship.Port = port;
				singlePortRelationship.initRelationship();

				( (IRelationship)singlePortRelationship ).Tag = jvmsForm.CurrentView.ToString();
				SelectedRelationship = (IRelationship)singlePortRelationship;
				relationships.Add( singlePortRelationship );
				workingRelationship = null;

				mode = IDLE;
				jvmsForm.CurrentView.Cursor = Cursors.Default;
				StatusBarManager.Instance.StatusText = "";
				ProjectManager.Instance.ChangedFlag = true;
			}
		}

		/// <summary>
		/// Called when a uses port has been selected in the view.
		/// </summary>
		/// <param name="component">The drawable component which contains the port.</param>
		/// <param name="port">The data model object represeting the port.</param>
		public void usesPortSelected( BaseSCAComponentDrawableComponent component, ComponentPortEndPoint port )
		{
			if ( mode == SELECT_USES_PORT && workingRelationship != null )
			{
				PortConnectionRelationship portRelationship = (PortConnectionRelationship)workingRelationship;
				portRelationship.UsesComponent = component;
				portRelationship.UsesPort = port;

				mode = SELECT_PROVIDES_PORT;
				JVMSForm jvmsForm = JVMSMain.JvmsForm;
				jvmsForm.CurrentView.Cursor = selectProvidesCursor;
				StatusBarManager.Instance.StatusText = "Select Provides Port - Right-click to cancel";
			}
			else if ( mode == SELECT_EITHER_PORT && workingRelationship != null )
			{
				JVMSForm jvmsForm = JVMSMain.JvmsForm;
				SinglePortConnectionRelationship singlePortRelationship  = (SinglePortConnectionRelationship)workingRelationship;
				singlePortRelationship.Component = component;
				singlePortRelationship.Port = port;
				singlePortRelationship.initRelationship();

				( (IRelationship)singlePortRelationship ).Tag = jvmsForm.CurrentView.ToString();
				SelectedRelationship = (IRelationship)singlePortRelationship;
				relationships.Add( singlePortRelationship );
				workingRelationship = null;

				mode = IDLE;
				jvmsForm.CurrentView.Cursor = Cursors.Default;
				StatusBarManager.Instance.StatusText = "";
				ProjectManager.Instance.ChangedFlag = true;
			}
		}

		/// <summary>
		/// Called when a properties component is selected in the view.
		/// </summary>
		/// <param name="props">The selected component.</param>
		public void propertiesSelected( PropertiesDrawableComponent props )
		{
			if ( mode == SELECT_PROPERTIES && workingRelationship != null )
			{
				PropertiesRelationship r = (PropertiesRelationship)workingRelationship;
				r.Properties = props;
				mode = SELECT_COMPONENT;
				StatusBarManager.Instance.StatusText = "Select Component - Right-click to cancel";
				JVMSMain.JvmsForm.CurrentView.Cursor = selectComponentCursor;
			}
		}

		/// <summary>
		/// Called when an SCA component is selected in the view.
		/// </summary>
		/// <param name="props">The selected component.</param>
		public void scaComponentSelected( SCAComponentDrawableComponent component )
		{
			if ( mode == SELECT_COMPONENT && workingRelationship != null )
			{
				if ( workingRelationship is PropertiesRelationship )
				{
					JVMSForm jvmsForm = JVMSMain.JvmsForm;
					PropertiesRelationship relationship = (PropertiesRelationship)workingRelationship;
					relationship.ScaComponent = component;
					component.Component.Properties = relationship.Properties.Properties;
		
					( (IRelationship)workingRelationship ).Tag = jvmsForm.CurrentView.ToString();
					SelectedRelationship = (IRelationship)workingRelationship;
					relationships.Add( workingRelationship );
					workingRelationship = null;

					jvmsForm.CurrentView.Cursor = Cursors.Default;
					StatusBarManager.Instance.StatusText = "";
					ProjectManager.Instance.ChangedFlag = true;
					mode = IDLE;
				}
			}
		}

		/// <summary>
		/// Called when a Non-SCA component is selected in the view.
		/// </summary>
		/// <param name="props">The selected component.</param>
		public void nonScaComponentSelected( NonSCAComponentDrawableComponent component )
		{
			if ( mode == SELECT_COMPONENT && workingRelationship != null )
			{
				if ( workingRelationship is PropertiesRelationship )
				{
					JVMSForm jvmsForm = JVMSMain.JvmsForm;
					PropertiesRelationship relationship = (PropertiesRelationship)workingRelationship;
					relationship.NonScaComponent = component;
					relationship.NonScaComponent.NonSCAComponent.Properties = relationship.Properties.Properties;
		
					( (IRelationship)workingRelationship ).Tag = jvmsForm.CurrentView.ToString();
					SelectedRelationship = (IRelationship)workingRelationship;
					relationships.Add( workingRelationship );
					workingRelationship = null;

					jvmsForm.CurrentView.Cursor = Cursors.Default;
					StatusBarManager.Instance.StatusText = "";
					ProjectManager.Instance.ChangedFlag = true;
					mode = IDLE;
				}
			}
		}

		/// <summary>
		/// Called when an application component is selected in the view.
		/// </summary>
		/// <param name="props">The selected component.</param>
		public void applicationSelected( ApplicationDrawableComponent app )
		{
			if ( mode == SELECT_CONTAINER && workingRelationship != null )
			{
				if ( workingRelationship is ApplicationRelationship )
				{
					( (ApplicationRelationship)workingRelationship ).Application = app;
					mode = SELECT_INSTANCE;
					StatusBarManager.Instance.StatusText = "Select Instance - Right-click to cancel";
					JVMSMain.JvmsForm.CurrentView.Cursor = selectInstanceCursor;
				}
			}
		}

		/// <summary>
		/// Called when a device configuration component is selected in the view.
		/// </summary>
		/// <param name="props">The selected component.</param>
		public void deviceConfigSelected( DeviceConfigurationDrawableComponent deviceConfig )
		{
			if ( mode == SELECT_CONTAINER && workingRelationship != null )
			{
				if ( workingRelationship is DeviceConfigRelationship )
				{
					( (DeviceConfigRelationship)workingRelationship ).DeviceConfiguration = deviceConfig;
					mode = SELECT_INSTANCE;
					StatusBarManager.Instance.StatusText = "Select Instance - Right-click to cancel";
					JVMSMain.JvmsForm.CurrentView.Cursor = selectInstanceCursor;
				}
			}
		}

		/// <summary>
		/// Called when a platform component instance is selected in the view.
		/// </summary>
		/// <param name="props">The selected component.</param>
		public void platformInstanceSelected( DeviceConfigurationComponentInstantiationDrawableComponent component )
		{
			if ( mode == SELECT_INSTANCE && workingRelationship != null )
			{
				if ( workingRelationship is DeviceConfigRelationship )
				{
					JVMSForm jvmsForm = JVMSMain.JvmsForm;
					DeviceConfigRelationship dcRelationship = 
						(DeviceConfigRelationship)workingRelationship;
					dcRelationship.Component = component;
					DeviceConfiguration dc = 
						((DeviceConfiguration)dcRelationship.DeviceConfiguration.getWrappedObject());
					dc.Partitioning.Add( component.getWrappedObject() );
					
					( (IRelationship)workingRelationship ).Tag = jvmsForm.CurrentView.ToString();
					SelectedRelationship = (IRelationship)workingRelationship;
					relationships.Add( workingRelationship );
					workingRelationship = null;

					jvmsForm.CurrentView.Cursor = Cursors.Default;
					StatusBarManager.Instance.StatusText = "";
					ProjectManager.Instance.ChangedFlag = true;
					mode = IDLE;
				}
			}
		}

		/// <summary>
		/// Called when an applicaton component instance is selected in the view.
		/// </summary>
		/// <param name="props">The selected component.</param>
		public void applicationInstanceSelected( ApplicationComponentInstantiationDrawableComponent component )
		{
			if ( mode == SELECT_INSTANCE && workingRelationship != null )
			{
				if ( workingRelationship is ApplicationRelationship )
				{
					JVMSForm jvmsForm = JVMSMain.JvmsForm;
					ApplicationRelationship appRelationship = (ApplicationRelationship)workingRelationship;
					appRelationship.Component = component;
					jvms.datamodel.sad.Application app = (jvms.datamodel.sad.Application)appRelationship.Application.getWrappedObject();
					app.ComponentPlacements.Add( component.getWrappedObject() );
					
					( (IRelationship)workingRelationship ).Tag = jvmsForm.CurrentView.ToString();
					SelectedRelationship = (IRelationship)workingRelationship;
					relationships.Add( workingRelationship );
					workingRelationship = null;

					jvmsForm.CurrentView.Cursor = Cursors.Default;
					StatusBarManager.Instance.StatusText = "";
					ProjectManager.Instance.ChangedFlag = true;
					mode = IDLE;
				}
			}
		}

		/// <summary>
		/// Called when the view is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void viewClicked( Object source, MouseEventArgs e )
		{
			if ( mode != IDLE && e.Button == MouseButtons.Right )
			{
				mode = IDLE;
				JVMSMain.JvmsForm.CurrentView.Cursor = Cursors.Default;
				StatusBarManager.Instance.StatusText = "";
			}
		}

		/// <summary>
		/// Called when a key is pressed on the view.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void viewKeyDown( Object source, KeyEventArgs e )
		{
			if ( mode != IDLE && e.KeyCode == Keys.Escape )
			{
				mode = IDLE;
				JVMSMain.JvmsForm.CurrentView.Cursor = Cursors.Default;
				StatusBarManager.Instance.StatusText = "";
			}
		}

		/// <summary>
		/// This will take a list of ports and remove all relationships 
		/// that use this port as an endpoint.
		/// </summary>
		/// <param name="ports">A list of ComponentPortEndPoints.</param>
		public void removeRelationsWithPorts( IList ports )
		{
			IList relationshipsCopy = new ArrayList( relationships );
			foreach ( Object obj in relationshipsCopy )
			{
				if ( obj is SinglePortConnectionRelationship )
				{
					SinglePortConnectionRelationship connection = (SinglePortConnectionRelationship)obj;
					if ( ports.Contains( connection.Port ) )
					{
						removeRelationship( (IRelationship)connection );
					}
				}
				else if ( obj is PortConnectionRelationship )
				{
					PortConnectionRelationship connection = (PortConnectionRelationship)obj;
					if ( ports.Contains( connection.UsesPort ) ||
						ports.Contains( connection.ProvidesPort ) )
					{
						removeRelationship( (IRelationship)connection );
					}
				}
			}
		}

		/// <summary>
		/// This will remove all relationships to the drawable component.
		/// </summary>
		/// <param name="component"></param>
		public void removeRelationshipsToDrawableComponent( DrawableComponent component )
		{
			IList relationshipsToDelete = getRelationshipsConnectingTo( component );
			foreach ( IRelationship relationship in relationshipsToDelete )
			{
				removeRelationship( relationship );
			}
		}

		/// <summary>
		/// Updates relationships that are connected to the specified component.
		/// </summary>
		/// <param name="component">The desired component.</param>
		public void invalidateRelationshipsConnectingTo( DrawableComponent component )
		{
			IList relationshipsToDelete = getRelationshipsConnectingTo( component );
			foreach ( IRelationship relationship in relationshipsToDelete )
			{
				relationship.update();
			}
		}

		/// <summary>
		/// This will get all relationships that are connected to a drawable component.
		/// </summary>
		/// <param name="component">The component to that relationships are connected to.</param>
		/// <returns>A list of all relationships that are connected to that component.</returns>
		private IList getRelationshipsConnectingTo( DrawableComponent component )
		{
			IList connectingRelationships = new ArrayList();
			foreach ( IRelationship relationship in relationships )
			{
				if ( relationship is ApplicationRelationship )
				{
					ApplicationRelationship appRelationship = (ApplicationRelationship)relationship;
					if ( appRelationship.Application == component ||
						 appRelationship.Component == component )
					{
						connectingRelationships.Add( relationship );
					}
				}
				else if ( relationship is DeviceConfigRelationship )
				{
					DeviceConfigRelationship dcRelationship = (DeviceConfigRelationship)relationship;
					if ( dcRelationship.DeviceConfiguration == component ||
						dcRelationship.Component == component )
					{
						connectingRelationships.Add( relationship );
					}
				}
				else if ( relationship is SinglePortConnectionRelationship )
				{
					SinglePortConnectionRelationship portRelationship = (SinglePortConnectionRelationship)relationship;
					if ( portRelationship.Component == component )
					{
						connectingRelationships.Add( relationship );
					}
				}
				else if ( relationship is PortConnectionRelationship )
				{
					PortConnectionRelationship portRelationship = (PortConnectionRelationship)relationship;
					if ( portRelationship.ProvidesComponent == component ||
						portRelationship.UsesComponent == component )
					{
						connectingRelationships.Add( relationship );
					}
				}
				else if ( relationship is PropertiesRelationship )
				{
					PropertiesRelationship propRelationship = (PropertiesRelationship)relationship;
					if ( propRelationship.Properties == component ||
						propRelationship.ScaComponent == component )
					{
						connectingRelationships.Add( relationship );
					}
				}
			}
			return connectingRelationships;
		}

		/// <summary>
		/// This will remove a relationship and all references to it.
		/// </summary>
		/// <param name="relationship">The relationship to remove.</param>
		public void removeRelationship( IRelationship relationship )
		{
			relationships.Remove( relationship );
			relationship.delete();

			if ( selectedRelationship == relationship )
			{
				selectedRelationship = null;
			}
			if ( workingRelationship == relationship )
			{
				workingRelationship = null;
			}
		}

		/// <summary>
		/// This will get all of the connections that have one of the instantiations
		/// as an endpoint.
		/// </summary>
		/// <param name="instantiations">A list of instantiations to check against.</param>
		/// <returns>A list of all connections for the instantiations.</returns>
		public TypedArrayList getConnectionsForInstantiations( IList instantiations )
		{
			TypedArrayList connections = new TypedArrayList(typeof(ConnectInterface));

			foreach ( IRelationship relationship in relationships )
			{
				if ( relationship is IPortRelationship )
				{
					IPortRelationship portRelationship = 
						(IPortRelationship)relationship;
					ConnectInterface connection = portRelationship.getConnectInterface();
					bool connectsToInstance = false;					
					if( connection.UsesPort is ComponentInstantiationPortConnection )
					{
						ComponentInstantiationPortConnection portInstance = 
							(ComponentInstantiationPortConnection)connection.UsesPort;
						foreach( ComponentInstantiation instance in instantiations )
						{
							if( instance == portInstance.Instance )
							{
								connectsToInstance = true;
							}
						}
					}
					else if( connection.UsesPort is ComponentSupportedInterface )
					{
						ComponentSupportedInterface portInstance = 
							(ComponentSupportedInterface)connection.UsesPort;
						foreach( ComponentInstantiation instance in instantiations )
						{
							if( instance == portInstance.Instance )
							{
								connectsToInstance = true;
							}
						}
					}
					if( connection.ProvidesPort is ComponentInstantiationPortConnection )
					{
						ComponentInstantiationPortConnection portInstance = 
							(ComponentInstantiationPortConnection)connection.ProvidesPort;
						foreach( ComponentInstantiation instance in instantiations )
						{
							if( instance == portInstance.Instance )
							{
								connectsToInstance = true;
							}
						}
					}
					else if( connection.ProvidesPort is ComponentSupportedInterface )
					{
						ComponentSupportedInterface portInstance = 
							(ComponentSupportedInterface)connection.ProvidesPort;
						foreach( ComponentInstantiation instance in instantiations )
						{
							if( instance == portInstance.Instance )
							{
								connectsToInstance = true;
							}
						}
					}
					if( connectsToInstance && !connections.Contains( connection ) )
					{
						connections.Add( connection );
					}
				}
			}
			return connections;
		}

		private ComponentInstantiation getInstantiationFromConnection( PortConnection connection )
		{
			ComponentInstantiation retval = null;

			if ( connection is ComponentInstantiationPortConnection )
			{
				retval = ((ComponentInstantiationPortConnection)connection).Instance;
			}
			else if ( connection is ComponentSupportedInterface )
			{
				retval = ((ComponentSupportedInterface)connection).Instance;
			}

			return retval;
		}

		/// <summary>
		/// Deletes the currently selected relationship.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void deleteRelationshipMenuItemSelected( Object source, EventArgs e )
		{
			removeRelationship( selectedRelationship );
			SelectedRelationship = null;
			ProjectManager.Instance.ChangedFlag = true;
			JVMSMain.JvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
			JVMSMain.JvmsForm.CurrentView.Invalidate( false );
		}

		[DllImport( "user32.dll" )]
		private static extern int LoadCursorFromFile( String FileName );
	}
}
