using System;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;

using Microsoft.Win32;

using jvms.datamodel.dcd;
using jvms.datamodel.prf;
using jvms.datamodel.sad;
using jvms.datamodel.spd;
using jvms.presentation;
using jvms.system;

namespace jvms.presentation
{
	/// <summary>
	/// Shows a splash screen while loading some JVMS resources.
	/// </summary>
	public class SplashScreen
	{
		private Panel splashPanel = new Panel();
		private Form splashForm = new Form();
		private Image logoImage;
		private String assemblyVersion = "";
		private String assemblyPath = JVMSMain.AssemblyDir;

		/// <summary>
		/// Creates a new splash screen.
		/// </summary>
		public SplashScreen()
		{
			Assembly jvmsAssembly = Assembly.GetExecutingAssembly();
			assemblyVersion = jvmsAssembly.GetName().Version.ToString();

			Stream stream = jvmsAssembly.GetManifestResourceStream( "jvms.resources.JVMSLogo.png" );
			logoImage = Image.FromStream( stream );
			stream.Close();
			splashPanel.Paint += new PaintEventHandler( paintSplashPanel );
		}

		/// <summary>
		/// Shows the splash screen.
		/// </summary>
		public void showSplashScreen()
		{
			splashForm.FormBorderStyle = FormBorderStyle.None;
			splashForm.ShowInTaskbar = false;
			splashForm.MaximizeBox = false;
			splashForm.MinimizeBox = false;
			splashForm.ControlBox = false;
			splashForm.StartPosition = FormStartPosition.CenterScreen;
			splashForm.Load += new EventHandler( splashScreenLoaded );

			splashPanel.Dock = DockStyle.Fill;
			splashForm.Controls.Add( splashPanel );
			splashForm.Size = new Size( 550, 300 );
			splashForm.ShowDialog();
		}

		/// <summary>
		/// Called when the splash panel should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void paintSplashPanel( Object source, PaintEventArgs e )
		{
			Graphics g = e.Graphics;
			Rectangle clientRect = splashForm.ClientRectangle;
			Font f = new Font( "Arial", 8.0f );

			String authorString = "The A-Team";
			String versionString = "JVMS Version: " + assemblyVersion;
			g.DrawImage( logoImage, clientRect );
			g.DrawString( authorString, f, Brushes.Black, new PointF( clientRect.Left + 2, clientRect.Bottom - 28 ) );
			g.DrawString( versionString, f, Brushes.Black, new PointF( clientRect.Left + 2, clientRect.Bottom - 15 ) );
		}

		/// <summary>
		/// Called when the splash screen has been loaded.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void splashScreenLoaded( Object source, EventArgs e )
		{
			Thread loadResourcesThread = new Thread( new ThreadStart( initResources ) );
			loadResourcesThread.Start();
		}

		/// <summary>
		/// Initializes various resources.
		/// </summary>
		private void initResources()
		{
			DateTime start = DateTime.Now;
			try
			{
				createFileAssociations();
			}
			catch ( Exception e )
			{
				Console.WriteLine( "Cannot create file associations: " + e.Message );
			}

			AttributeSheetFactory factory = AttributeSheetFactory.Instance;
			AttributeSheet sheet = factory.createAttributeSheet( new SCAComponent() );
			sheet = factory.createAttributeSheet( new NonSCAComponent() );
			sheet = factory.createAttributeSheet( new DeviceConfiguration() );
			sheet = factory.createAttributeSheet( new jvms.datamodel.sad.Application() );
			sheet = factory.createAttributeSheet( new Properties() );
			DateTime end = DateTime.Now;

			long waitTime = 2500 - ( ( end.Ticks - start.Ticks ) / 10000 );
			if ( waitTime > 0 )
			{
				Thread.Sleep( (int)waitTime );
			}

			splashForm.Close();
		}

		/// <summary>
		/// Creates file associations.
		/// </summary>
		private void createFileAssociations()
		{
			RegistryKey classesRoot = Registry.ClassesRoot;

			RegistryKey vendorKey = classesRoot.CreateSubKey( "ateam.jvms.1" );
			vendorKey.SetValue( "", "JVMS Project File" );
			
			RegistryKey iconKey = vendorKey.CreateSubKey( "DefaultIcon" );
			iconKey.SetValue( "", assemblyPath + "JVMSProject.ico" );
			RegistryKey shellKey = vendorKey.CreateSubKey( "shell" );
			RegistryKey openKey = shellKey.CreateSubKey( "open" );
			RegistryKey openCommandKey = openKey.CreateSubKey( "command" );
			openCommandKey.SetValue( "", "\"" + Assembly.GetExecutingAssembly().Location + "\" \"%1\"" );

			RegistryKey shortExtKey = classesRoot.CreateSubKey( ".jvp" );
			shortExtKey.SetValue( "", "ateam.jvms.1" );

			RegistryKey longExtKey = classesRoot.CreateSubKey( ".jvproj" );
			longExtKey.SetValue( "", "ateam.jvms.1" );

			SHChangeNotify(
				(uint)ShellChangeNotificationEvents.SHCNE_ASSOCCHANGED,
				(uint)ShellChangeNotificationFlags.SHCNF_DWORD,
				IntPtr.Zero, IntPtr.Zero);
		}

		/// <summary>
		/// Notifies the system of an event that an application has performed.
		/// </summary>
		[DllImport( "shell32.dll" )]
		public static extern void SHChangeNotify( UInt32 wEventId, UInt32 uFlags, IntPtr dwItem1, IntPtr dwItem2 );

		/// <summary>
		/// A list of shell event types.
		/// </summary>
		[Flags]
		public enum ShellChangeNotificationEvents : uint
		{
			SHCNE_RENAMEITEM          = 0x00000001,	// The name of a nonfolder item has changed.
			SHCNE_CREATE              = 0x00000002,	// A nonfolder item has been created.
			SHCNE_DELETE              = 0x00000004,	// A nonfolder item has been deleted.
			SHCNE_MKDIR               = 0x00000008,	// A folder has been created.
			SHCNE_RMDIR               = 0x00000010,	// A folder has been removed.
			SHCNE_MEDIAINSERTED       = 0x00000020,	// Storage media has been inserted into a drive.
			SHCNE_MEDIAREMOVED        = 0x00000040,	// Storage media has been removed from a drive.
			SHCNE_DRIVEREMOVED        = 0x00000080,	// A drive has been removed.
			SHCNE_DRIVEADD            = 0x00000100,	// A drive has been added.
			SHCNE_NETSHARE            = 0x00000200,	// A folder on the local computer is being shared via the network.
			SHCNE_NETUNSHARE          = 0x00000400,	// A folder on the local computer is no longer being shared
			SHCNE_ATTRIBUTES          = 0x00000800,	// The attributes of an item or folder have changed.
			SHCNE_UPDATEDIR           = 0x00001000,	// The contents of an existing folder have changed 
			SHCNE_UPDATEITEM          = 0x00002000,	// An existing nonfolder item has changed 
			SHCNE_SERVERDISCONNECT    = 0x00004000,	// The computer has disconnected from a server.
			SHCNE_UPDATEIMAGE         = 0x00008000,	// An image in the system image list has changed.
			SHCNE_DRIVEADDGUI         = 0x00010000,	// A drive has been added.
			SHCNE_RENAMEFOLDER        = 0x00020000,	// The name of a folder has changed.
			SHCNE_FREESPACE           = 0x00040000,	// The amount of free space on a drive has changed.
			SHCNE_EXTENDED_EVENT      = 0x04000000,	// Not currently used. 
			SHCNE_ASSOCCHANGED        = 0x08000000,	// A file type association has changed.
			SHCNE_DISKEVENTS          = 0x0002381F,	// Specifies a combination of all of the disk event identifiers. 
			SHCNE_GLOBALEVENTS        = 0x0C0581E0,	// Specifies a combination of all of the global event identifiers. 
			SHCNE_ALLEVENTS           = 0x7FFFFFFF,	// All events have occurred. 
			SHCNE_INTERRUPT           = 0x80000000	// The specified event occurred as a result of a system interrupt.
		}

		/// <summary>
		/// A list of shell event flags,
		/// </summary>
		public enum ShellChangeNotificationFlags
		{
			SHCNF_IDLIST		= 0x0000,	// dwItem1 and dwItem2 are the addresses of ITEMIDLIST structures
			SHCNF_PATHA			= 0x0001,	// dwItem1 and dwItem2 are the addresses of null-terminated strings
			SHCNF_PRINTERA		= 0x0002,	// dwItem1 and dwItem2 are the addresses of null-terminated strings
			SHCNF_DWORD			= 0x0003,	// The dwItem1 and dwItem2 parameters are DWORD values. 
			SHCNF_PATHW			= 0x0005,	// like SHCNF_PATHA but unicode string
			SHCNF_PRINTERW		= 0x0006,	// like SHCNF_PRINTERA but unicode string
			SHCNF_TYPE			= 0x00FF,	
			SHCNF_FLUSH			= 0x1000,	// The function should not return until the notification has been delivered
			SHCNF_FLUSHNOWAIT	= 0x2000	// The function should begin delivering notifications to all affected
		}
	}
}
