using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.Runtime;
using System.Runtime.Serialization;

using jvms.project;
using jvms.system;
using jvms.presentation.drawablecomponent;
using jvms.presentation.relationships;

namespace jvms.presentation
{
	[Serializable]
	/// <summary>
	/// Represents a base view component.
	/// </summary>
	public abstract class View : Panel, ISerializable
	{
		protected delegate DrawableComponent AddComponent();

		private bool adding = false;
		private AddComponent addComponent;

		private HScrollBar hScrollBar;
		private VScrollBar vScrollBar;
		private int lastHValue = 0;
		private int lastVValue = 0;

		protected float zoomFactor = 1.0f;
		protected int gridSize = 10;

		private IList drawableComponents = new ArrayList();

		/// <summary>Gets the factory by which the view is zoomed.</summary>
		/// <value>A float.</value>
		public float ZoomFactor
		{
			get
			{
				return zoomFactor;
			}
		}

		/// <summary>Gets the previous value of the vertical scroll bar.</summary>
		/// <value>An int.</value>
		public int LastVerticalScrollValue
		{
			get
			{
				return lastVValue;
			}
		}

		/// <summary>Gets the previous value of the horizontal scroll bar.</summary>
		/// <value>An int.</value>
		public int LastHorizontalScrollValue
		{
			get
			{
				return lastHValue;
			}
		}

		/// <summary>Gets a list of drawable components on the view.</summary>
		/// <value>An ArrayList.</value>
		public IList DrawableComponents
		{
			get
			{
				return drawableComponents;
			}

			set
			{
				drawableComponents = value;
			}
		}

		/// <summary>
		/// Default Constructor
		/// 
		/// Creates a new View object.  Cannot be called directly; subclass View
		/// to obtain this functionality.
		/// </summary>
		public View()
		{
			this.ControlAdded += new ControlEventHandler( componentAdded );
			this.ControlRemoved += new ControlEventHandler( componentRemoved );
			this.Paint += new PaintEventHandler( paintView );
			this.Layout += new LayoutEventHandler( doLayout );
			this.MouseDown += new MouseEventHandler( viewMouseDown );
			this.MouseWheel += new MouseEventHandler( mouseWheelMoved );
			this.MouseMove += new MouseEventHandler( viewMouseMove );
			this.MouseUp += new MouseEventHandler( mouseUp );
			this.DragEnter += new DragEventHandler( dragEnter );
			this.DragDrop += new DragEventHandler( dragDrop );

			this.AllowDrop = true;
			this.BorderStyle = BorderStyle.Fixed3D;

			this.SetStyle( ControlStyles.UserPaint, true );
			this.SetStyle( ControlStyles.AllPaintingInWmPaint, true );
			this.SetStyle( ControlStyles.DoubleBuffer, true );

			initScrollBars();
		}

		/// <summary>
		/// Deserialization Constructor
		/// 
		/// Creates a new View object.  Cannot be called directly; subclass View
		/// to obtain this functionality.
		/// </summary>
		protected View( SerializationInfo info, StreamingContext context ) : 
			this()
		{
			adding = info.GetBoolean( "adding" );
			lastHValue = info.GetInt32( "lastHValue" );
			lastVValue = info.GetInt32( "lastVValue" );						

			IList components = (IList)info.GetValue( "drawableComponents", typeof( ArrayList ) );
			foreach( DrawableComponent component in components )
			{
				addDrawableComponent( component );
			}
			Invalidate();
		}
		
		/// <summary>
		/// Serialization Function
		/// </summary>
		public virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue( "adding", adding, adding.GetType() );
			info.AddValue( "lastHValue", lastHValue, lastHValue.GetType() );
			info.AddValue( "lastVValue", lastVValue, lastVValue.GetType() );
			info.AddValue( "drawableComponents", drawableComponents, drawableComponents.GetType() );
		}

		/// <summary>
		/// Initiates adding a component to the view.
		/// </summary>
		/// <param name="componentType">The type of component being added.</param>
		public void beginAddComponent( String componentType )
		{
			Cursor = Cursors.Cross;
			StatusBarManager.Instance.StatusText = "Click view to add component - right-click to cancel";
			addComponent = provideAddComponentDelegate( componentType );
			adding = true;
        }

		/// <summary>
		/// Removes the specified DrawableComponent from the view.
		/// </summary>
		/// <param name="component">The component to remove.</param>
		public void removeDrawableComponent( DrawableComponent component )
		{
			drawableComponents.Remove( component );
			this.MouseDown -= component.MouseDownEvent;
			this.MouseMove -= component.MouseMoveEvent;
			this.MouseUp -= component.MouseUpEvent;
		}

		/// <summary>
		/// Clears the view of all components.
		/// </summary>
		public void clearView()
		{
			this.Controls.Clear();
			drawableComponents.Clear();

			this.Controls.Add( hScrollBar );
			this.Controls.Add( vScrollBar );
		}

		/// <summary>
		/// Ensures the specified component is visible in the view.
		/// </summary>
		/// <param name="component"></param>
		public void makeVisible( DrawableComponent component )
		{
			if ( drawableComponents.Contains( component ) )
			{
				if ( !ClientRectangle.Contains( component.Location ) )
				{
					int hDistance = component.Location.X - ( ( Width / 2 ) - ( component.Width / 2 ) );
					int vDistance = component.Location.Y - ( ( Height / 2 ) - ( component.Height / 2 ) );

					zoomFactor = 1.0f;

					lastHValue += hDistance;
					lastVValue += vDistance;

					if ( lastHValue < 0 )
					{
						lastHValue = 0;
					}

					if ( lastHValue > hScrollBar.Maximum )
					{
						lastHValue = hScrollBar.Maximum;
					}

					if ( lastVValue < 0 )
					{
						lastVValue = 0;
					}

					if ( lastVValue > vScrollBar.Maximum )
					{
						lastVValue = vScrollBar.Maximum;
					}
					
					hScrollBar.Value = lastHValue;
					vScrollBar.Value = lastVValue;
				}
			}
		}

		/// <summary>
		/// Exports the current view as an image file.
		/// </summary>
		/// <param name="imageFile">The image file to export to.</param>
		public void saveAsImage( String imageFile )
		{
			try
			{
				SuspendLayout();

				StatusBarManager sbm = StatusBarManager.Instance;
				sbm.StatusText = "Saving view as image.";

				int previousLastHValue = lastHValue;
				int previousLastVValue = lastVValue;

				lastHValue = 0;
				lastVValue = 0;

				int width = 0;
				int height = 0;
				foreach ( DrawableComponent component in drawableComponents )
				{
					if ( component.Location.X + component.Width > width )
					{
						width = component.Location.X + component.Width;
					}
					
					if ( component.Location.Y + component.Height > height )
					{
						height = component.Location.Y + component.Height;
					}
				}

				View view = JVMSMain.JvmsForm.CurrentView;
				if ( view.ToString().Equals( "Platform Assembly" ) || view.ToString().Equals( "Application Assembly" ) )
				{
					int newHeight = 0;
					int newWidth = 0;
					RelationshipManager rm = RelationshipManager.Instance;
					foreach ( IRelationship r in rm.Relationships )
					{
						if ( r is InstanceRefRelationship )
						{
							foreach ( Vertex v in ( (InstanceRefRelationship)r ).Verticies )
							{
								if ( v.Point.Y > newHeight )
								{
									newHeight = v.Point.Y;
								}
								
								if ( v.Point.X > newWidth )
								{
									newWidth = v.Point.X;
								}
							}
						}
					}

					if ( newHeight > height )
					{
						height = newHeight;
					}

					if ( newWidth > width )
					{
						width = newWidth;
					}
				}

				width = (int)( width * zoomFactor );
				height = (int)( height * zoomFactor );

				if ( view.ToString().Equals( "Platform Assembly" ) || view.ToString().Equals( "Application Assembly" ) )
				{
					width += 150;
				}
				else
				{
					width += 50;
				}
				height += 50;

				Bitmap image = new Bitmap( width, height );
				Graphics g = Graphics.FromImage( image );
				
				PaintEventArgs e = new PaintEventArgs( g, new Rectangle( 0, 0, image.Width, image.Height ) );
				paintView( this, e );

				lastHValue = previousLastHValue;
				lastVValue = previousLastVValue;

				Color empty = Color.FromArgb( 0, 0, 0, 0 );
				for ( int i = 0; i < image.Width; i += 1 )
				{
					for ( int j = 0; j < image.Height; j += 1 )
					{
						Color pixelColor = image.GetPixel( i, j );
						if ( pixelColor == empty )
						{
							image.SetPixel( i, j, Color.Transparent );
						}
					}
				}

				ImageFormat format = ImageFormat.Bmp;
				if ( imageFile.EndsWith( ".jpg" ) || imageFile.EndsWith( ".jpeg" ) )
				{
					format = ImageFormat.Jpeg;
				}
				else if ( imageFile.EndsWith( ".gif" ) )
				{
					format = ImageFormat.Gif;
				}
				else if ( imageFile.EndsWith( ".png" ) )
				{
					format = ImageFormat.Png;
				}
				else if ( imageFile.EndsWith( ".tif" ) || imageFile.EndsWith( ".tiff" ) )
				{
					format = ImageFormat.Tiff;
				}

				image.Save( imageFile, format );
				
				MessageBox.Show( JVMSMain.JvmsForm, "Saved image to: " + imageFile, "Export Image",
					MessageBoxButtons.OK, MessageBoxIcon.Information );
			}
			catch ( Exception e )
			{
				MessageBox.Show( JVMSMain.JvmsForm, "Cannot save image: " + e.Message, "Error Saving Image",
					MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
			finally
			{
				StatusBarManager.Instance.StatusText = "";
				ResumeLayout();
			}
		}

		/// <summary>
		/// Returns a delegate which will add the appropriate component to the view.
		/// </summary>
		/// <param name="componentType">The type of component which will be added.</param>
		/// <returns>An AddComponent delegate.</returns>
		protected abstract AddComponent provideAddComponentDelegate( String componentType );

		/// <summary>
		/// Called when the view receives a mouse up event.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected virtual void viewMouseUp( Object source, MouseEventArgs e )
		{
		}

		/// <summary>
		/// Called when the view receives a mouse down event.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected virtual void viewMouseDown( Object source, MouseEventArgs e )
		{
		}

		/// <summary>
		/// Called when the view receives a mouse move event.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected virtual void viewMouseMove( Object source, MouseEventArgs e )
		{
		}

		/// <summary>
		/// Called when a component has been added to the view.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected virtual void componentAdded( Object source, ControlEventArgs e )
		{
		}

		/// <summary>
		/// Called when a component has been removed from the view.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		protected virtual void componentRemoved( Object source, ControlEventArgs e )
		{
		}

		/// <summary>
		/// Called at the end of the base View paint event handler, should be overridden
		/// by subclasses.
		/// </summary>
		/// <param name="clipRect">The area in which the view is drawn.</param>
		/// <param name="g">The graphics context to use.</param>
		protected virtual void paintComponentView( Rectangle clipRect, Graphics g )
		{
		}

		/// <summary>
		/// Called when a key is pressed on the view.
		/// </summary>
		/// <param name="e">The event's data.</param>
		protected override void OnKeyDown( KeyEventArgs e )
		{
			RelationshipManager.Instance.viewKeyDown( this, e );
			if ( e.KeyCode == Keys.PageDown )
			{
				zoomFactor -= 0.05f;

				if ( zoomFactor < 0.1f )
				{
					zoomFactor = 0.1f;
				}
			}
			else if ( e.KeyCode == Keys.PageUp )
			{
				zoomFactor += 0.05f;

				if ( zoomFactor > 1.5f )
				{
					zoomFactor = 1.5f;
				}
			}
			else if ( e.KeyCode == Keys.Home )
			{
				zoomFactor = 1.0f;
			}
			else if ( e.KeyCode == Keys.Escape && adding )
			{
				adding = false;
				this.Cursor = Cursors.Default;
				StatusBarManager.Instance.StatusText = "";
			}

			Invalidate();
			base.OnKeyDown( e );
		}

		/// <summary>
		/// Called when the view should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void paintView( Object source, PaintEventArgs e )
		{
//			drawGrid( e.ClipRectangle, e.Graphics );
			e.Graphics.ScaleTransform( zoomFactor, zoomFactor );
			RelationshipManager.Instance.drawRelationships( e.ClipRectangle, e.Graphics );
			paintDrawableComponents( source, e );
			paintComponentView( e.ClipRectangle, e.Graphics );
		}

		/// <summary>
		/// Called when the view should paint all of its components.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void paintDrawableComponents( Object source, PaintEventArgs e )
		{						
			foreach( DrawableComponent component in drawableComponents )
			{
				e.Graphics.TranslateTransform( component.Location.X, component.Location.Y );
				component.paint( source, e );	
				e.Graphics.TranslateTransform( -component.Location.X, -component.Location.Y );
			}
		}

		/// <summary>
		/// Invalidates each drawable component.
		/// </summary>
		private void invalidateDrawableComponents()
		{
			foreach( DrawableComponent component in drawableComponents )
			{
				component.invalidate();
			}
		}

		/// <summary>
		/// Called when the view should update its layout.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void doLayout( Object source, LayoutEventArgs e )
		{
			hScrollBar.Location = new Point( 0, this.Height - hScrollBar.Height - 4 );
			hScrollBar.Width = this.Width - vScrollBar.Width - 4;
			vScrollBar.Location = new Point( this.Width - vScrollBar.Width - 4, 0 );
			vScrollBar.Height = this.Height - hScrollBar.Height - 4;
		}

		/// <summary>
		/// Called when the view is clicked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void mouseUp( Object source, MouseEventArgs e )
		{
			JVMSMain.JvmsForm.ActiveControl = this;
			viewMouseUp( source, e );
			if ( adding && e.Button == MouseButtons.Left )
			{
				double x = Math.Round( (double)e.X / gridSize ) * gridSize;
				double y = Math.Round( (double)e.Y / gridSize ) * gridSize;
				
				DrawableComponent component = addComponent();
				if( component != null )
				{
					int xPoint = (int)( x / zoomFactor );
					int yPoint = (int)( y / zoomFactor );
					Point location = new Point( xPoint, yPoint );
					location.Offset( lastHValue, lastVValue );
					component.Location = location;
					addDrawableComponent( component );
					JVMSMain.JvmsForm.SelectedComponent = component;
				}
				
				adding = false;
				this.Cursor = Cursors.Default;
				StatusBarManager.Instance.StatusText = "";
				ProjectManager.Instance.ChangedFlag = true;
			}
			else if ( adding && e.Button == MouseButtons.Right )
			{
				adding = false;
				this.Cursor = Cursors.Default;
				StatusBarManager.Instance.StatusText = "";
			}
			else if ( e.Button == MouseButtons.Middle )
			{
				zoomFactor = 1.0f;				
			}
			Invalidate();
		}

		/// <summary>
		/// Called when the mouse wheel is moved inside the control.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void mouseWheelMoved( Object source, MouseEventArgs e )
		{
			if ( e.Delta > 0 )
			{
				zoomFactor += 0.05f;

				if ( zoomFactor > 1.5f )
				{
					zoomFactor = 1.5f;
				}
			}
			else
			{
				zoomFactor -= 0.05f;

				if ( zoomFactor < 0.1f )
				{
					zoomFactor = 0.1f;
				}
			}
			Invalidate();
		}

		/// <summary>
		/// Adds the specified DrawableComponent to the view.
		/// </summary>
		/// <param name="component">The component to add.</param>
		private void addDrawableComponent( DrawableComponent component )
		{
			DrawableComponents.Add( component );
			this.MouseDown += component.MouseDownEvent;
			this.MouseMove += component.MouseMoveEvent;
			this.MouseUp += component.MouseUpEvent;
		}

		/// <summary>
		/// Called when a drag operation enters the view.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void dragEnter( Object source, DragEventArgs e )
		{
			if ( e.Data.GetDataPresent( DataFormats.Text ) )
			{
				e.Effect = DragDropEffects.Copy;
			}
		}

		/// <summary>
		/// Called when a drop has occured.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void dragDrop( Object source, DragEventArgs e )
		{
			String data = e.Data.GetData( DataFormats.Text ).ToString();
			addComponent = provideAddComponentDelegate( data );
			Point p = this.PointToClient( new Point( e.X, e.Y ) );
			double x = Math.Round( (double)p.X / gridSize ) * gridSize;
			double y = Math.Round( (double)p.Y / gridSize ) * gridSize;
			int xPoint = (int)( x / zoomFactor );
			int yPoint = (int)( y / zoomFactor );
			Point location = new Point( xPoint, yPoint );
			location.Offset( lastHValue, lastVValue );
			DrawableComponent component = addComponent();
			component.Location = location;
			addDrawableComponent( component );
			Invalidate();
			JVMSMain.JvmsForm.SelectedComponent = component;
			ProjectManager.Instance.ChangedFlag = true;
		}

		/// <summary>
		/// Draws the view's grid.
		/// </summary>
		/// <param name="rect">The area in which to draw the grid.</param>
		/// <param name="g">The graphics context to use.</param>
		private void drawGrid( Rectangle rect, Graphics g )
		{
			Pen gridPen = new Pen( Color.Black, 1 );
			Bitmap bitmap = new Bitmap( 1, 1 );
			bitmap.SetPixel( 0, 0, Color.Black );

			int rows = (int)( ( rect.Height - hScrollBar.Height ) / gridSize );
			int cols = (int)( ( rect.Width - vScrollBar.Width ) / gridSize );
			
			for ( int i = 0; i < rows; i += 1 )
			{
				for ( int j = 0; j < cols; j += 1 )
				{
					Point p = new Point( j * gridSize, i * gridSize );
					g.DrawImageUnscaled( bitmap, p );
				}
			}
		}

		/// <summary>
		/// Initializes the view's scroll bars.
		/// </summary>
		private void initScrollBars()
		{
			hScrollBar = new HScrollBar();
			hScrollBar.Minimum = 0;
			hScrollBar.Maximum = 500;
			hScrollBar.SmallChange = 10;
			hScrollBar.LargeChange = 50;
			hScrollBar.Value = 0;
			hScrollBar.Scroll += new ScrollEventHandler( onHorizontalScroll );

			vScrollBar = new VScrollBar();
			vScrollBar.Minimum = 0;
			vScrollBar.Maximum = 500;
			vScrollBar.SmallChange = 10;
			vScrollBar.LargeChange = 50;
			vScrollBar.Value = 0;
			vScrollBar.Scroll += new ScrollEventHandler( onVerticalScroll );
			
			this.Controls.AddRange( new Control[] { hScrollBar, vScrollBar } );
		}

		/// <summary>
		/// Called when the vertical scroll bar has been scrolled.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void onVerticalScroll( Object source, ScrollEventArgs e )
		{
			if ( e.Type == ScrollEventType.EndScroll || e.Type == ScrollEventType.ThumbPosition )
			{
				return;
			}
			int change = 0;

			if ( e.Type == ScrollEventType.ThumbTrack )
			{
				int newValue = (int)Math.Round( (double)e.NewValue / gridSize ) * gridSize;
				change = lastVValue - newValue;
				vScrollBar.Value = newValue;
				lastVValue = newValue;

				change = (int)Math.Round( (double)change / gridSize ) * gridSize;

				if ( change != 0 )
				{
					invalidateDrawableComponents();
					RelationshipManager.Instance.notifyViewScrolled( Orientation.Vertical, change );
				}

				return;
			}

			if ( e.Type == ScrollEventType.SmallIncrement )
			{
				change = -( vScrollBar.SmallChange );
			}
			else if ( e.Type == ScrollEventType.LargeIncrement )
			{
				change = -( vScrollBar.LargeChange );
			}
			else if ( e.Type == ScrollEventType.SmallDecrement )
			{
				change = vScrollBar.SmallChange;
			}
			else if ( e.Type == ScrollEventType.LargeDecrement )
			{
				if ( e.NewValue == 0 )
				{
					change = vScrollBar.Value;
				}
				else
				{
					change = vScrollBar.LargeChange;
				}
			}
			
			if ( lastVValue != e.NewValue )
			{
				lastVValue = e.NewValue;
				invalidateDrawableComponents();
				RelationshipManager.Instance.notifyViewScrolled( Orientation.Vertical, change );
			}

			if ( ( e.NewValue == vScrollBar.Maximum - vScrollBar.LargeChange + 1 ) && ( e.Type != ScrollEventType.ThumbPosition ) && ( e.Type != ScrollEventType.ThumbTrack ) )
			{
				vScrollBar.Maximum += 100;
			}			
		}

		/// <summary>
		/// Called when the horizontal scroll bar has been scrolled.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void onHorizontalScroll( Object source, ScrollEventArgs e )
		{
			if ( e.Type == ScrollEventType.EndScroll || e.Type == ScrollEventType.ThumbPosition )
			{
				return;
			}
			int change = 0;

			if ( e.Type == ScrollEventType.ThumbTrack )
			{
				int newValue = (int)Math.Round( (double)e.NewValue / gridSize ) * gridSize;
				change = lastHValue - newValue;
				hScrollBar.Value = newValue;
				lastHValue = newValue;

				change = (int)Math.Round( (double)change / gridSize ) * gridSize;

				if ( change != 0 )
				{
					invalidateDrawableComponents();
					RelationshipManager.Instance.notifyViewScrolled( Orientation.Horizontal, change );
				}

				return;
			}

			if ( e.Type == ScrollEventType.SmallIncrement )
			{
				change = -( hScrollBar.SmallChange );
			}
			else if ( e.Type == ScrollEventType.LargeIncrement )
			{
				change = -( hScrollBar.LargeChange );
			}
			else if ( e.Type == ScrollEventType.SmallDecrement )
			{
				change = hScrollBar.SmallChange;
			}
			else if ( e.Type == ScrollEventType.LargeDecrement )
			{
				if ( e.NewValue == 0 )
				{
					change = hScrollBar.Value;
				}
				else
				{
					change = hScrollBar.LargeChange;
				}
			}

			if ( lastHValue != e.NewValue )
			{
				lastHValue = e.NewValue;
				invalidateDrawableComponents();
				RelationshipManager.Instance.notifyViewScrolled( Orientation.Horizontal, change );
			}

			if ( ( e.NewValue == hScrollBar.Maximum - hScrollBar.LargeChange + 1 ) && ( e.Type != ScrollEventType.ThumbPosition ) && ( e.Type != ScrollEventType.ThumbTrack ) )
			{
				hScrollBar.Maximum += 100;
			}
		}
	}
}
