using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Runtime.Serialization;

using jvms.datamodel.sad;
using jvms.datamodel.spd;
using jvms.project;
using jvms.system;

namespace jvms.presentation.drawablecomponent
{
	[Serializable]
	/// <summary>
	/// Summary description for NonSCAComponentDrawableComponent.
	/// </summary>
	public class ApplicationComponentInstantiationDrawableComponent : BaseSCAComponentDrawableComponent
	{
		private const int TITLE_HEIGHT = 25;

		private ApplicationComponentInstantiation instantiation = null;

		private static Icon ICON;

		/// <summary>
		/// This will get the instantiation.
		/// </summary>
		public ApplicationComponentInstantiation Instantiation
		{
			get
			{
				return instantiation;
			}
			set
			{
				instantiation = value;
			}
		}

		/// <summary>
		/// This is a static constructor which means that it is only called once when the 
		/// class is loaded.
		/// </summary>
		static ApplicationComponentInstantiationDrawableComponent()
		{
			System.Reflection.Assembly myApp = System.Reflection.Assembly.GetExecutingAssembly();
			Stream stream = null;

			stream = myApp.GetManifestResourceStream("jvms.resources.InstanceIcon_Small.ico");
			ICON = new Icon( stream );
			stream.Close();
		}

		public ApplicationComponentInstantiationDrawableComponent( NonSCAComponent component )
		{
			instantiation = new ApplicationComponentInstantiation();
			instantiation.Component = component;
			instantiation.Properties.BaseComponent = component;
			if( component is SCAComponent )
			{
				Component = (SCAComponent)component;
			}
			ProjectManager.Instance.Project.addApplicationInstantiation( instantiation );
			this.Size = new Size(250, 100 );
			initComponent();
		}

		/// <summary>
		/// Creates a new ApplicationAssemblyView.
		/// </summary>
		protected ApplicationComponentInstantiationDrawableComponent(SerializationInfo info, StreamingContext context) :
			base( info, context )
		{
			instantiation = (ApplicationComponentInstantiation)info.GetValue( "instantiation", typeof( ApplicationComponentInstantiation ) );
			StatusBarManager.Instance.StatusText = "Loading " + getWrappedObject().ToString();
			initComponent();
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData( info, context );
			info.AddValue( "instantiation", instantiation, typeof(ApplicationComponentInstantiation) );
		}

		private void initComponent()
		{
		}

		public override void mouseUp( object sender, System.Windows.Forms.MouseEventArgs e )
		{
			base.mouseUp( sender, e );
			if( isOnComponent( e.X, e.Y ) )
			{
				RelationshipManager.Instance.applicationInstanceSelected( this );
			}
		}

		public override Object getWrappedObject()
		{
			return instantiation;
		}

		/// <summary>
		/// Thiw will perform the action of deleting this component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public override void deleteComponent(object sender, System.EventArgs e)
		{
			ViewManager.Instance.AppAssmView.removeDrawableComponent( this );
			ProjectManager.Instance.Project.ApplicationInstantiations.Remove( getWrappedObject() );
			RelationshipManager.Instance.removeRelationshipsToDrawableComponent( this );
		}

		/// <summary>
		/// Called when the view should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public override void paintComponent( Object source, System.Windows.Forms.PaintEventArgs e )
		{
			if( instantiation.Component is SCAComponent )
			{
				paintSCAComponent( e );
			}
			else
			{
				paintNonSCAComponent( instantiation.Component, e );
			}
		}

		private void paintNonSCAComponent( NonSCAComponent component, System.Windows.Forms.PaintEventArgs e )
		{
			JVMSProperties props = JVMSProperties.Instance;
			Brush headerBrush = new SolidBrush( (Color)props.getProperty("color.instanceheader", Color.CornflowerBlue ) );
			Brush bodyBrush = new SolidBrush( (Color)props.getProperty("color.content", Color.Beige ) );

			//fill background
			e.Graphics.FillRectangle( bodyBrush, 0 ,0,Size.Width,Size.Height );
			//show title
			e.Graphics.FillRectangle( headerBrush, 0, 0,Size.Width,TITLE_HEIGHT );

			//draw border
			Pen smallPen = new Pen( Color.Black, 1 );
			Pen borderPen = getBorderPen();
			e.Graphics.DrawRectangle( borderPen, borderPen.Width-1, borderPen.Width-1, Size.Width-borderPen.Width, Size.Height-borderPen.Width );

			Font titleFont = new Font( "Arial", 12 );
			Brush titleBrush = Brushes.Black;	
	
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			Rectangle targetRect = new Rectangle( 
				(int)(5*zoomFactor), 
				(int)(5 * zoomFactor), 
				(int)(ICON.Width*zoomFactor),
				(int)(ICON.Height*zoomFactor) );
			e.Graphics.DrawIcon( ICON, targetRect );

			StringFormat titleFormat = new StringFormat();
			titleFormat.Alignment = StringAlignment.Near;

			//draw component name and the line
			Rectangle titleRectangle = new Rectangle( 19, 5, Size.Width-19, 20 );
			drawTruncatedString( instantiation.UsageName, titleFont, titleRectangle, titleFormat, titleBrush, e.Graphics );

			e.Graphics.DrawLine( smallPen, new Point( 0, TITLE_HEIGHT ), 
				new Point( Size.Width-1, TITLE_HEIGHT ) );
		}		

		/// <summary>
		/// This will return a string representation of this drawable component.
		/// </summary>
		/// <returns></returns>
		public override String ToString()
		{
			return instantiation.UsageName;
		}
	}
}