using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;
using System.Reflection;
using System.Runtime.Serialization;

using jvms.datamodel.common;
using jvms.datamodel.dcd;
using jvms.datamodel.sad;
using jvms.datamodel.scd;
using jvms.datamodel.spd;

using jvms.system;

namespace jvms.presentation.drawablecomponent
{
	[Serializable]
	/// <summary>
	/// Summary description for SCAComponentDrawableComponent.
	/// </summary>
	public abstract class BaseSCAComponentDrawableComponent : DrawableComponent
	{
		private Font TITLE_FONT = new Font( "Arial", 12 );
		private Font PORT_FONT = new Font( "Arial", 8 );
		private Pen SMALL_PEN = new Pen( Color.Black, 1 );

		private const int TITLE_HEIGHT = 22;
		private const int TYPE_HEIGHT = 18;
		private const int TYPE_POSITION = TITLE_HEIGHT;
		private const int TYPE_LINE_POSITION = TITLE_HEIGHT + TYPE_HEIGHT;
		private const int PORT_START_POSITION = TYPE_LINE_POSITION + 4;
		private const int PORT_HEIGHT = 32;
		private const int PORT_SPACING = 8;
		private const int PORT_WIDTH = 10;	
	
		private static Hashtable ICON_MAP = new Hashtable();
		private static Icon INSTANCE_ICON = null;

		private SCAComponent component = null;

		private ComponentPortEndPoint selectedPort = null;

		/// <summary>
		/// This is a static constructor which means that it is only called once when the 
		/// class is loaded.
		/// </summary>
		static BaseSCAComponentDrawableComponent()
		{
			System.Reflection.Assembly myApp = System.Reflection.Assembly.GetExecutingAssembly();
			Stream stream = null;

			stream = myApp.GetManifestResourceStream("jvms.resources.CFResourceIcon_Small.ico");
			ICON_MAP.Add( "resource", new Icon( stream ) );
			stream.Close();

			stream = myApp.GetManifestResourceStream("jvms.resources.CFDeviceIcon_Small.ico");
			ICON_MAP.Add( "device", new Icon( stream ) );
			stream.Close();

			stream = myApp.GetManifestResourceStream("jvms.resources.CFLoggerIcon_Small.ico");
			ICON_MAP.Add( "log", new Icon( stream ) );
			stream.Close();

			stream = myApp.GetManifestResourceStream( "jvms.resources.CFFilesystemIcon_Small.ico" );
			ICON_MAP.Add( "filesystem", new Icon( stream ) );
			stream.Close();

			stream = myApp.GetManifestResourceStream( "jvms.resources.CFResourceFactoryIcon_Small.ico" );
			ICON_MAP.Add( "resourcefactory", new Icon( stream ) );
			stream.Close();

			stream = myApp.GetManifestResourceStream( "jvms.resources.CFResourceIcon_Small.ico" );
			ICON_MAP.Add( "namingservice", new Icon( stream ) );
			stream.Close();

			stream = myApp.GetManifestResourceStream( "jvms.resources.DeviceManagerIcon_Small.ico" );
			ICON_MAP.Add( "devicemanager", new Icon( stream ) );
			stream.Close();

			stream = myApp.GetManifestResourceStream( "jvms.resources.InstanceIcon_Small.ico" );
			INSTANCE_ICON = new Icon( stream );
			stream.Close();
		}

		/// <summary>
		/// 
		/// </summary>
		public BaseSCAComponentDrawableComponent() :
			base()
		{
			initComponent();
		}

		/// <summary>
		/// Creates a new BaseSCAComponentDrawableComponent.
		/// </summary>
		protected BaseSCAComponentDrawableComponent(SerializationInfo info, StreamingContext context) :
			base( info, context )
		{
			component = (SCAComponent)info.GetValue( "component", typeof( SCAComponent ) );
			StatusBarManager.Instance.StatusText = "Loading " + getWrappedObject();
			initComponent();			
		}

		private void initComponent()
		{
			ContextMenu.Popup += new EventHandler( contextMenuEventHandler );
		}

		private void contextMenuEventHandler(System.Object sender, System.EventArgs e)
		{			
			Point clickedPoint = JVMSMain.JvmsForm.CurrentView.PointToClient( Cursor.Position );
			selectedPort = getPortAtPosition( clickedPoint.X, clickedPoint.Y );
			if( selectedPort != null &&
				getWrappedObject() is ApplicationComponentInstantiation )
			{
				MenuItem makeExternalMenuItem = new MenuItem( "External Port" );
				makeExternalMenuItem.Click += new EventHandler( makeExternal );
				ApplicationComponentInstantiation instance = 
					(ApplicationComponentInstantiation)getWrappedObject();

				if( instance.ExternalPorts.Contains( selectedPort ) )
				{
					makeExternalMenuItem.Checked = true;
				}

				ContextMenu.MenuItems.Add( makeExternalMenuItem );
			}
			
		}

		private void makeExternal(object sender, System.EventArgs e)
		{
			ApplicationComponentInstantiation instance = 
				(ApplicationComponentInstantiation)getWrappedObject();
			if( instance.ExternalPorts.Contains( selectedPort ) )
			{
				instance.ExternalPorts.Remove( selectedPort );
			}
			else
			{
				instance.ExternalPorts.Add( selectedPort );
			}
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData( info, context );
			info.AddValue( "component", component, typeof(SCAComponent) );
		}

		public override void mouseUp( object sender, MouseEventArgs e )
		{
			base.mouseUp( sender, e );
			if( isOnComponent( e.X, e.Y ) && e.Button == MouseButtons.Left && component != null )
			{
				ComponentPortEndPoint port = getPortAtPosition( e.X, e.Y );
				if( port != null )
				{
					if( component.Descriptor.Features.ProvidesPorts.Contains( port ) ||
						component.Descriptor.Features.SupportInterfaces.Contains( port.RepositoryID ) )
					{
						RelationshipManager.Instance.providesPortSelected(this, port );
					}
					else if( component.Descriptor.Features.UsesPorts.Contains( port ) )
					{
						RelationshipManager.Instance.usesPortSelected(this, port );
					}
				}
			}
		}

		private ComponentPortEndPoint getPortAtPosition( int x, int y )
		{
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			x = (int)(x / zoomFactor);
			y = (int)(y / zoomFactor);
			if( component == null )
			{
				return null;
			}
			//we need to determine if they clicked on a port.
			ComponentPortEndPoint port = null;
			bool leftSide = true;
			int portPosition = Location.Y + PORT_START_POSITION;
			foreach( ComponentPortEndPoint nextPort in component.Descriptor.Features.ProvidesPorts )
			{
				if( y >= portPosition && y <= portPosition + PORT_HEIGHT )
				{
					if( leftSide )
					{
						if( x>=Location.X+PORT_WIDTH && x <= Location.X +2*PORT_WIDTH  )
						{
							port = nextPort;
						}
					}
					else
					{
						if( x >= Location.X + Size.Width - 2*PORT_WIDTH && x <= Location.X + Size.Width-PORT_WIDTH )
						{
							port = nextPort;
						}
					}
				}
				leftSide = !leftSide;
				if( leftSide )
				{
					portPosition += PORT_HEIGHT + PORT_SPACING;
				}
			}
			foreach( Interface supports in component.Descriptor.Features.SupportInterfaces )
			{
				if( y >= portPosition && y <= portPosition + PORT_HEIGHT )
				{
					if( leftSide )
					{
						if( x>=Location.X+PORT_WIDTH && x <= Location.X +2*PORT_WIDTH  )
						{
							port = createPort( supports );
						}
					}
					else
					{
						if( x >= Location.X + Size.Width - 2*PORT_WIDTH && x <= Location.X + Size.Width-PORT_WIDTH )
						{
							port = createPort( supports );
						}
					}
				}
				leftSide = !leftSide;
				if( leftSide )
				{
					portPosition += PORT_HEIGHT + PORT_SPACING;
				}
			}
			foreach( ComponentPortEndPoint nextPort in component.Descriptor.Features.UsesPorts )
			{
				if( y >= portPosition && y <= portPosition + PORT_HEIGHT )
				{
					if( leftSide )
					{
						if( x>=Location.X && x <= Location.X +PORT_WIDTH )
						{
							port = nextPort;
						}
					}
					else
					{
						if( x >= Location.X + Size.Width - PORT_WIDTH && x <= Location.X + Size.Width)
						{
							port = nextPort;
						}
					}
				}
				leftSide = !leftSide;
				if( leftSide )
				{
					portPosition += PORT_HEIGHT + PORT_SPACING;
				}
			}

			return port;
		}

		private ComponentPortEndPoint createPort( Interface supports )
		{
			ComponentPortEndPoint endPoint = new ComponentPortEndPoint();
			endPoint.RepositoryID = supports;
			endPoint.UsesName = supports.Name;
			return endPoint;
		}


		/// <summary>
		/// Called when the view should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public void paintSCAComponent( PaintEventArgs e )
		{
			JVMSProperties props = JVMSProperties.Instance;
			Brush headerBrush = new SolidBrush( (Color)props.getProperty("color.scaheader", Color.Aqua ) );
			Brush bodyBrush = new SolidBrush( (Color)props.getProperty("color.content", Color.Beige ) );
			Brush portBrush = new SolidBrush( (Color)props.getProperty("color.port", Color.LightBlue) );

			int calculatedHeight = PORT_START_POSITION;
			int numberOfPortRows = ((component.Descriptor.Features.ProvidesPorts.Count + 
				component.Descriptor.Features.UsesPorts.Count+
				component.Descriptor.Features.SupportInterfaces.Count+1) /2 );
			//calculate height of port rows, leaving off the spacing at the very end though
			//which is why we use n-1 for port rows
			calculatedHeight += numberOfPortRows * (PORT_HEIGHT+PORT_SPACING);

			//This is for the first time when the component has no ports or 
			//implementations we add some space to make it look good.
			if( numberOfPortRows == 0 )
			{
				calculatedHeight += PORT_HEIGHT+PORT_SPACING;
			}
			this.Size = new Size( this.Size.Width, calculatedHeight );

			Brush titleBrush = Brushes.Black;

			Rectangle titleRectangle = new Rectangle(
				PORT_WIDTH + 16,
				3,
				Size.Width - 2*PORT_WIDTH - 16*2,
				20 );
			Rectangle typeRectangle = new Rectangle(
				PORT_WIDTH,
				TYPE_POSITION,
				Size.Width - 2*PORT_WIDTH,
				20 );
			StringFormat titleFormat = new StringFormat();
			titleFormat.Alignment = StringAlignment.Center;
			String title = component.Name;
			String type =  "<<" + component.Descriptor.ComponentType + ">>";
			Icon icon = (Icon)ICON_MAP[component.Descriptor.ComponentType];
			if( getWrappedObject() is ComponentInstantiation )
			{
				icon = INSTANCE_ICON;
				title = ((ComponentInstantiation)getWrappedObject()).UsageName;
				type = component.Name;
				headerBrush = new SolidBrush( (Color)props.getProperty("color.instanceheader", Color.CornflowerBlue ) );
			}
			else if( getWrappedObject() is ComponentPlacement )
			{
				icon = INSTANCE_ICON;
				title = ((ComponentPlacement)getWrappedObject()).Instantiation.UsageName;
				type = component.Name;
				headerBrush = new SolidBrush( (Color)props.getProperty("color.instanceheader", Color.CornflowerBlue ) );
			}

			e.Graphics.FillRectangle( bodyBrush,PORT_WIDTH,0,Size.Width-2*PORT_WIDTH,Size.Height );
			e.Graphics.FillRectangle( headerBrush, PORT_WIDTH,0,Size.Width-2*PORT_WIDTH,TITLE_HEIGHT );

			//draw border around box
			Pen borderPen = getBorderPen();
			e.Graphics.DrawRectangle( borderPen, PORT_WIDTH, borderPen.Width-1, Size.Width-2*PORT_WIDTH-1, Size.Height-borderPen.Width );
			drawTruncatedString(
				title,
				TITLE_FONT,
				titleRectangle,
				titleFormat,
				titleBrush,
				e.Graphics );
			drawTruncatedString(
				type,
				TITLE_FONT,
				typeRectangle,
				titleFormat,
				titleBrush,
				e.Graphics );

			e.Graphics.DrawLine( SMALL_PEN, PORT_WIDTH, TITLE_HEIGHT, 
				                            Size.Width-PORT_WIDTH-1, TITLE_HEIGHT);
			e.Graphics.DrawLine( SMALL_PEN, PORT_WIDTH, TYPE_LINE_POSITION,
				                            Size.Width-PORT_WIDTH-1, TYPE_LINE_POSITION );


			if( icon != null )
			{
				float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
				Rectangle targetRect = new Rectangle( 
					(int)((PORT_WIDTH+3)*zoomFactor), 
					(int)(5 * zoomFactor), 
					(int)(icon.Width*zoomFactor),
					(int)(icon.Height*zoomFactor) );
				e.Graphics.DrawIcon( icon, targetRect );
			}
			bool leftSide = true;
			int portPosition = PORT_START_POSITION;
			foreach( ComponentPortEndPoint endPoint in component.Descriptor.Features.ProvidesPorts )
			{
				drawPort( e.Graphics, leftSide, false, endPoint, portPosition );
				leftSide = !leftSide;
				if( leftSide )
				{
					portPosition += PORT_HEIGHT + PORT_SPACING;
				}
			}
			foreach( Interface supports in component.Descriptor.Features.SupportInterfaces )
			{
				ComponentPortEndPoint endPoint = createPort( supports );
				drawPort( e.Graphics, leftSide, false, endPoint, portPosition );
				leftSide = !leftSide;
				if( leftSide )
				{
					portPosition += PORT_HEIGHT + PORT_SPACING;
				}
			}
			foreach( ComponentPortEndPoint endPoint in component.Descriptor.Features.UsesPorts )
			{
				drawPort( e.Graphics, leftSide, true, endPoint, portPosition );
				leftSide = !leftSide;
				if( leftSide )
				{
					portPosition += PORT_HEIGHT + PORT_SPACING;
				}
			}
		}

		public SCAComponent Component
		{
			get
			{
				return component;
			}
			set
			{
				component = value;
			}
		}

		/// <summary>
		/// This will determine if a port is on the left side.
		/// </summary>
		/// <param name="port">The port.</param>
		/// <returns>true if the port is drawn on the left side.</returns>
		public bool isPortOnLeftSide( ComponentPortEndPoint port )
		{
			bool leftSide = false;			
			int usesIndex = component.Descriptor.Features.UsesPorts.IndexOf( port );
			int supportsIndex = component.Descriptor.Features.SupportInterfaces.IndexOf( port.RepositoryID );
			int providesIndex = component.Descriptor.Features.ProvidesPorts.IndexOf( port );
			if( providesIndex != -1 )
			{
				//we are on left if we are a multiple of 2
				leftSide = providesIndex % 2 == 0;
			}
			else if( supportsIndex != -1 )
			{
				leftSide = 
					(component.Descriptor.Features.ProvidesPorts.Count + supportsIndex) % 2 == 0;
			}
			else if( usesIndex != -1 )
			{
				leftSide = 
					(component.Descriptor.Features.ProvidesPorts.Count + 
					 component.Descriptor.Features.SupportInterfaces.Count + usesIndex) % 2 == 0;
			}
			return leftSide;
		}

		/// <summary>
		/// This will get the point on the port where a relationship
		/// should be connected.
		/// </summary>
		/// <param name="port">The port to find the location of.</param>
		/// <returns>A point at the end of the port</returns>
		public Point getPortLocation( ComponentPortEndPoint port )
		{
			if( component == null )
			{
				throw new Exception( "Error cannot find port location on NonSCAComponent" );
			}
			Point retval = new Point( 0, 0 );
			int providesIndex = component.Descriptor.Features.ProvidesPorts.IndexOf( port );
			int supportsIndex = component.Descriptor.Features.SupportInterfaces.IndexOf( port.RepositoryID );
			int usesIndex = component.Descriptor.Features.UsesPorts.IndexOf( port );
			

			//is the port on the left side?
			bool leftSide = isPortOnLeftSide( port );
			//the number of components drawn down the side
			//the first component will be zero and the third component
			//will be 1 because it is the second component drawn on the
			//left side(zero based indexing of course)
			int numberOnSide;

			//this algorithm depends on the provides ports being drawn and then the uses
			//ports being drawn
			if( providesIndex != -1 )
			{
				numberOnSide = (int)(providesIndex / 2);
				if( leftSide )
				{
					retval.X = PORT_WIDTH;
				}
				else
				{
					retval.X = Size.Width - PORT_WIDTH;
				}

			}
			else if( supportsIndex != -1 )
			{
				numberOnSide = (int)(
					(component.Descriptor.Features.ProvidesPorts.Count + providesIndex) / 2);
				if( leftSide )
				{
					retval.X = PORT_WIDTH;
				}
				else
				{
					retval.X = Size.Width - PORT_WIDTH;
				}

			}
			else if( usesIndex != -1 )
			{
				numberOnSide = 
					(int)(
					(component.Descriptor.Features.ProvidesPorts.Count +
					 component.Descriptor.Features.SupportInterfaces.Count + usesIndex) / 2);

				if( leftSide )
				{
					retval.X = 0;
				}
				else
				{
					retval.X = Size.Width;
				}
			}
			else
			{
				throw new Exception( "Error: attempt to get port location of a port " +
					"that is not on a component! component=" + component + " port=" + port );
			}

			//For the Y component it does not matter if it is a 
			//uses/provides or left/right side
			//
			//The y is made up of three parts
			//1)PORT_START_POSITION - where the ports actually start
			//2)The space from all previous ports
			//3)Half the space from this port
			retval.Y = 
				PORT_START_POSITION + 
				(PORT_HEIGHT + PORT_SPACING)*numberOnSide +
				PORT_HEIGHT / 2;

			//now offset this point to be relative to the view.
			retval.Offset( Location.X, Location.Y );
			return retval;
		}
		
		private void drawPort( Graphics g, bool leftSide, bool usesPort, ComponentPortEndPoint endPoint, int portPosition )
		{
			JVMSProperties props = JVMSProperties.Instance;
			Brush portBrush = new SolidBrush( (Color)props.getProperty("color.port", Color.LightBlue) );

			int startAngle = 90;
			int sweepAngle = 180;
			int circleX = 0;
			int circleY = portPosition;				
			int portDescriptorX = 0;
			int portDescriptorWidth = 0;
			StringFormat stringFormat = new StringFormat();
			if( usesPort )
			{
				portDescriptorX = PORT_WIDTH + 5;
				portDescriptorWidth = (this.Size.Width/2) - PORT_WIDTH-5;
			}
			else
			{
				sweepAngle *= -1;
				portDescriptorX = 2 * PORT_WIDTH + 5;
				portDescriptorWidth = (this.Size.Width/2) - 2 * PORT_WIDTH-5;
			}
			if( !leftSide )
			{
				sweepAngle *= -1;
				circleX = Size.Width-2*PORT_WIDTH-1;
				portDescriptorX = Size.Width/2;
				stringFormat.Alignment = StringAlignment.Far;
			}
			Rectangle nameRectangle = new Rectangle(
				portDescriptorX,
				portPosition,
				portDescriptorWidth,
				13 );
			Rectangle portTypeRectangle = new Rectangle(
				portDescriptorX,
				portPosition + 15,
				portDescriptorWidth,
				13 );

			g.FillPie(portBrush, circleX, circleY, 2*PORT_WIDTH, PORT_HEIGHT, startAngle, sweepAngle );
			g.DrawPie(getBorderPen(), circleX, circleY, 2*PORT_WIDTH, PORT_HEIGHT, startAngle, sweepAngle );

			String portTypeString = "";
			if( endPoint.RepositoryID != null )
			{
				portTypeString = endPoint.RepositoryID.Name;
			}

			drawTruncatedString( 
				portTypeString, 
				PORT_FONT,
				portTypeRectangle,
				stringFormat,
				Brushes.Black,
				g );
			drawTruncatedString( 
				endPoint.UsesName,
				PORT_FONT,
				nameRectangle,
				stringFormat,
				Brushes.Black,
				g );
		}
	}
}
