using System;
using System.Collections;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using System.Runtime;
using System.Runtime.Serialization;
using System.Runtime.InteropServices;

using jvms.datamodel;
using jvms.datamodel.dcd;
using jvms.datamodel.sad;
using jvms.datamodel.spd;
using jvms.presentation;
using jvms.validation;
using jvms.project;
using jvms.system;
using jvms.xmlgeneration;

namespace jvms.presentation.drawablecomponent
{
	[Serializable]
	public abstract class DrawableComponent : ISerializable
	{
		private static Pen SELECTED_BORDER_PEN = new Pen( Color.Black, 2 );
		private static Pen UNSELECTED_BORDER_PEN = new Pen( Color.Black, 1 );
		private bool selected = false;
		private JVMSForm form;

		private Point location;
		private Size size;

		//there are used to properly invalidate the region
		private Point lastLocation;
		private Size lastSize;
		private float lastZoomFactor = 1.0f;
		private ContextMenu contextMenu;

		//dragging information
		private bool dragging = false;
		private int draggingX;
		private int draggingY;
		private int gridSize = 10;

		//we need to keep these around to remove them properly
		//
		private MouseEventHandler mouseDownEvent = null;
		private MouseEventHandler mouseMoveEvent = null;
		private MouseEventHandler mouseUpEvent = null;

		/// <summary>
		/// Property to get event
		/// </summary>
		public MouseEventHandler MouseDownEvent
		{
			get
			{ 
				if( mouseDownEvent == null )
				{
					mouseDownEvent = new MouseEventHandler( mouseDown );
				}
				return mouseDownEvent; 
			}
		}

		/// <summary>
		/// Property to get event
		/// </summary>
		public MouseEventHandler MouseMoveEvent
		{
			get
			{ 
				if( mouseMoveEvent == null )
				{
					mouseMoveEvent = new MouseEventHandler( mouseMove );
				}
				return mouseMoveEvent; 
			}
		}

		/// <summary>
		/// Property to get event
		/// </summary>
		public MouseEventHandler MouseUpEvent
		{
			get
			{ 
				if( mouseUpEvent == null )
				{
					mouseUpEvent = new MouseEventHandler( mouseUp );
				}
				return mouseUpEvent; 
			}
		}

		/// <summary>
		/// This will get the context menu.
		/// </summary>
		public ContextMenu ContextMenu
		{
			get
			{
				return contextMenu;
			}
			set
			{
				contextMenu = value;
			}
		}

		/// <summary>
		/// The location to draw the component.
		/// </summary>
		public Point Location
		{
			get
			{
				Point p = new Point( 
					location.X - JVMSMain.JvmsForm.CurrentView.LastHorizontalScrollValue,
					location.Y - JVMSMain.JvmsForm.CurrentView.LastVerticalScrollValue );
				return p;
			}
			set
			{
				location = value;
			}
		}

		/// <summary>
		/// This will get the size of the component.
		/// </summary>
		public Size Size
		{
			get
			{
				return size;
			}
			set
			{
				size = value;
			}
		}

		public int Width
		{
			get
			{
				return size.Width;
			}
			set
			{
				size.Width = value;
			}
		}
		public int Height
		{
			get
			{
				return size.Height;
			}
			set
			{
				size.Height = value;
			}
		}

		public int Top
		{
			get
			{
				return Location.Y;
			}
		}

		public int Bottom
		{
			get
			{
				return Location.Y + Size.Height;
			}
		}


		/// <summary>
		/// Constructor
		/// </summary>
		public DrawableComponent()
		{
			initComponent();
		}

		private void initComponent()
		{
			this.form = JVMSMain.JvmsForm;			

			contextMenu = new ContextMenu();
			contextMenu.Popup += new EventHandler( contextMenuEventHandler );
		}

		/// <summary>
		/// Flashes the border of this component with the specified color.
		/// </summary>
		/// <param name="c">The desired border color.</param>
		public void flashBorder( Color c )
		{
			Pen flashPen = new Pen( c, 2.0f );

			Timer t = new Timer();
			t.Interval = 200;
			t.Tick += new EventHandler( flashBorderTick );

			SELECTED_BORDER_PEN = flashPen;
			invalidate();

			t.Start();
		}

		/// <summary>
		/// Called when the flash border timer has ticked.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		private void flashBorderTick( Object source, EventArgs e )
		{
			SELECTED_BORDER_PEN = new Pen( Color.Black, 2.0f );
			invalidate();
			( (Timer)source ).Stop();
		}

		private void contextMenuEventHandler(System.Object sender, System.EventArgs e)
		{
			ContextMenu.MenuItems.Clear();
			if( getWrappedObject() is FileSystemObject )
			{
				MenuItem generateXMLMenuItem = new MenuItem( "Generate XML" );
				generateXMLMenuItem.Click += new EventHandler( generateXML );

				ContextMenu.MenuItems.Add( generateXMLMenuItem );
			}

			MenuItem validateMenuItem = new MenuItem( "Validate" );
			validateMenuItem.Click += new EventHandler( handleValidate );
			ContextMenu.MenuItems.Add( validateMenuItem );

			MenuItem deleteMenuItem = new MenuItem( "Delete" );
			deleteMenuItem.Click += new EventHandler( handleDelete );
			ContextMenu.MenuItems.Add( deleteMenuItem );

		}

		private void handleValidate( object sender, System.EventArgs e )
		{
			//first setup all of the relationships.

			setupRelationships();
			RuleEngine engine = RuleEngine.Instance;
			if ( engine.validate( JVMSMain.JvmsForm.CurrentView, this ) )
			{
				flashBorder( Color.FromArgb( 255, 128, 128 ) );
			}
			else
			{
				flashBorder( Color.FromArgb( 128, 128, 255 ) );
			}
		}

		private void handleDelete(object sender, System.EventArgs e )
		{
			DialogResult r = 
				MessageBox.Show( "Deleting this component will remove all references to it.  " +
					"Are you sure you want to delete this item?", 
					"Delete?", 
					MessageBoxButtons.YesNo, 
					MessageBoxIcon.Question );
			if( r == DialogResult.Yes )
			{
				deleteComponent( sender, e );
				JVMSMain.JvmsForm.CurrentView.Refresh();
				JVMSMain.JvmsForm.CurrentAttributeSheetControl.clearAttributeSheet();
				JVMSMain.JvmsForm.TaskListControl.clearItem( this );
				ProjectManager.Instance.ChangedFlag = true;
			}
		}

		/// <summary>
		/// Thiw will perform the action of deleting this component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public abstract void deleteComponent(object sender, System.EventArgs e);

		/// <summary>
		/// Deserialization Constructor
		/// 
		/// Creates a new View object.  Cannot be called directly; subclass View
		/// to obtain this functionality.
		/// </summary>
		protected DrawableComponent(SerializationInfo info, StreamingContext context)
		{
			//selected = info.GetBoolean( "selected" );
			//form = (jvms.presentation.JVMSForm)info.GetValue( "form", form.GetType() );
			gridSize = info.GetInt32( "gridSize" );
			this.Location = (Point)info.GetValue( "location", typeof( Point ) );
			this.Size = (Size)info.GetValue( "size", typeof( Size ) );

			initComponent();
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public virtual void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			//info.AddValue( "selected", selected, selected.GetType() );
			//info.AddValue( "form", form, form.GetType() );
			info.AddValue( "gridSize", gridSize, gridSize.GetType() );
			info.AddValue( "location", location );
			info.AddValue( "size", Size );
		}

		private void generateXML(object sender, System.EventArgs e)
		{			
			String projDir = ProjectManager.Instance.Project.ProjectDirectory;
			if( projDir == null ||
				projDir.Length == 0 )
			{
				MessageBox.Show( 
					"The project output directory must be set in the project settings.", 
					"Error Generating XML", 
					MessageBoxButtons.OK, 
					MessageBoxIcon.Warning );
			}
			else
			{
				try
				{
					RuleEngine engine = RuleEngine.Instance;
					//first validate the component before allowing for xml generation.
					if( engine.validate( JVMSMain.JvmsForm.CurrentView, this ) )
					{
						MessageBox.Show( 
							"There were validation errors.", 
							"Error!", 
							MessageBoxButtons.OK,
							MessageBoxIcon.Error );
					}
					else
					{
						//no errors so start the recursive call of writing XML
						writeXML( (FileSystemObject)getWrappedObject() );
						MessageBox.Show( 
							"XML successfully generated to " + 
							ProjectManager.Instance.Project.ProjectDirectory + ".", 
							"Successful", 
							MessageBoxButtons.OK, 
							MessageBoxIcon.Information );
					}
				}
				catch( Exception exception )
				{
					Console.WriteLine( exception );
					if( exception.InnerException != null )
					{
						Console.WriteLine( exception.InnerException );
					}
					MessageBox.Show( 
						"Cannot generate XML.\n" + exception.Message, 
						"Error Generating XML", 
						MessageBoxButtons.OK, 
						MessageBoxIcon.Error );
				}
			}
		}

		private void setupRelationships()
		{
			RelationshipManager manager = RelationshipManager.Instance;				
			foreach( jvms.datamodel.sad.Application app in ProjectManager.Instance.Project.Applications )
			{
				app.Connections = 
					manager.getConnectionsForInstantiations( app.ComponentPlacements );
			}

			foreach( DeviceConfiguration dcd in ProjectManager.Instance.Project.DeviceConfigurations )
			{
				IList instances = new ArrayList();
				foreach( ComponentPlacement placement in dcd.Partitioning )
				{
					instances.Add( placement.Instantiation );
				}				
				dcd.Connections = manager.getConnectionsForInstantiations( instances );
			}
		}

		private void writeXML( FileSystemObject theObject )
		{
			if( theObject is jvms.datamodel.sad.Application )
			{
				jvms.datamodel.sad.Application app = (jvms.datamodel.sad.Application)theObject;

				foreach( NonSCAComponent component in app.getComponentFiles() )
				{
					writeXML( component );
				}
			}
			else if( theObject is DeviceConfiguration )
			{
				DeviceConfiguration dcd = (DeviceConfiguration)theObject;

				foreach( NonSCAComponent component in ((DeviceConfiguration)theObject).getComponentFiles() )
				{
					writeXML( component );
				}
			}
			else if( theObject is NonSCAComponent )
			{
				NonSCAComponent component = (NonSCAComponent)theObject;
				if( component.Properties != null )
				{
					writeXML( component.Properties );
				}
				if( component is SCAComponent )
				{
					//we want the SCD in the same directory as the SPD
					((SCAComponent)component).Descriptor.Directory = component.Directory;
					writeXML( ((SCAComponent)component).Descriptor );
				}
			}
			//this must happen last because we populate the connectinterfaces for the
			//SAD and DCD
			XMLWriter writer = new XMLWriter();
			writer.writeObject( theObject, ProjectManager.Instance.Project );

		}

		public bool Selected
		{
			get
			{				
				return selected;
			}
			set
			{
				selected = value;
				if( selected )
				{
					BringToFront();
				}
				invalidate();
			}
		}

		public void invalidate()
		{
			Region region = new Region( new Rectangle( 0, 0, 0, 0 ) );
			
			Rectangle currentRectangle = new Rectangle( Location, Size );			
			Rectangle lastRectangle = new Rectangle( lastLocation, lastSize );
			currentRectangle = 
				scaleRectangle( currentRectangle, JVMSMain.JvmsForm.CurrentView.ZoomFactor );
			lastRectangle =
				scaleRectangle( lastRectangle, lastZoomFactor );
			
			region.Union( currentRectangle );
			region.Union( lastRectangle );
			form.CurrentView.Invalidate( region );
		}

		private Rectangle scaleRectangle( Rectangle rect, float zoomFactor )
		{			
			Rectangle retval = new Rectangle(
				(int)(rect.X * zoomFactor)-2,
				(int)(rect.Y * zoomFactor)-2,
				(int)(rect.Width * zoomFactor)+10,
				(int)(rect.Height * zoomFactor)+10 );
			return retval;
		}

		public bool isOnComponent( float x, float y )
		{
			float xLocation = Location.X * JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			float yLocation = Location.Y * JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			float width = Size.Width * JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			float height = Size.Height * JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			//p.X = (int)(p.X * JVMSMain.JvmsForm.CurrentView.ZoomFactor);
			//p.Y = (int)(p.Y * JVMSMain.JvmsForm.CurrentView.ZoomFactor);
			return x >= xLocation && x < xLocation + width &&
				y >= yLocation && y < yLocation + height;
		}

		public void mouseDown( object sender, MouseEventArgs ex)
		{
			if( isOnComponent( ex.X, ex.Y ) )
			{
				dragging = true;
				float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
				draggingX = (int)(location.X*zoomFactor - ex.X);
				draggingY = (int)(location.Y*zoomFactor - ex.Y);
				//draggingX = (int)(draggingX / zoomFactor);
				//draggingY = (int)(draggingY / zoomFactor);
				form.SelectedComponent = this;
				this.BringToFront();
			}
			else
			{
				if( form.SelectedComponent == this )
				{
					form.SelectedComponent = null;
				}
			}
		}

		public void BringToFront()
		{
			if( form.CurrentView.DrawableComponents.Contains( this ) )
			{
				form.CurrentView.DrawableComponents.Remove( this );
				form.CurrentView.DrawableComponents.Add( this );
			}
		}

		public void mouseMove( object sender, MouseEventArgs ex)
		{
			if( dragging && Selected)
			{
				float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
				int x = ex.X + draggingX;
				x = (int)(x / zoomFactor);
				x = (int)Math.Round( (double)x / gridSize ) * gridSize;
				int y = ex.Y + draggingY;
				y = (int)(y / zoomFactor);
				y = (int)Math.Round( (double)y / gridSize ) * gridSize;

				if( Location.X != x || Location.Y != y )
				{		               			
					Location = new Point( (int)x, (int)y );
					RelationshipManager.Instance.invalidateRelationshipsConnectingTo( this );
					invalidate();
				}
			}
		}

		public virtual void mouseUp( object sender, MouseEventArgs ex)
		{
			dragging = false;
			if( isOnComponent( ex.X, ex.Y ) && ex.Button == MouseButtons.Right )
			{
				contextMenu.Show( form.CurrentView, new Point( ex.X, ex.Y ) );
			}
		}

		protected Pen getBorderPen()
		{
			Pen retval = null;
			if( selected )
			{
				retval = SELECTED_BORDER_PEN;
			}
			else
			{
				retval = UNSELECTED_BORDER_PEN;
			}
			return retval;
		}

		/// <summary>
		/// This is the wrapped object that is being drawn.
		/// </summary>
		/// <returns>The wrapped object.</returns>
		public abstract Object getWrappedObject();

		public void paint( Object source, PaintEventArgs e )
		{
			Rectangle clippedRectangle = scaleRectangle( new Rectangle( Location, Size ), 
				JVMSMain.JvmsForm.CurrentView.ZoomFactor );
			if( e.ClipRectangle.IntersectsWith( clippedRectangle ) )
			{				
				paintComponent( source, e );
				lastLocation = new Point( Location.X, Location.Y );
				lastSize = new Size( this.Size.Width, this.Size.Height );
				lastZoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			}
		}

		/// <summary>
		/// Called when the view should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public abstract void paintComponent( Object source, PaintEventArgs e );

		/// <summary>
		/// This will draw a string and truncate it if necessary to make it fit on one
		/// line in the rectangle.
		/// 
		/// So
		/// A really long string to print.
		/// while turn into
		/// A really long...
		/// </summary>
		/// <param name="theString"></param>
		/// <param name="font"></param>
		/// <param name="rect"></param>
		/// <param name="stringFormat"></param>
		/// <param name="g"></param>
		protected void drawTruncatedString( 
			String theString, 
			Font font, 
			Rectangle rect, 
			StringFormat stringFormat,
			Brush brush,
			Graphics g )
		{
			int charactersFitted;
			int unusedLines;
			SizeF rectangleSize = new SizeF( rect.Width, rect.Height );
			SizeF size = g.MeasureString( "..." + theString,
				font, 
				rectangleSize, 
				stringFormat, 
				out charactersFitted, 
				out unusedLines );
			if( charactersFitted < theString.Length+3 )
			{
				theString = theString.Substring(0, charactersFitted-3) + "...";
			}
			g.DrawString( theString, 
				font,
				brush,
				rect,
				stringFormat
				);
		}
	}
}
