using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;
using System.Runtime.Serialization;

using jvms.datamodel.dcd;
using jvms.datamodel.sad;
using jvms.datamodel.scd;
using jvms.datamodel.spd;
using jvms.project;
using jvms.system;

namespace jvms.presentation.drawablecomponent
{
	[Serializable]
	/// <summary>
	/// Summary description for NonSCAComponentDrawableComponent.
	/// </summary>
	public class NonSCAComponentDrawableComponent : DrawableComponent
	{
		private const int TITLE_HEIGHT = 25;

		private NonSCAComponent component = new NonSCAComponent();

		private static Icon ICON;

		/// <summary>
		/// The component that is being drawn.
		/// </summary>
		public NonSCAComponent NonSCAComponent
		{
			get
			{
				return component;
			}
			set
			{
				component = value;
			}
		}

		/// <summary>
		/// This is a static constructor which means that it is only called once when the 
		/// class is loaded.
		/// </summary>
		static NonSCAComponentDrawableComponent()
		{
			System.Reflection.Assembly myApp = System.Reflection.Assembly.GetExecutingAssembly();
			Stream stream = null;

			stream = myApp.GetManifestResourceStream("jvms.resources.NonSCAIcon_Small.ico");
			ICON = new Icon( stream );
			stream.Close();
		}

		public NonSCAComponentDrawableComponent()
		{
			ProjectManager.Instance.Project.addComponent( component );
			this.Size = new Size(250, 100 );
			//
			// TODO: Add constructor logic here
			//
			initComponent();
		}

		/// <summary>
		/// Creates a new ApplicationAssemblyView.
		/// </summary>
		protected NonSCAComponentDrawableComponent(SerializationInfo info, StreamingContext context) :
			base( info, context )
		{
			component = (NonSCAComponent)info.GetValue( "component", typeof(NonSCAComponent) );
			StatusBarManager.Instance.StatusText = "Loading " + getWrappedObject().ToString();
			initComponent();
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData( info, context );
			info.AddValue( "component", component, component.GetType() );
		}

		private void initComponent()
		{
			component.Type = "sca_non_compliant";
		}

		public override void mouseUp( object sender, MouseEventArgs e )
		{
			base.mouseUp( sender, e );
			if( isOnComponent( e.X, e.Y ) )
			{
				RelationshipManager.Instance.nonScaComponentSelected( this );
			}
		}

		public override Object getWrappedObject()
		{
			return component;
		}

		/// <summary>
		/// Thiw will perform the action of deleting this component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public override void deleteComponent(object sender, System.EventArgs e)
		{
			//first delete myself
			ViewManager.Instance.AppCompView.removeDrawableComponent( this );
			ViewManager.Instance.PlatCompView.removeDrawableComponent( this );
			//then delete all references to instances of this
			deleteInstancesInList( ViewManager.Instance.PlatAssmView.DrawableComponents, sender, e );
			deleteInstancesInList( ViewManager.Instance.AppAssmView.DrawableComponents, sender, e );
			ProjectManager.Instance.Project.Components.Remove( getWrappedObject() );
			RelationshipManager.Instance.removeRelationshipsToDrawableComponent( this );
		}

		private void deleteInstancesInList( IList controls, Object sender, EventArgs e )
		{
			IList controlsCopy = new ArrayList( controls );
			foreach( DrawableComponent dc in controlsCopy )
			{
				if( dc.getWrappedObject() is ApplicationComponentInstantiation )
				{
					ApplicationComponentInstantiation instance = 
						(ApplicationComponentInstantiation)dc.getWrappedObject();
					if( instance.Component == getWrappedObject() )
					{
						dc.deleteComponent( sender, e );
					}
				}
				else if( dc.getWrappedObject() is ComponentPlacement )
				{
					ComponentPlacement instance = 
						(ComponentPlacement)dc.getWrappedObject();
					if( instance.ComponentFile == getWrappedObject() )
					{
						dc.deleteComponent( sender, e );
					}
				}
			}
		}

		/// <summary>
		/// Called when the view should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public override void paintComponent( Object source, PaintEventArgs e )
		{
			JVMSProperties props = JVMSProperties.Instance;
			Brush headerBrush = new SolidBrush( (Color)props.getProperty("color.nonscaheader", Color.LemonChiffon ) );
			Brush bodyBrush = new SolidBrush( (Color)props.getProperty("color.content", Color.Beige ) );
			e.Graphics.FillRectangle( bodyBrush, 0 ,0,Size.Width,Size.Height );
			e.Graphics.FillRectangle( headerBrush, 0, 0,Size.Width,TITLE_HEIGHT );

			//draw border
			Pen smallPen = new Pen( Color.Black, 1 );
			Pen borderPen = getBorderPen();
			e.Graphics.DrawRectangle( borderPen, borderPen.Width-1, borderPen.Width-1, Size.Width-borderPen.Width, Size.Height-borderPen.Width );

			Font titleFont = new Font( "Arial", 12 );
			Brush titleBrush = Brushes.Black;	
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			Rectangle targetRect = new Rectangle( 
				(int)(5*zoomFactor), 
				(int)(5 * zoomFactor), 
				(int)(ICON.Width*zoomFactor),
				(int)(ICON.Height*zoomFactor) );
			e.Graphics.DrawIcon( ICON, targetRect );
			StringFormat titleFormat = new StringFormat();
			titleFormat.Alignment = StringAlignment.Near;

			//draw component name and the line
			Rectangle titleRectangle = new Rectangle( 19, 5, Size.Width-19, 20 );
			drawTruncatedString( component.Name, titleFont, titleRectangle, titleFormat, titleBrush, e.Graphics );
			e.Graphics.DrawLine( smallPen, new Point( 0, TITLE_HEIGHT ), 
				                           new Point( Size.Width-1, TITLE_HEIGHT ) );
		}		

		/// <summary>
		/// This will return a string representation of this drawable component.
		/// </summary>
		/// <returns></returns>
		public override String ToString()
		{
			return component.Name;
		}
	}
}
