using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;
using System.Runtime.Serialization;

using jvms.datamodel.prf;

using jvms.project;
using jvms.system;

namespace jvms.presentation.drawablecomponent
{
	[Serializable]
	/// <summary>
	/// Summary description for NonSCAComponentDrawableComponent.
	/// </summary>
	public class PropertiesDrawableComponent : DrawableComponent
	{
		private const int TITLE_HEIGHT = 25;
		private const int PROPERTY_HEIGHT = 10;
		private const int PROPERTY_SPACING = 5;
		private Font PROPERTY_FONT = new Font( "Arial", 8 );

		private Properties properties = new Properties();

		private static Icon ICON;

		/// <summary>
		/// This will get the properties that are being drawn.
		/// </summary>
		public Properties Properties
		{
			get
			{
				return properties;
			}
			set
			{
				properties = value;
			}
		}

		/// <summary>
		/// This is a static constructor which means that it is only called once when the 
		/// class is loaded.
		/// </summary>
		static PropertiesDrawableComponent()
		{
			System.Reflection.Assembly myApp = System.Reflection.Assembly.GetExecutingAssembly();
			Stream stream = null;

			stream = myApp.GetManifestResourceStream("jvms.resources.PropsIcon_Small.ico");
			ICON = new Icon( stream );
			stream.Close();
		}

		/// <summary>
		/// Constructor
		/// </summary>
		public PropertiesDrawableComponent()
		{
			ProjectManager.Instance.Project.addProperties( properties );
			this.Size = new Size(200, 0 );
			initComponent();
		}

		/// <summary>
		/// Creates a new ApplicationAssemblyView.
		/// </summary>
		protected PropertiesDrawableComponent(SerializationInfo info, StreamingContext context) :
			base( info, context )
		{
			properties = (Properties)info.GetValue( "properties", typeof(Properties) );
			StatusBarManager.Instance.StatusText = "Loading " + getWrappedObject().ToString();
			initComponent();
		}

		/// <summary>
		/// Serialization Function
		/// </summary>
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData( info, context );
			info.AddValue( "properties", properties, properties.GetType() );
		}

		private void initComponent()
		{
		}

		public override void mouseUp( object sender, MouseEventArgs e )
		{
			base.mouseUp( sender, e );
			if( isOnComponent( e.X, e.Y ) )
			{
				RelationshipManager.Instance.propertiesSelected( this );
			}
		}

		/// <summary>
		/// Thiw will perform the action of deleting this component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public override void deleteComponent(object sender, System.EventArgs e)
		{
			ViewManager.Instance.AppCompView.removeDrawableComponent( this );
			ViewManager.Instance.PlatCompView.removeDrawableComponent( this );
			ProjectManager.Instance.Project.Properties.Remove( getWrappedObject() );
			RelationshipManager.Instance.removeRelationshipsToDrawableComponent( this );
		}

		public override Object getWrappedObject()
		{
			return properties;
		}

		/// <summary>
		/// Called when the view should paint itself.
		/// </summary>
		/// <param name="source">A reference to the source of the event.</param>
		/// <param name="e">The event's data.</param>
		public override void paintComponent( Object source, PaintEventArgs e )
		{

			JVMSProperties props = JVMSProperties.Instance;
			Brush headerBrush = new SolidBrush( (Color)props.getProperty("color.propsheader", Color.PowderBlue ) );
			Brush bodyBrush = new SolidBrush( (Color)props.getProperty("color.content", Color.Beige ) );

			int calculatedHeight = TITLE_HEIGHT+5;
			calculatedHeight += (properties.Simples.Count +
				                properties.SimpleSequences.Count +
								properties.Structs.Count +
								properties.Test.Count) * 
				(PROPERTY_HEIGHT + PROPERTY_SPACING);

			//if there are no properties then simply add some spacing.
			if( calculatedHeight == TITLE_HEIGHT+5 )
			{
				calculatedHeight += 20;
			}
			
			this.Size = new Size( this.Size.Width, calculatedHeight );
			e.Graphics.FillRectangle( bodyBrush, 0 ,0,Size.Width,Size.Height );
			e.Graphics.FillRectangle( headerBrush, 0, 0,Size.Width,TITLE_HEIGHT );

			//draw border
			Pen smallPen = new Pen( Color.Black, 1 );
			Pen borderPen = getBorderPen();
			e.Graphics.DrawRectangle( borderPen, borderPen.Width-1, borderPen.Width-1, Size.Width-borderPen.Width, Size.Height-borderPen.Width );

			int propertyPosition = TITLE_HEIGHT+5;
			Font titleFont = new Font( "Arial", 12 );
			Brush titleBrush = Brushes.Black;
		
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			Rectangle targetRect = new Rectangle( 
				(int)(5*zoomFactor), 
				(int)(5 * zoomFactor), 
				(int)(ICON.Width*zoomFactor),
				(int)(ICON.Height*zoomFactor) );
			e.Graphics.DrawIcon( ICON, targetRect );

			StringFormat titleFormat = new StringFormat();
			titleFormat.Alignment = StringAlignment.Near;

			//draw component name and the line
			Rectangle titleRectangle = new Rectangle( 19, 5, Size.Width-19, 20 );
			drawTruncatedString( properties.Description, titleFont, titleRectangle, titleFormat, titleBrush, e.Graphics );
			e.Graphics.DrawLine( smallPen, new Point( 0, TITLE_HEIGHT ), 
				                           new Point( Size.Width-1, TITLE_HEIGHT ) );
			foreach( Simple simple in properties.Simples )
			{
				String stringToDraw = simple.Name + ":" + simple.Type;
				e.Graphics.DrawString( stringToDraw, PROPERTY_FONT, titleBrush, 5, propertyPosition ); 
				propertyPosition += PROPERTY_HEIGHT + PROPERTY_SPACING;
			}
			foreach( SimpleSequence simpleSequence in properties.SimpleSequences )
			{
				String stringToDraw = simpleSequence.Name + ":" + simpleSequence.Type;
				stringToDraw += "(";
				for( int i=0; i<simpleSequence.Values.Count; i++ )
				{
					stringToDraw += simpleSequence.Values[i];
					if( i != simpleSequence.Values.Count-1 )
					{
						stringToDraw += ",";
					}
				}
				stringToDraw += ")";
				e.Graphics.DrawString( stringToDraw, PROPERTY_FONT, titleBrush, 5, propertyPosition ); 
				propertyPosition += PROPERTY_HEIGHT + PROPERTY_SPACING;
			}
			foreach( Struct aStruct in properties.Structs )
			{
				String stringToDraw = aStruct.Name + ":struct";
				e.Graphics.DrawString( stringToDraw, PROPERTY_FONT, titleBrush, 5, propertyPosition ); 
				propertyPosition += PROPERTY_HEIGHT + PROPERTY_SPACING;
			}
		}		

		/// <summary>
		/// This will return a string representation of this drawable component.
		/// </summary>
		/// <returns></returns>
		public override String ToString()
		{
			return properties.Description;
		}
	}
}
