using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

using jvms.datamodel.dcd;
using jvms.presentation;
using jvms.presentation.drawablecomponent;
using jvms.system;

namespace jvms.presentation.relationships
{
	/// <summary>
	/// Represents a relationship between a component instance and an Application.
	/// </summary>
	[Serializable]
	public class ApplicationRelationship : IRelationship
	{
		private bool selected = false;
		private Object tag;
		private ArrayList verticies;
		private static Pen normalPen = new Pen( Color.DarkGray, 1.0f );
		private static Pen selectedPen = new Pen( Color.DarkGray, 2.0f );

		private ApplicationDrawableComponent application;
		private ApplicationComponentInstantiationDrawableComponent component;

		/// <summary>Gets or sets the application component.</summary>
		/// <value>The Application component.</value>
		public ApplicationDrawableComponent Application
		{
			get
			{
				return application;
			}

			set
			{
				application = value;
			}
		}

		/// <summary>Gets or sets the component.</summary>
		/// <value>The component.</value>
		public ApplicationComponentInstantiationDrawableComponent Component
		{
			get
			{
				return component;
			}

			set
			{
				component = value;
			}
		}

		/// <summary>Sets or gets data associated with the relationship.</summary>
		/// <value>A data object.</value>
		public Object Tag
		{
			get
			{
				return tag;
			}

			set
			{
				tag = value;
			}
		}

		/// <summary>Returns true if the relationship is selected, false otherwise.</summary>
		/// <value>A bool.</value>
		public bool Selected
		{
			get
			{
				return selected;
			}

			set
			{
				selected = value;
			}
		}
		/// <summary>Gets or sets a list of verticies in the relationship.</summary>
		/// <value>A list of points.</value>
		public IList Verticies
		{
			get
			{
				return verticies;
			}

			set
			{
				verticies = new ArrayList( value );
			}
		}

		/// <summary>
		/// Creates a new ApplicationRelationship.
		/// </summary>
		public ApplicationRelationship()
		{
			verticies = new ArrayList();

			verticies.Add( new Vertex( new Point( 0, 0 ) ) );
			verticies.Add( new Vertex( new Point( 0, 0 ) ) );
		}

		/// <summary>
		/// Draws the relationship.
		/// </summary>
		/// <param name="clientRect">A area in which to draw the relationship.</param>
		/// <param name="g">The graphics context to use.</param>
		public void drawRelationship( Rectangle clientRect, Graphics g )
		{
			int containerMidX = application.Location.X + ( application.Width / 2 );
			int containerMidY = application.Location.Y + ( application.Height / 2 );
			int compMidX = component.Location.X + ( component.Width / 2 );
			int compMidY = component.Location.Y + ( component.Height / 2 );

			containerMidX = (int)Math.Round( (double)containerMidX / 10 ) * 10;
			containerMidY = (int)Math.Round( (double)containerMidY / 10 ) * 10;
			compMidX = (int)Math.Round( (double)compMidX / 10 ) * 10;
			compMidY = (int)Math.Round( (double)compMidY / 10 ) * 10;

			( (Vertex)verticies[0] ).Point = new Point( containerMidX, containerMidY );
			( (Vertex)verticies[1] ).Point = new Point( compMidX, compMidY );

			Pen drawingPen = normalPen;
			if ( selected )
			{
				drawingPen = selectedPen;
			}

			g.DrawLine( drawingPen, ( (Vertex)verticies[0] ).Point, ( (Vertex)verticies[1] ).Point );
		}
		
		/// <summary>
		/// Called when the relationship should redraw itself.
		/// </summary>
		public void update()
		{
			Point firstPoint = ( (Vertex)verticies[0] ).Point;
			Point secondPoint = ( (Vertex)verticies[1] ).Point;

			int topLeftX = Math.Min( firstPoint.X, secondPoint.X ) - 1;
			int topLeftY = Math.Min( firstPoint.Y, secondPoint.Y ) - 1;
			int bottomRightX = Math.Max( firstPoint.X, secondPoint.X ) + 1;
			int bottomRightY = Math.Max( firstPoint.Y, secondPoint.Y ) + 1;

			Rectangle rect =
				new Rectangle( topLeftX, topLeftY, bottomRightX - topLeftX, bottomRightY - topLeftY );

			rect = scaleRectangle( rect, JVMSMain.JvmsForm.CurrentView.ZoomFactor );

			JVMSMain.JvmsForm.CurrentView.Invalidate( rect );
		}

		private Rectangle scaleRectangle( Rectangle rect, float zoomFactor )
		{			
			Rectangle retval = new Rectangle(
				(int)(rect.X * zoomFactor)-2,
				(int)(rect.Y * zoomFactor)-2,
				(int)(rect.Width * zoomFactor)+10,
				(int)(rect.Height * zoomFactor)+10 );
			return retval;
		}

		/// <summary>
		/// Returns whether the specified point is on a draggable portion of the relationship.
		/// </summary>
		/// <param name="p">The point in question.</param>
		/// <returns>Always returns false.</returns>
		public bool isDragging( Point p )
		{
			return false;
		}

		/// <summary>
		/// Notifies that the relationship has been dragged to the specified point.
		/// </summary>
		/// <param name="p">The point.</param>
		public void dragNotify( Point p )
		{
		}

		/// <summary>
		/// Notifies of the relationship being scrolled.
		/// </summary>
		/// <param name="direction">The direction.</param>
		/// <param name="amount">The amount.</param>
		public void scrollNotify( Orientation direction, int amount )
		{
		}

		/// <summary>
		/// From IRelationship.  Determines if given point is on line.
		/// </summary>
		/// <param name="p">The desired point.</param>
		/// <returns>True if point is on line, false otherwise.</returns>
		public bool isIntersection( Point p )
		{
			bool retVal = false;
			Point p1 = ( (Vertex)verticies[0] ).Point;
			Point p2 = ( (Vertex)verticies[1] ).Point;
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			p1.X = (int)(p1.X * zoomFactor);
			p1.Y = (int)(p1.Y * zoomFactor);
			p2.X = (int)(p2.X * zoomFactor);
			p2.Y = (int)(p2.Y * zoomFactor);

			double segmentLength = Math.Sqrt( Math.Pow( p2.X - p1.X, 2 ) + Math.Pow( p2.Y - p1.Y, 2 ) );
			double len1 = Math.Sqrt( Math.Pow( p.X - p1.X, 2 ) + Math.Pow( p.Y - p1.Y, 2 ) );
			double len2 = Math.Sqrt( Math.Pow( p.X - p2.X, 2 ) + Math.Pow( p.Y - p2.Y, 2 ) );
			double diff = ( Math.Round( len1 ) + Math.Round( len2 ) ) - Math.Round( segmentLength );

			if ( ( diff >= -1 ) && ( diff <= 1 ) )
			{
				retVal = true;
			}
			return retVal;
		}

		/// <summary>
		/// This will perform the operation of deleting this relationship and
		/// removing all references to other components.
		/// </summary>
		public void delete()
		{
			application.Application.ComponentPlacements.Remove( component.Instantiation );
		}

		public override String ToString()
		{
			return application.Application.Name + " - " + component.Instantiation.UsageName;
		}
	}
}
