using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

using jvms.datamodel.common;
using jvms.datamodel.dcd;
using jvms.datamodel.sad;
using jvms.datamodel.scd;
using jvms.datamodel.spd;
using jvms.presentation.drawablecomponent;
using jvms.system;

namespace jvms.presentation.relationships
{
	/// <summary>
	/// Represents a device used by this component relationship.
	/// </summary>
	[Serializable]
	public class DeviceUsedByRelationship : SinglePortConnectionRelationship, IRelationship
	{
		private const int BOX_SIZE = 5;

		private bool selected = false;
		private Object tag;
		private ArrayList verticies;
		private static Pen normalPen = new Pen( Color.Orange, 1.0f );
		private static Pen selectedPen = new Pen( Color.Orange, 2.0f );
		private static Font textFont = new Font( "Courier", 7.0f );
		private Point lastPortPoint;
		private ConnectInterface connectInterface;
		private UsesDevice usesDevice;
		private ComponentPortEndPoint usesPort;

		/// <summary>
		/// This is the port for the uses device relationship.
		/// </summary>
		public ComponentPortEndPoint UsesPort
		{
			get{ return usesPort; }
			set{ usesPort = value; }
		}

		/// <summary>Gets or sets a data object associated with the relationship.</summary>
		/// <value>A data object.</value>
		public Object Tag
		{
			get
			{
				return tag;
			}

			set
			{
				tag = value;
			}
		}

		/// <summary>
		/// This will return the uses device of this relationship.
		/// </summary>
		public UsesDevice UsesDevice
		{
			get
			{
				return usesDevice;
			}
			set
			{
				usesDevice = value;
			}
		}

		/// <summary>Gets or sets a flag indicating if the relationship is selected.</summary>
		/// <value>True if the relationship is selected, false otherwise.</value>
		public bool Selected
		{
			get
			{
				return selected;
			}

			set
			{
				selected = value;
			}
		}

		/// <summary>Gets or sets a list of points in the relationship.</summary>
		/// <value>A list of verticies.</value>
		public IList Verticies
		{
			get
			{
				return verticies;
			}

			set
			{
				verticies = new ArrayList( value );
			}
		}

		/// <summary>
		/// Creates a new DeviceUsedByRelationship.
		/// </summary>
		public DeviceUsedByRelationship()
		{
			verticies = new ArrayList();
			connectInterface = new ConnectInterface();
		}

		/// <summary>
		/// Called when the relationship has been set up.
		/// </summary>
		public override void initRelationship()
		{
			Point portPoint = Component.getPortLocation( Port );
			bool onLeft = Component.isPortOnLeftSide( Port );

			verticies.Add( new Vertex( portPoint ) );
			
			if ( onLeft )
			{
				verticies.Add( new Vertex( new Point( portPoint.X - 50, portPoint.Y ) ) );
			}
			else
			{
				verticies.Add( new Vertex( new Point( portPoint.X + 50, portPoint.Y ) ) );
			}

			Point endPoint = ( (Vertex)verticies[1] ).Point;
			if ( onLeft )
			{
				verticies.Add( new Vertex( new Point( endPoint.X, endPoint.Y - BOX_SIZE ) ) );
				verticies.Add( new Vertex( new Point( endPoint.X - BOX_SIZE, endPoint.Y - BOX_SIZE ) ) );
				verticies.Add( new Vertex( new Point( endPoint.X - BOX_SIZE, endPoint.Y + BOX_SIZE ) ) );
				verticies.Add( new Vertex( new Point( endPoint.X, endPoint.Y + BOX_SIZE ) ) );
				verticies.Add( new Vertex( new Point( endPoint.X, endPoint.Y ) ) );
			}
			else
			{
				verticies.Add( new Vertex( new Point( endPoint.X, endPoint.Y - BOX_SIZE ) ) );
				verticies.Add( new Vertex( new Point( endPoint.X + BOX_SIZE, endPoint.Y - BOX_SIZE ) ) );
				verticies.Add( new Vertex( new Point( endPoint.X + BOX_SIZE, endPoint.Y + BOX_SIZE ) ) );
				verticies.Add( new Vertex( new Point( endPoint.X, endPoint.Y + BOX_SIZE ) ) );
				verticies.Add( new Vertex( new Point( endPoint.X, endPoint.Y ) ) );
			}
		}

		/// <summary>
		/// This will get the connectInterface that this relationship represents.
		/// </summary>
		/// <returns>The connection for this relationship.</returns>
		public override ConnectInterface getConnectInterface()
		{
			Object wrappedObject = Component.getWrappedObject();
			SCAComponent component = null;
			ComponentInstantiation instance = null;
			if( wrappedObject is ApplicationComponentInstantiation )
			{
				instance = (ComponentInstantiation)wrappedObject;
				component = (SCAComponent)((ApplicationComponentInstantiation)wrappedObject).Component;
			}
			else if( wrappedObject is ComponentPlacement )
			{
				instance = ((ComponentPlacement)wrappedObject).Instantiation;
				component = (SCAComponent)((ComponentPlacement)wrappedObject).ComponentFile;
			}
			else
			{
				throw new Exception( "DeviceUsedByRelationship is invalid wrappedObject=" + wrappedObject );
			}
			if( component.Descriptor.Features.UsesPorts.Contains( Port ) )
			{
				ComponentInstantiationPortConnection instanceConnection = 
					new ComponentInstantiationUsesPortConnection();
				connectInterface.UsesPort = instanceConnection;
				instanceConnection.Instance = instance;
				instanceConnection.Port = Port;

				DeviceUsedByThisComponentPortConnection portConnection =
					new DeviceUsedByThisComponentPortConnection( false );
				connectInterface.ProvidesPort = portConnection;
				portConnection.Component = component;
				portConnection.UsesDevice = UsesDevice;
				portConnection.Port = UsesPort;
			}
			else if( component.Descriptor.Features.ProvidesPorts.Contains( Port ) )
			{
				ComponentInstantiationPortConnection instanceConnection = 
					new ComponentInstantiationProvidesPortConnection();
				connectInterface.ProvidesPort = instanceConnection;
				instanceConnection.Instance = instance;
				instanceConnection.Port = Port;

				DeviceUsedByThisComponentPortConnection portConnection =
					new DeviceUsedByThisComponentPortConnection( true );
				connectInterface.UsesPort = portConnection;
				portConnection.Component = component;
				portConnection.UsesDevice = UsesDevice;
				portConnection.Port = UsesPort;
			}
			else if( component.Descriptor.Features.SupportInterfaces.Contains( Port.RepositoryID ) )
			{
				//first setup provides port
				ComponentSupportedInterface instanceConnection = 
					new ComponentSupportedInterface();
				connectInterface.ProvidesPort = instanceConnection;
				instanceConnection.Instance = instance;
				instanceConnection.SupportedIdentifier = Port.RepositoryID;

				//now setup uses port
				DeviceUsedByThisComponentPortConnection portConnection =
					new DeviceUsedByThisComponentPortConnection( true );
				connectInterface.UsesPort = portConnection;
				portConnection.Component = component;
				portConnection.UsesDevice = UsesDevice;
				portConnection.Port = UsesPort;
			}
			else
			{
				throw new Exception( "Port is not part of interface port=" + Port );
			}
			return connectInterface;
		}

		/// <summary>
		/// Draws the relationship.
		/// </summary>
		/// <param name="clientRect">The area in which to draw the relationship.</param>
		/// <param name="g">The graphics context to use.</param>
		public void drawRelationship( Rectangle clientRect, Graphics g )
		{
			Point portPoint = Component.getPortLocation( Port );
			lastPortPoint = portPoint;
			bool onLeft = Component.isPortOnLeftSide( Port );
			String deviceText = "Device Used By";
			String portText = "";

			if ( usesDevice != null )
			{
				deviceText = usesDevice.ToString();
			}
			
			if ( usesPort != null )
			{
				portText = usesPort.UsesName;
			}

			( (Vertex)verticies[0] ).Point = portPoint;
			
			if ( onLeft )
			{
				( (Vertex)verticies[1] ).Point = new Point( portPoint.X - 50, portPoint.Y );
			}
			else
			{
				( (Vertex)verticies[1] ).Point = new Point( portPoint.X + 50, portPoint.Y );
			}

			Point endPoint = ( (Vertex)verticies[1] ).Point;
			RectangleF deviceTextRect = new RectangleF( (float)portPoint.X, portPoint.Y - 17.0f, 75.0f, 12.0f );
			RectangleF portTextRect = new RectangleF( (float)portPoint.X, portPoint.Y + 4.0f, 75.0f, 12.0f );
			StringFormat format = new StringFormat();
			if ( onLeft )
			{
				( (Vertex)verticies[2] ).Point = new Point( endPoint.X, endPoint.Y - 5 );
				( (Vertex)verticies[3] ).Point = new Point( endPoint.X - 5, endPoint.Y - 5 );
				( (Vertex)verticies[4] ).Point = new Point( endPoint.X - 5, endPoint.Y + 5 );
				( (Vertex)verticies[5] ).Point = new Point( endPoint.X, endPoint.Y + 5 );
				( (Vertex)verticies[6] ).Point = new Point( endPoint.X, endPoint.Y );

				deviceTextRect = new RectangleF( portPoint.X - 75.0f, portPoint.Y - 17.0f, 75.0f, 12.0f );
				portTextRect = new RectangleF( portPoint.X - 75.0f, portPoint.Y + 4.0f, 75.0f, 12.0f );
				format.Alignment = StringAlignment.Far;
			}
			else
			{
				( (Vertex)verticies[2] ).Point = new Point( endPoint.X, endPoint.Y - 5 );
				( (Vertex)verticies[3] ).Point = new Point( endPoint.X + 5, endPoint.Y - 5 );
				( (Vertex)verticies[4] ).Point = new Point( endPoint.X + 5, endPoint.Y + 5 );
				( (Vertex)verticies[5] ).Point = new Point( endPoint.X, endPoint.Y + 5 );
				( (Vertex)verticies[6] ).Point = new Point( endPoint.X, endPoint.Y );

				format.Alignment = StringAlignment.Near;
			}

			Point[] pts = new Point[verticies.Count];
			for ( int i = 0; i < verticies.Count; i += 1 )
			{
				pts[i] = ( (Vertex)verticies[i] ).Point;
			}

			Pen drawingPen = normalPen;
			if ( selected )
			{
				drawingPen = selectedPen;
			}

			g.DrawLines( drawingPen, pts );

			
			g.DrawString( deviceText, textFont, Brushes.Black, deviceTextRect, format );
			g.DrawString( portText, textFont, Brushes.Black, portTextRect, format );
		}

		/// <summary>
		/// Called when the relationship should redraw itself.
		/// </summary>
		public void update()
		{
			bool onLeft = Component.isPortOnLeftSide( Port );

			Rectangle updateRect = new Rectangle( 0, 0, 0, 0 );
			JVMSForm form = JVMSMain.JvmsForm;
			float zoomFactor = form.CurrentView.ZoomFactor;

			if ( onLeft )
			{
				RectangleF rect = new RectangleF( ( lastPortPoint.X - 90 ) * zoomFactor,
												  ( lastPortPoint.Y - ( 5 * BOX_SIZE ) ) * zoomFactor,
												  90 * zoomFactor, 10 * BOX_SIZE * zoomFactor );
				updateRect = Rectangle.Round( rect );
			}
			else
			{
				RectangleF rect = new RectangleF( lastPortPoint.X * zoomFactor,
												  ( lastPortPoint.Y - ( 5 * BOX_SIZE ) ) * zoomFactor,
												  90 * zoomFactor, 10 * BOX_SIZE * zoomFactor );
				updateRect = Rectangle.Round( rect );
			}

			form.CurrentView.Invalidate( updateRect );
		}

		/// <summary>
		/// Returns true if the given point is on a draggable point in the relationship.
		/// </summary>
		/// <param name="p">The desired point.</param>
		/// <returns>True if the point is draggable, false otherwise.</returns>
		public bool isDragging( Point p )
		{
			return false;
		}

		/// <summary>
		/// Notifies of the relationship being dragged.
		/// </summary>
		/// <param name="p">The new point dragged to.</param>
		public void dragNotify( Point p )
		{
		}

		/// <summary>
		/// Notifies of the relationship being scrolled.
		/// </summary>
		/// <param name="direction">The direction.</param>
		/// <param name="amount">The amount.</param>
		public void scrollNotify( Orientation direction, int amount )
		{
		}

		/// <summary>
		/// From IRelationship.  Determines if given point is on line.
		/// </summary>
		/// <param name="p">The desired point.</param>
		/// <returns>True if point is on line, false otherwise.</returns>
		public bool isIntersection( Point p )
		{
			bool retVal = false;

			for ( int i = 1; i < verticies.Count; i += 1 )
			{
				Point p1 = ( (Vertex)verticies[i - 1] ).Point;
				Point p2 = ( (Vertex)verticies[i] ).Point;

				if ( isPointOnLine( p, p1, p2 ) )
				{
					retVal = true;
					break;
				}
			}

			return retVal;
		}

		/// <summary>
		/// Returns true if the given point is on the line segment given by two endpoint.
		/// </summary>
		/// <param name="p">The test point.</param>
		/// <param name="p1">One endpoint of the segment.</param>
		/// <param name="p2">The other endpoint of the segment.</param>
		/// <returns>True if the point is on the line, false otherwise.</returns>
		private bool isPointOnLine( Point p, Point p1, Point p2 )
		{
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			p1.X = (int)(p1.X * zoomFactor);
			p1.Y = (int)(p1.Y * zoomFactor);
			p2.X = (int)(p2.X * zoomFactor);
			p2.Y = (int)(p2.Y * zoomFactor);
			bool retVal = false;
			double segmentLength = Math.Sqrt( Math.Pow( p2.X - p1.X, 2 ) + Math.Pow( p2.Y - p1.Y, 2 ) );
			double len1 = Math.Sqrt( Math.Pow( p.X - p1.X, 2 ) + Math.Pow( p.Y - p1.Y, 2 ) );
			double len2 = Math.Sqrt( Math.Pow( p.X - p2.X, 2 ) + Math.Pow( p.Y - p2.Y, 2 ) );
			double diff = ( Math.Round( len1 ) + Math.Round( len2 ) ) - Math.Round( segmentLength );

			if ( ( diff >= -1 ) && ( diff <= 1 ) )
			{
				retVal = true;
			}

			return retVal;
		}

		/// <summary>
		/// This will perform the operation of deleting this relationship and
		/// removing all references to other components.
		/// </summary>
		public void delete()
		{
			//nothing because the connect interface is generated at XML gen time.
		}

		public override String ToString()
		{
			return usesDevice.ID + " - " + usesPort.UsesName;
		}
	}
}
