using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using jvms.datamodel.common;
using jvms.datamodel.dcd;
using jvms.datamodel.sad;
using jvms.system;

namespace jvms.presentation.relationships
{
	/// <summary>
	/// Represents a component instantiation reference relationship.
	/// </summary>
	[Serializable]
	public class InstanceRefRelationship : PortConnectionRelationship, IRelationship
	{
		private const int DRAG_NONE = 0;
		private const int DRAG_MID_X = 1;
		private const int DRAG_PROVIDES_DIST = 2;
		private const int DRAG_USES_DIST = 3;
		private const int DRAG_COMPONENT_DIST = 4;

		private bool selected = false;
		private Object tag;
		private ArrayList verticies;
		private static Pen normalPen = new Pen( Color.Blue, 1.5f );
		private static Pen selectedPen = new Pen( Color.Blue, 3.0f );

		private int dragValue = -1;
		private int dragId = DRAG_NONE;
		private int lastProvDist = Int32.MinValue;
		private int lastUsesDist = Int32.MinValue;
		private int lastCompDist = Int32.MinValue;

		private Point lastUsesPoint;
		private Point lastProvidesPoint;

		private ConnectInterface connectInterface;

		/// <summary>Gets or sets a data object associated with the relationship.</summary>
		/// <value>A data object.</value>
		public Object Tag
		{
			get
			{

				return tag;
			}

			set
			{
				tag = value;
			}
		}

		/// <summary>Gets or sets a flag indicating if the relationship is selected.</summary>
		/// <value>True if the relationship is selected, false otherwise.</value>
		public bool Selected
		{
			get
			{
				return selected;
			}

			set
			{
				selected = value;
			}
		}

		/// <summary>Gets or sets a list of points in the relationship's line.</summary>
		/// <value>A list of Points.</value>
		public IList Verticies
		{
			get
			{
				return verticies;
			}

			set
			{
				verticies = new ArrayList( value );
			}
		}

		/// <summary>
		/// Creates a new InstanceRefRelationship.
		/// </summary>
		public InstanceRefRelationship()
		{
			verticies = new ArrayList();

			connectInterface = new ConnectInterface();			
		}

		/// <summary>
		/// Called after the relationship has been completed.
		/// </summary>
		public override void initRelationship()
		{
			Point providesPoint = ProvidesComponent.getPortLocation( ProvidesPort );
			Point usesPoint = UsesComponent.getPortLocation( UsesPort );

			bool providesOnLeft = ProvidesComponent.isPortOnLeftSide( ProvidesPort );
			bool usesOnLeft = UsesComponent.isPortOnLeftSide( UsesPort );

			verticies.Add( new Vertex( providesPoint ) );

			if ( providesOnLeft )
			{
				verticies.Add( new Vertex( new Point( providesPoint.X - 30, providesPoint.Y ) ) );
				verticies.Add( new Vertex( new Point( providesPoint.X - 30, ProvidesComponent.Bottom + 20 ) ) );
			}
			else
			{
				verticies.Add( new Vertex( new Point( providesPoint.X + 50, providesPoint.Y ) ) );
				verticies.Add( new Vertex( new Point( providesPoint.X + 50, ProvidesComponent.Top - 30 ) ) );
			}

			if ( providesOnLeft && !usesOnLeft )
			{
				verticies.Add( new Vertex( new Point( usesPoint.X + 50, ProvidesComponent.Bottom + 20 ) ) );
			}
			else if ( !providesOnLeft && usesOnLeft )
			{
				verticies.Add( new Vertex( new Point( usesPoint.X - 30, ProvidesComponent.Top - 30 ) ) );
			}
			else if ( providesOnLeft && usesOnLeft )
			{
				verticies.Add( new Vertex( new Point( usesPoint.X - 30, ProvidesComponent.Bottom + 20 ) ) );
			}
			else if ( !providesOnLeft && !usesOnLeft )
			{
				verticies.Add( new Vertex( new Point( usesPoint.X + 50, ProvidesComponent.Top - 30 ) ) );
			}

			if ( usesOnLeft )
			{
				verticies.Add( new Vertex( new Point( usesPoint.X - 30, usesPoint.Y ) ) );
			}
			else
			{
				verticies.Add( new Vertex( new Point( usesPoint.X + 50, usesPoint.Y ) ) );
			}

			verticies.Add( new Vertex( usesPoint ) );
		}

		/// <summary>
		/// Draws the relationship.
		/// </summary>
		/// <param name="clientRect">The area in which to draw the relationship.</param>
		/// <param name="g">The graphics context to use.</param>
		public void drawRelationship( Rectangle clientRect, Graphics g )
		{
			Point[] pts = new Point[verticies.Count];
			for ( int i = 0; i < verticies.Count; i += 1 )
			{
				pts[i] = ( (Vertex)verticies[i] ).Point;
			}

			Pen drawingPen = normalPen;
			if ( selected )
			{
				drawingPen = selectedPen;
			}

			g.DrawLines( drawingPen, pts );
		}

		/// <summary>
		/// Called when the relationship should redraw itself.
		/// </summary>
		public void update()
		{
			Point providesPoint = ProvidesComponent.getPortLocation( ProvidesPort );
			Point usesPoint = UsesComponent.getPortLocation( UsesPort );

			lastProvidesPoint = providesPoint;
			lastUsesPoint = usesPoint;

			bool providesOnLeft = ProvidesComponent.isPortOnLeftSide( ProvidesPort );
			bool usesOnLeft = UsesComponent.isPortOnLeftSide( UsesPort );

			int usesDist = lastUsesDist;
			int provDist = lastProvDist;
			int compDist = lastCompDist;

			if ( dragId == DRAG_USES_DIST )
			{
				usesDist = dragValue;
				lastUsesDist = dragValue;
			}
			else if ( dragId == DRAG_PROVIDES_DIST )
			{
				provDist = dragValue;
				lastProvDist = dragValue;
			}
			else if ( dragId == DRAG_COMPONENT_DIST )
			{
				compDist = dragValue;
				lastCompDist = dragValue;
			}

			if ( usesDist == Int32.MinValue )
			{
				if ( usesOnLeft )
				{
					usesDist = usesPoint.X - 30;
				}
				else
				{
					usesDist = usesPoint.X + 50;
				}
			}
			if ( provDist == Int32.MinValue )
			{
				if ( providesOnLeft )
				{
					provDist = providesPoint.X - 30;
				}
				else
				{
					provDist = providesPoint.X + 50;
				}
			}
			if ( compDist == Int32.MinValue )
			{
				if ( ( providesOnLeft && !usesOnLeft ) || ( providesOnLeft && usesOnLeft ) )
				{
					compDist = ProvidesComponent.Bottom + 20;
				}
				else if ( ( !providesOnLeft && usesOnLeft ) || ( !providesOnLeft && !usesOnLeft ) )
				{
					compDist = ProvidesComponent.Top - 30;
				}
			}

			Region region = new Region( new Rectangle( 0, 0, 0, 0 ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[0], (Vertex)verticies[1] ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[1], (Vertex)verticies[2] ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[2], (Vertex)verticies[3] ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[3], (Vertex)verticies[4] ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[4], (Vertex)verticies[5] ) );
			
			( (Vertex)verticies[0] ).Point = providesPoint;
			( (Vertex)verticies[1] ).Point = new Point( provDist, providesPoint.Y );
			( (Vertex)verticies[2] ).Point = new Point( provDist, compDist );
			( (Vertex)verticies[3] ).Point = new Point( usesDist, compDist );
			( (Vertex)verticies[4] ).Point = new Point( usesDist, usesPoint.Y );
			( (Vertex)verticies[5] ).Point = usesPoint;
			region.Union( calculateRectangleFromList( (Vertex)verticies[0], (Vertex)verticies[1] ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[1], (Vertex)verticies[2] ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[2], (Vertex)verticies[3] ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[3], (Vertex)verticies[4] ) );
			region.Union( calculateRectangleFromList( (Vertex)verticies[4], (Vertex)verticies[5] ) );
			JVMSMain.JvmsForm.CurrentView.Invalidate( region );
		}

		/// <summary>
		/// Returns the properly scaled rectangle with the given upper-left and lower-right corners.
		/// </summary>
		/// <param name="first">The upper-left hand corner of the rectangle.</param>
		/// <param name="second">The lower-right hand corner of the rectangle.</param>
		/// <returns>A rectangle.</returns>
		private Rectangle calculateRectangleFromList( Vertex first, Vertex second )
		{
			int topLeftX = Math.Min( first.Point.X, second.Point.X ) - 5;
			int topLeftY = Math.Min( first.Point.Y, second.Point.Y ) - 5;
			int bottomRightX = Math.Max( first.Point.X, second.Point.X ) + 5;
			int bottomRightY = Math.Max( first.Point.Y, second.Point.Y ) + 5;

			Rectangle rect = 
				new Rectangle( topLeftX, topLeftY, bottomRightX - topLeftX, bottomRightY - topLeftY );
			return scaleRectangle( rect, JVMSMain.JvmsForm.CurrentView.ZoomFactor );
		}

		/// <summary>
		/// Scales the given rectangle by the given factor.
		/// </summary>
		/// <param name="rect">The rectangle.</param>
		/// <param name="zoomFactor">The scale factor.</param>
		/// <returns>A rectangle.</returns>
		private Rectangle scaleRectangle( Rectangle rect, float zoomFactor )
		{			
			Rectangle retval = new Rectangle(
				(int)(rect.X * zoomFactor) - 2,
				(int)(rect.Y * zoomFactor) - 2,
				(int)(rect.Width * zoomFactor) + 10,
				(int)(rect.Height * zoomFactor) + 10 );
			return retval;
		}

		/// <summary>
		/// Returns true if the given point is on a draggable point in the relationship.
		/// </summary>
		/// <param name="p">The desired point.</param>
		/// <returns>True if the point is draggable, false otherwise.</returns>
		public bool isDragging( Point p )
		{
			bool retVal = false;
			bool providesOnLeft = ProvidesComponent.isPortOnLeftSide( ProvidesPort );
			bool usesOnLeft = UsesComponent.isPortOnLeftSide( UsesPort );
			Point providesPoint = ProvidesComponent.getPortLocation( ProvidesPort );
			Point usesPoint = UsesComponent.getPortLocation( UsesPort );

			Point p1 = ( (Vertex)verticies[1] ).Point;
			Point p2 = ( (Vertex)verticies[2] ).Point;
			Point p3 = ( (Vertex)verticies[3] ).Point;
			Point p4 = ( (Vertex)verticies[4] ).Point;

			if ( isPointOnLine( p, p1, p2 ) )
			{
				dragId = DRAG_PROVIDES_DIST;
				retVal = true;
				if ( dragValue == -1 )
				{
					if ( providesOnLeft )
					{
						dragValue = providesPoint.X - 30;
					}
					else if ( !providesOnLeft )
					{
						dragValue = providesPoint.X + 50;
					}
				}
				else
				{
					dragValue = lastProvDist;
				}
			}
			else if ( isPointOnLine( p, p2, p3 ) )
			{
				dragId = DRAG_COMPONENT_DIST;
				retVal = true;
				if ( dragValue == -1 )
				{
					if ( ( providesOnLeft && usesOnLeft ) || ( !providesOnLeft && !usesOnLeft ) )
					{
						dragValue = ProvidesComponent.Bottom + 40;
					}
					else if ( ( providesOnLeft && !usesOnLeft ) || ( !providesOnLeft && usesOnLeft ) )
					{
						dragValue = ProvidesComponent.Top + 30;
					}
				}
				else
				{
					dragValue = lastCompDist;
				}
			}
			else if ( isPointOnLine( p, p3, p4 ) )
			{
				dragId = DRAG_USES_DIST;
				retVal = true;
				if ( dragValue == -1 )
				{
					if ( usesOnLeft )
					{
						dragValue = usesPoint.X - 30;
					}
					else if ( !usesOnLeft )
					{
						dragValue = usesPoint.X + 50;
					}
				}
				else
				{
					dragValue = lastUsesDist;
				}
			}

			return retVal;
		}

		/// <summary>
		/// Notifies of the relationship being dragged.
		/// </summary>
		/// <param name="p">The new point dragged to.</param>
		public void dragNotify( Point p )
		{
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			p.X = (int)(p.X / zoomFactor);
			p.Y = (int)(p.Y / zoomFactor);
			if ( dragId == DRAG_MID_X || dragId == DRAG_PROVIDES_DIST || dragId == DRAG_USES_DIST )
			{
				dragValue = p.X;
			}
			else if ( dragId == DRAG_COMPONENT_DIST )
			{
				dragValue = p.Y;
			}
		}

		/// <summary>
		/// Notifies of the relationship being scrolled.
		/// </summary>
		/// <param name="direction">The direction.</param>
		/// <param name="amount">The amount.</param>
		public void scrollNotify( Orientation direction, int amount )
		{
			if ( direction == Orientation.Vertical )
			{
				if ( lastCompDist != Int32.MinValue )
				{
					lastCompDist += amount;
				}
				dragId = DRAG_NONE;
			}
			else if ( direction == Orientation.Horizontal )
			{
				if ( lastProvDist != Int32.MinValue )
				{
					lastProvDist += amount;
				}

				if ( lastUsesDist != Int32.MinValue )
				{
					lastUsesDist += amount;
				}
				dragId = DRAG_NONE;
			}
		}

		/// <summary>
		/// From IRelationship.  Determines if given point is on the line (roughly).
		/// </summary>
		/// <param name="p">The desired point.</param>
		/// <returns>True if point is on line, false otherwise.</returns>
		public bool isIntersection( Point p )
		{
			bool retVal = false;

			for ( int i = 1; i < verticies.Count; i += 1 )
			{
				Point p1 = ( (Vertex)verticies[i - 1] ).Point;
				Point p2 = ( (Vertex)verticies[i] ).Point;

				if ( isPointOnLine( p, p1, p2 ) )
				{
					retVal = true;
					break;
				}
			}

			return retVal;
		}

		/// <summary>
		/// This will get the connectInterface that this relationship represents.
		/// </summary>
		/// <returns>The connection for this relationship.</returns>
		public override ConnectInterface getConnectInterface()
		{
			ComponentInstantiationPortConnection usesPortConnection = 
				new ComponentInstantiationUsesPortConnection();
			ComponentInstantiationPortConnection providesPortConnection = null;
			if( ProvidesComponent.Component.Descriptor.Features.ProvidesPorts.Contains( ProvidesPort ) )
			{
				providesPortConnection = new ComponentInstantiationProvidesPortConnection();
			}
			else if( ProvidesComponent.Component.Descriptor.Features.SupportInterfaces.Contains( ProvidesPort.RepositoryID ) )
			{
				providesPortConnection = new ComponentInstantiationSupportInterfacePortConnection();
			}
			else
			{
				throw new Exception( 
					"Error creating connect interface for " + ProvidesComponent.Component + 
					" because of unknown port " + ProvidesPort );
			}
			connectInterface.UsesPort = usesPortConnection;
			connectInterface.ProvidesPort = providesPortConnection;

			usesPortConnection.Port = UsesPort;
			providesPortConnection.Port = ProvidesPort;

			//now deal with the case where it is either a SAD or
			//a DCD instance.
			if( UsesComponent.getWrappedObject() is ApplicationComponentInstantiation )
			{
				usesPortConnection.Instance =
					(ApplicationComponentInstantiation)UsesComponent.getWrappedObject();
				providesPortConnection.Instance =
					(ApplicationComponentInstantiation)ProvidesComponent.getWrappedObject();
			}
			else if( UsesComponent.getWrappedObject() is ComponentPlacement )
			{
				usesPortConnection.Instance =
					((ComponentPlacement)UsesComponent.getWrappedObject()).Instantiation;
				providesPortConnection.Instance =
					((ComponentPlacement)ProvidesComponent.getWrappedObject()).Instantiation;
			}
			else
			{
				throw new Exception( "Unkown instance ref relationship" );
			}
			return connectInterface;
		}

		/// <summary>
		/// Returns true if the given point is on the line segment given by two endpoint.
		/// </summary>
		/// <param name="p">The test point.</param>
		/// <param name="p1">One endpoint of the segment.</param>
		/// <param name="p2">The other endpoint of the segment.</param>
		/// <returns>True if the point is on the line, false otherwise.</returns>
		private bool isPointOnLine( Point p, Point p1, Point p2 )
		{
			float zoomFactor = JVMSMain.JvmsForm.CurrentView.ZoomFactor;
			p1.X = (int)(p1.X * zoomFactor);
			p1.Y = (int)(p1.Y * zoomFactor);
			p2.X = (int)(p2.X * zoomFactor);
			p2.Y = (int)(p2.Y * zoomFactor);
			bool retVal = false;
			double segmentLength = Math.Sqrt( Math.Pow( p2.X - p1.X, 2 ) + Math.Pow( p2.Y - p1.Y, 2 ) );
			double len1 = Math.Sqrt( Math.Pow( p.X - p1.X, 2 ) + Math.Pow( p.Y - p1.Y, 2 ) );
			double len2 = Math.Sqrt( Math.Pow( p.X - p2.X, 2 ) + Math.Pow( p.Y - p2.Y, 2 ) );
			double diff = ( Math.Round( len1 ) + Math.Round( len2 ) ) - Math.Round( segmentLength );

			if ( ( diff >= -1 ) && ( diff <= 1 ) )
			{
				retVal = true;
			}

			return retVal;
		}

		/// <summary>
		/// This will perform the operation of deleting this relationship and
		/// removing all references to other components.
		/// </summary>
		public void delete()
		{
			//nothing because the connect interface is generated at XML gen time.
		}

		public override String ToString()
		{
			return UsesPort.UsesName + " - " + ProvidesPort.UsesName;
		}
	}
}
