using System;
using System.Collections;

using jvms.datamodel.dcd;
using jvms.datamodel.sad;

using jvms.presentation;
using jvms.presentation.drawablecomponent;
using jvms.project;
using jvms.system;

using jvms.ruleframework;

namespace jvms.validation
{
	/// <summary>
	/// This class will handle loading rules and executing them on an object.
	/// </summary>
	public class RuleEngine
	{	
		private static RuleEngine instance = new RuleEngine();

		private IList rules = null;

		private RuleEngine()
		{
			RuleManager manager = new RuleManager();
			rules = manager.getRules();
		}

		/// <summary>
		/// Returns the single instance of the rule engine.
		/// </summary>
		public static RuleEngine Instance
		{
			get{ return instance; }
		}


		/// <summary>
		/// This is the list of rules in the system.
		/// </summary>
		public IList Rules
		{
			get
			{
				return rules;
			}
		}

		/// <summary>
		/// This will run all the rules on a component.
		/// </summary>
		/// <param name="view">The view that the drawable component is on.</param>
		/// <param name="component"></param>
		/// <returns>true if there were validation errors.</returns>
		public bool validate( View view, DrawableComponent component )
		{					
			bool retval = false;
			if( component is ApplicationDrawableComponent ||
				component is DeviceConfigurationDrawableComponent )
			{
				//in this case we need to validate the entire project
				retval = validateProject();
			}
			else
			{
				IList errors = new ArrayList();
				runRulesOnObject( view, component, errors, true );
				showErrors( errors );
				retval = errors.Count > 0;
			}
			return retval;
		}

		private void showErrors( IList errors )
		{
			TaskList taskList = JVMSMain.JvmsForm.TaskListControl;
			taskList.clearTasks();

			if( errors.Count > 0 )
			{
				foreach ( RuleResult result in errors )
				{
					taskList.addTask( result );
				}

				JVMSMain.JvmsForm.setTaskListVisible( true );
			}
		}


		/// <summary>
		/// This will validate the entire project.
		/// </summary>
		/// <returns>true if there were validation errors.</returns>
		public bool validateProject()
		{
			IList errors = new ArrayList();
			ViewManager viewManager = ViewManager.Instance;
			validateView( viewManager.AppAssmView, errors );
			validateView( viewManager.AppCompView, errors );
			validateView( viewManager.PlatAssmView, errors );
			validateView( viewManager.PlatCompView, errors );
			showErrors( errors );
			return errors.Count > 0;
		}

		/// <summary>
		/// This will validate all of the drawable components on a view.
		/// </summary>
		/// <param name="view"></param>
		/// <param name="errors"></param>
		private void validateView( View view, IList errors )
		{
			foreach( DrawableComponent component in view.DrawableComponents )
			{
				runRulesOnObject( view, component, errors, false );
			}
		}

		/// <summary>
		/// This will run all of the applicable rules on the object.
		/// </summary>
		/// <param name="view"></param>
		/// <param name="component"></param>
		/// <param name="errors">A list of errors.</param>
		/// <param name="local">True if not validating SAD or DCD</param>
		private void runRulesOnObject( View view, DrawableComponent component, IList errors, bool local )
		{
			Object obj = component.getWrappedObject();
			//we must first set the relationships on the component
			if( obj is Application )
			{
				Application app = (Application)obj;
				RelationshipManager manager = RelationshipManager.Instance;				
				app.Connections = 
					manager.getConnectionsForInstantiations( app.ComponentPlacements );
			}
			else if( obj is DeviceConfiguration )
			{
				DeviceConfiguration dcd = (DeviceConfiguration)obj;
				RelationshipManager manager = RelationshipManager.Instance;
				IList instances = new ArrayList();
				foreach( ComponentPlacement placement in dcd.Partitioning )
				{
					instances.Add( placement.Instantiation );
				}				
				dcd.Connections = manager.getConnectionsForInstantiations( instances );
			}

			IList selectedCategories = (IList)JVMSProperties.Instance.getProperty( "selectedrulecategories", new ArrayList() );
			IList selectedRules = (IList)JVMSProperties.Instance.getProperty( "selectedrules", new ArrayList() );

			//now run every rule on the object
			foreach( Rule rule in rules )
			{
				if( rule.worksOnType( obj.GetType() ) && 
					(!local || rule.isLocal()) &&
					(selectedCategories.Contains( rule.Category ) ||
					 selectedRules.Contains( rule.Name ) ) )
				{
					rule.Project = ProjectManager.Instance.Project;
					IList ruleErrors = rule.validate( obj );
					foreach( String error in ruleErrors )
					{
						RuleResult result = new RuleResult();
						result.View = view;
						result.Component = component;
						result.ErrorMessage = error;
						errors.Add( result );
					}
				}
			}
		}
	}
}