using System;
using System.Collections;
using System.IO;
using System.Xml;
using System.Reflection;

using jvms.ruleframework;
using jvms.system;

namespace jvms.rules
{
	/// <summary>
	/// Checks required field on an application.
	/// </summary>
	public class GlobalDomainRule : Rule
	{
		private XmlDocument config = null;
		private String assemblyLocation = null;
		
		/// <summary>
		/// Creates a new ApplicationDomainRule.
		/// </summary>
		public GlobalDomainRule()
		{
			Name = "Required Fields";
			Description = "Ensures that all required fields on components are set.";
			Category = "General";

			Assembly jvmsAssembly = Assembly.GetExecutingAssembly();
			assemblyLocation = jvmsAssembly.Location.Substring( 0, jvmsAssembly.Location.LastIndexOf( '\\' ) + 1 );
			JVMSProperties props = JVMSProperties.Instance;
			string configFile = assemblyLocation + props.getProperty( "AttributeService.config", "AttributeService.xml" );
			config = new XmlDocument();
			config.Load( configFile );
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return true;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			String id = obj.GetType().Name;
			XmlElement element = config.GetElementById(id);
			XmlNode n = config.FirstChild;
			foreach ( XmlNode n2 in n.ChildNodes )
			{
				if ( n2.NodeType == XmlNodeType.Comment )
				{
					continue;
				}
				if ( n2.Attributes["id"].Value.Equals(id) )
				{
					
					foreach ( XmlNode node in n2.ChildNodes )
					{
						XmlAttributeCollection attributes;
						bool required;
						string property;
						string name;
						XmlAttribute xmlAttr;
						if ( node.NodeType == XmlNodeType.Comment )
						{
							continue;
						}
						foreach ( XmlNode attribute in node.ChildNodes )
						{
							if ( attribute.NodeType == XmlNodeType.Comment )
							{
								continue;
							}
							attributes = attribute.Attributes;
							if ( attributes == null )
							{
								continue;
							}

							xmlAttr = attributes["name"];
							name = (xmlAttr == null) ? null : xmlAttr.Value;

							xmlAttr = attributes["required"];
							required = (xmlAttr == null) ? false : Boolean.Parse(xmlAttr.Value);
							
							xmlAttr = attributes["property"];
							property = (xmlAttr == null) ? null : xmlAttr.Value;
							
							Object obj2 = null;
							PropertyInfo pInfo = getPropertyInfo( obj, property, out obj2 );
							Type t = pInfo.PropertyType;
							
							if ( required )
							{
								object value = pInfo.GetValue( obj2, null );
								if ( value == null || "".Equals(value) )
								{
									errors.Add( "Required field '" + name + "' is not set on component '" + obj + "'." );
								}
								else if ( value is IList )
								{
									if ( ((IList)value).Count == 0 )
									{
										errors.Add( "Required field '" + name + "' has no members on component '" + obj + "'." );
									}
								}
							}
						}
					}
					break;
				}
			}

			return errors;
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return true;
		}

		private PropertyInfo getPropertyInfo( Object obj, String property, out Object ob2 )
		{
			PropertyInfo pInfo = null;
			int index = property.IndexOf( "." );
			String tempProp;
			while ( index > 0 )
			{
				tempProp = property.Substring( 0, index );
				property = property.Substring( index + 1, property.Length - index - 1 );
				pInfo = obj.GetType().GetProperty( tempProp );
				obj = pInfo.GetValue( obj, null );
				index = property.IndexOf( "." );
			}
			pInfo = obj.GetType().GetProperty(property);
			ob2 = obj;
			return pInfo;
		}
	}
}
