using System;
using System.Collections;

using jvms.ruleframework;
using jvms.datamodel.spd;

namespace jvms.rules
{
	/// <summary>
	/// Checks required fields on a component's implementations.
	/// </summary>
	public class ImplementationSyntacticRule : Rule
	{
		/// <summary>
		/// Creates a new ImplementationSyntaxRule.
		/// </summary>
		public ImplementationSyntacticRule()
		{
			Name = "Implementation Syntax";
			Description = "Checks required fields on component implementations.";
			Category = "Component";
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return true;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			ArrayList impls = (ArrayList)( (NonSCAComponent)obj ).Implementations;
			string result = null;
			foreach ( Implementation impl in impls )
			{
				result = checkStackSize( impl );
				if ( result != null )
				{
					errors.Add( result );
				}

				result = checkPriority( impl );
				if ( result != null )
				{
					errors.Add( result );
				}

				if ( impl.Dependencies.Count == 0 && impl.OSList.Count == 0 && impl.Processors.Count == 0 )
				{
					errors.Add( "In Implementations, at least one of 'os', 'processor', 'dependency' is required (" + impl.ToString() + ").");
				}
			}
			return errors;
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return type.Equals( typeof( NonSCAComponent ) ) || type.Equals( typeof( SCAComponent ) );
		}

		/// <summary>
		/// StackSize should be of type unsigned long
		/// </summary>
		/// <param name="impl"></param>
		/// <returns>error or null</returns>
		private string checkStackSize( Implementation impl )
		{
			string result = null;
			try
			{
				if ( !impl.Code.StackSize.Equals("") )
				{
					UInt64.Parse( impl.Code.StackSize );
				}
			}
			catch ( Exception )
			{
				result = "In Implementations, 'Stack Size' must be unsigned long integer (" + impl.ToString() + ").";
			}
			return result;
		}

		/// <summary>
		/// Priority must be an unsigned long
		/// </summary>
		/// <param name="impl"></param>
		/// <returns>error or null</returns>
		private string checkPriority( Implementation impl )
		{
			string result = null;
			try
			{
				if ( !impl.Code.Priority.Equals("") )
				{
					UInt64.Parse( impl.Code.Priority );
				}
			}
			catch( Exception )
			{
				result = "In Implementations, 'Priority' must be unsigned long integer (" + impl.ToString() + ").";
			}
			return result;
		}
	}
}
