using System;
using System.Collections;

using jvms.datamodel.common;
using jvms.datamodel.dcd;
using jvms.datamodel.sad;

using jvms.ruleframework;

namespace Rules.rules
{
	/// <summary>
	/// Checks required fields on relationships.
	/// </summary>
	public class RelationshipMetaRule : Rule
	{
		/// <summary>
		/// Creates a new RelationshipMetaRule.
		/// </summary>
		public RelationshipMetaRule()
		{
			Name = "Relationship Syntax";
			Description = "Checks required fields on relationships.";
			Category = "Relationships";
		}

		/// <summary>
		/// Returns true if the rule is local to a component, false if it has project wide scope.
		/// </summary>
		/// <returns>True if rule is local, false otherwise.</returns>
		public override bool isLocal()
		{
			return false;
		}

		/// <summary>
		/// Performs validation.
		/// </summary>
		/// <param name="obj">The data model object to validate.</param>
		/// <returns>A list of error messages.</returns>
		public override IList validate( Object obj )
		{
			IList errors = new ArrayList();
			if( obj is Application )
			{
				Application app = (Application)obj;				
				validateRelationships( app.Connections, errors );
			}
			else if( obj is DeviceConfiguration )
			{
				DeviceConfiguration dcd = (DeviceConfiguration)obj;
				validateRelationships( dcd.Connections, errors );
			}
			return errors;
		}

		/// <summary>
		/// Validates the given relationships.
		/// </summary>
		/// <param name="relationships">The list of relationships.</param>
		/// <param name="errors">The error list.</param>
		private void validateRelationships( IList relationships, IList errors )
		{
			foreach( ConnectInterface relationship in relationships )
			{
				if( relationship.FindBy != null &&
					relationship.ProvidesPort is ComponentInstantiationPortConnection )
				{
					validateFindBy( 
						(ComponentInstantiationPortConnection)relationship.ProvidesPort,
						relationship.FindBy, errors );
				}
				else if( relationship.FindBy != null &&
					relationship.UsesPort is ComponentInstantiationPortConnection )
				{
					validateFindBy( 
						(ComponentInstantiationPortConnection)relationship.UsesPort,
						relationship.FindBy, errors );
				}
				else if( relationship.UsesPort is DeviceUsedByThisComponentPortConnection )
				{
					validateDeviceUsedBy( 
						relationship.ProvidesPort,
						(DeviceUsedByThisComponentPortConnection)relationship.UsesPort, errors );
				}
				else if( relationship.ProvidesPort is DeviceUsedByThisComponentPortConnection )
				{
					validateDeviceUsedBy( 
						relationship.UsesPort,
						(DeviceUsedByThisComponentPortConnection)relationship.ProvidesPort, errors );
				}
			}
		}

		/// <summary>
		/// Validates a device used by relationship.
		/// </summary>
		/// <param name="instancePort">The port the relationship is connected to.</param>
		/// <param name="connection">The data model object.</param>
		/// <param name="errors">The error list.</param>
		private void validateDeviceUsedBy(
			PortConnection port,
			DeviceUsedByThisComponentPortConnection connection,
			IList errors )
		{
			String errorPostfix = "";
			if( port is ComponentInstantiationPortConnection )
			{
				ComponentInstantiationPortConnection instancePort = 
					(ComponentInstantiationPortConnection)port;
				errorPostfix = 
					instancePort.Port.UsesName + " on " + instancePort.Instance.UsageName + ".";
			}
			else if( port is ComponentSupportedInterface )
			{
				ComponentSupportedInterface supported = 
					(ComponentSupportedInterface)port;
				errorPostfix = supported.Instance.UsageName + ".";
			}

			if ( connection.Port == null )
			{
				errors.Add( "No port specified for the device used by that connects to " + errorPostfix );
			}
			if ( connection.UsesDevice == null )
			{
				errors.Add( "No device specified for the device used by that connects to " + errorPostfix );
			}
		}

		/// <summary>
		/// Validates a find by relationship.
		/// </summary>
		/// <param name="instancePort">The port the relationship is connected to.</param>
		/// <param name="findBy">The data model object.</param>
		/// <param name="errors">The error list.</param>
		private void validateFindBy(
			ComponentInstantiationPortConnection instancePort, 
			FindBy findBy, 
			IList errors )
		{
			String errorPostfix = 
				instancePort.Port.UsesName + " on " + instancePort.Instance.UsageName + ".";		
			if ( findBy == null )
			{
				errors.Add( "No findby field specifed for the find by that connects to " + errorPostfix );
			}
			else
			{
				if ( findBy is NamingService )
				{
					NamingService naming = (NamingService)findBy;
					if ( naming.Name.Equals( "" ) )
					{
						errors.Add( "No naming service specified for the find by that connects to " + errorPostfix );
					}
				}
				else if ( findBy is DomainFinder )
				{
					DomainFinder domain = (DomainFinder)findBy;
					if ( domain.Type == null || domain.Type.Equals( "" ) )
					{
						errors.Add( "No domain finder type specified for the find by that connects to " + errorPostfix );
					}
				}
				else
				{
					throw new Exception( "Unknown find by type: '" + findBy.GetType().FullName + "'" );
				}
			}
		}

		/// <summary>
		/// Returns true if the rule can be run on the given type, false otherwise.
		/// </summary>
		/// <param name="type">The type in question.</param>
		/// <returns>True if the rule can be run on the type, false otherwise.</returns>
		public override bool worksOnType( Type type )
		{
			return type.Equals( typeof( Application ) ) || type.Equals( typeof( DeviceConfiguration ) );
		}
	}
}
